      PROGRAM RGRUN
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE TEST STATISTIC ASSOCIATED WITH GENERALIZED RUNS PERMUTATION
C         PROCEDURES (GRUN) IS CALCULATED IN THIS FORTRAN-77 PROGRAM.
C         THIS TEST IS DESIGNED TO ANALYZE GENERALIZED RUNS DATA
C         CONSISTING OF POSITIONS WITHIN G A PRIORI DISJOINT GROUPS
C         OF OBJECTS AND A POSSIBLY EMPTY EXCESS GROUP IN A FINITE
C         POPULATION OF N OBJECTS.  THE PRESENT MAXIMUM VALUES OF
C         G AND N IN THIS PROGRAM ARE 40 AND 150, RESPECTIVELY.
C
C         PROGRAM MODIFIED 5/22/2000
C
C       FIRST LINE OF DATA FILE:  PROBLEM HEADING.
C
C       SECOND LINE:  N,CFORM,G
C       THIRD LINE:  n(1),...,n(G)
C       EITHER OF THE FOUR FOLLOWING WEIGHTING FORMS MAY BE CHOSEN.
C            CFORM = 1:  C(I) = n(I)/sum(n(I))
C            CFORM = 2:  C(I) = (n(I)-1)/sum(n(I)-1)
C            CFORM = 3:  C(I) = 1/sum(1)
C            CFORM = 4:  C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))
C       CFORM IS PASSED AS THE SECOND ENTRY ON THE SECOND LINE IN
C       THE DATA FILE.  WEIGHTS FOR EACH GROUP ARE CALCULATED IN
C       SUBROUTINE DIST AND STORED IN ARRAY C(I).
C
C       (2N-I)(I-1)/2+J-I+3RD LINE:  DEL(I,J) WHERE 0 < I < J < N+1.
C       THUS THE FOURTH THROUGH THE N(N-1)/2+3RD LINES ARE DEL(1,2)
C       THROUGH DEL(N-1,N), RESPECTIVELY.  DEL(I,J) IS 1 IF THE Ith
C       AND Jth DISTINCT OBJECTS ARE LINKED AND IS 0 OTHERWISE.
C
      CHARACTER  HEAD*80,FIN*40,FOUT*40
      DIMENSION  U(17),C(40),ITHGRP(40),XI1(40),XI(40),
     +           DEL(150,150),TD(150,150)
      INTEGER  CFORM
C
      ZERO = 0.0D0
      ONE = 1.0D0
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA'
      READ(*,1000) FIN
 1000 FORMAT(A)
      OPEN(3,FILE=FIN)
      READ(3,1000) HEAD
      READ(3,*) NCASES,CFORM,NGRPS
      READ(3,*) (ITHGRP(I),I = 1,NGRPS)
      DO 1010 I = 1,NCASES
         DEL(I,I) = ZERO
         TD(I,I) = ZERO
 1010 CONTINUE
      DO 1060 I = 1,NCASES-1
         DO 1050 J = I+1,NCASES
            READ(3,*,END=1070) DEL(I,J)
            DEL(J,I) = DEL(I,J)
            TD(I,J) = DEL(I,J)
            TD(J,I) = DEL(I,J)
 1050    CONTINUE
 1060 CONTINUE
      CLOSE(3)
      GO TO 1080
 1070 WRITE(*,1075) 'ERROR:  END-OF FILE ENCOUNTERED'
 1075 FORMAT(/,T3,A)
      STOP
 1080 WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,1000) FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 1104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 1102 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
 1102    CONTINUE
         U(II) = S
 1104 CONTINUE
      KZ = 0
      CALL DIST(CFORM,NGRPS,KZ,ITHGRP,TD,C,XI,DBAR)
      KZ = 1
      DBAR = DBAR*(ONE-1.0D-12)
      MP = 0
      DO 1150 IS = 1,MS
         CALL SAMPWOR(NCASES,NCASES,U,DEL,TD)
         CALL DIST(CFORM,NGRPS,KZ,ITHGRP,TD,C,XI1,DX)
         IF(DX.GT.DBAR) MP = MP+1
 1150 CONTINUE
      PROB = ONE*MP/MS
      OPEN(6,FILE=FOUT,STATUS='NEW')
      WRITE(6,1000) HEAD
      HEAD = 'GENERALIZED RUNS PERMUTATION PROCEDURES (RGRUN)'
      WRITE(6,1200) HEAD
 1200 FORMAT(/,T23,A,/)
      WRITE(6,1250) 'INPUT HAS ', NCASES,' OBSERVATIONS'
 1250 FORMAT(T10,A,I3,A)
      WRITE(6,1300) 'IN: ', NGRPS, ' GROUPS OF SIZES ',
     +              (ITHGRP(I),I = 1,NGRPS)
 1300 FORMAT(T10,A,I5,A,10I5)
      GO TO (1410,1420,1430,1440) CFORM
 1410 WRITE(6,1450) 'C(I) = n(I)/sum(n(I))'
      GO TO 1500
 1420 WRITE(6,1450) 'C(I) = (n(I)-1)/sum(n(I)-1)'
      GO TO 1500
 1430 WRITE(6,1450) 'C(I) = 1/sum(1)'
      GO TO 1500
 1440 WRITE(6,1450) 'C(I) = n(I)*(n(I)-1)/sum(n(I)*(n(I)-1))'
 1450 FORMAT(T10,A)
 1500 WRITE(6,1000) ' '
      DO 1700 I = 1,NGRPS
         WRITE(6,1600) 'GROUP NUMBER ', I, ' OF SIZE ', ITHGRP(I),
     +                 ' HAS AN AVERAGE DISTANCE =', XI(I)
 1600    FORMAT(T10,A,I2,A,I3,A,G16.8)
 1700 CONTINUE
      WRITE(6,1000) ' '
      WRITE(6,1750) 'RANDOM NUMBER GENERATOR SEED =',ISEED
 1750 FORMAT(T10,A,I9)
      WRITE(6,1750) 'NUMBER OF SIMULATIONS =',MS
      WRITE(6,1800) 'THE OBSERVED DELTA IS =', DBAR
 1800 FORMAT(T10,A,G16.8)
      WRITE(6,1850) 'P-VALUE OF A LARGER OR EQUAL DELTA =', PROB
 1850 FORMAT(/,T10,A,G16.8)
      WRITE(6,2000) '* * *   R G R U N   E N D   * * *'
 2000 FORMAT(/,T22,A)
      END
C
C
      SUBROUTINE DIST(CFORM,NGRPS,KZ,ITHGRP,TD,C,XI,DX)
      IMPLICIT REAL*8(A-H,O-Z)
C
C         SUBROUTINE DIST CALCULATES THE DISTANCES BETWEEN THE DATA
C         POINTS, PLUS THE PARAMETERS WHICH ARE DIRECTLY DEPENDENT
C         ON THE DISTANCES.
C
      DIMENSION  C(40),ITHGRP(40),XI(40),TD(150,150)
      INTEGER  CFORM
C
      ZERO = 0.0D0
      ONE = 1.0D0
      DX = ZERO
      IF(KZ.EQ.1) GO TO 1490
      GO TO (1410,1430,1450,1470) CFORM
 1410 FK = ZERO
      DO 1415 I = 1,NGRPS
         FK = FK+ITHGRP(I)
 1415 CONTINUE
      DO 1420 I = 1,NGRPS
         C(I) = ONE*ITHGRP(I)/FK
 1420 CONTINUE
      GO TO 1490
 1430 FK = ZERO
      DO 1435 I = 1,NGRPS
         FK = FK+ITHGRP(I)
 1435 CONTINUE
      DO 1440 I = 1,NGRPS
         C(I) = ONE*(ITHGRP(I)-1)/(FK-NGRPS)
 1440 CONTINUE
      GO TO 1490
 1450 DO 1460 I = 1,NGRPS
         C(I) = ONE/NGRPS
 1460 CONTINUE
      GO TO 1490
 1470 Y = ZERO
      DO 1475 I = 1,NGRPS
         Y = Y+ONE*ITHGRP(I)*(ITHGRP(I)-1)
 1475 CONTINUE
      DO 1480 I = 1,NGRPS
         C(I) = ONE*ITHGRP(I)*(ITHGRP(I)-1)/Y
 1480 CONTINUE
 1490 DO 1500 I = 1,NGRPS
         XI(I) = ZERO
 1500 CONTINUE
      L1 = ZERO
      L2 = ZERO
      DO 1800 I = 1,NGRPS
         L1 = L2
         L2 = L2+ITHGRP(I)
         DO 1700 J = L1+1,L2-1
            DO 1600 K = J+1,L2
               XI(I) = XI(I)+TD(J,K)
 1600       CONTINUE
 1700    CONTINUE
 1800 CONTINUE
      DO 1900 I = 1,NGRPS
         XI(I) = XI(I)*2/(ITHGRP(I)*(ITHGRP(I)-1))
 1900 CONTINUE
      DO 2000 I = 1,NGRPS
         DX = DX+C(I)*XI(I)
 2000 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE SAMPWOR(M,N,U,DEL,TD)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),L(150),DEL(150,150),TD(150,150)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     L: REAL INPUT WHICH CONTAINS 1,...,M POPULATION VALUES
C     WITHOUT REPLACEMENT SAMPLE CONTAINED IN L(1),...,L(N)
C
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 10 I = 1,M
         L(I) = I
   10 CONTINUE
      DO 20 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         LBIN = L(J)
         L(J) = L(I)
         L(I) =  LBIN
   20 CONTINUE
      DO 40 I = 1,N
         DO 30 J = I,N
            TD(I,J) = DEL(L(I),L(J))
   30    CONTINUE
   40 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
