      PROGRAM TJTEST
C
C     PROGRAM TJTEST COMPUTES THE TERPSTRA-JONCKHEERE TEST FOR ORDERED
C     ALTERNATIVES WHERE M(1),...,M(G) ARE G SAMPLE SIZES AND WHERE
C     N = M(1) + ... + M(G).  THE NULL HYPOTHESIS IS:
C                         H0: H(1) = H(2) ... = H(G)
C     AND THE ALTERNATIVE HYPOTHESIS IS:
C                         H1: H(1) <= H(2) ... <= H(G)
C     WHERE H REPRESENTS THE G SAMPLE MEDIANS.  THE DATA IS ASSUMED TO
C     CONSIST OF CONTINUOUS UNIVARIATE OBSERVATIONS WITH NO TIES AMONG
C     ANY OF THE N VALUES.
C
C     PROGRAM WRITTEN ON:  4 FEBRUARY 1997
C
C     PROGRAM TJTEST INPUTS THE NECESSARY PARAMETERS AND THE RAW DATA.
C     IN ADDITION, PROGRAM TJTEST PRINTS OUT THE RESULTS OF THE ANALYSIS
C     AND STORES THE RESULTS ON A FILE.  THE PARAMETERS ARE:
C
C          (1) THE NAME OF THE INPUT FILE CONTAINING THE RAW DATA
C          (2) THE NAME OF THE OUTPUT FILE TO CONTAIN THE RESULTS
C          (3) THE DESCRIPTIVE HEADING FOR THE ANALYSIS
C          (4) THE NUMBER OF SAMPLES [G]
C          (5) THE NUMBER OF OBJECTS IN EACH SAMPLE [M(1),...,M(G)]
C
C     EXAMPLE 1:
C          CONSIDER A PROBLEM WITH G = 3 SAMPLES OF SIZES M(1) = 3,
C     M(2) = 4, AND M(3) = 5.  THE INPUT FILE CONTAINING THE DATA
C     IS LISTED HERE:
C                                 57
C                                 21
C                                 29
C                                 23
C                                 36
C                                 12
C                                 29
C                                 81
C                                 90
C                                 36
C                                 45
C                                 83
C
C     WHERE THE FIRST SAMPLE CONSISTS OF SCORES 57, 21 AND 29; THE SECOND
C     SAMPLE CONSISTS OF SCORES 23, 36, 12 AND 29; AND THE THIRD SAMPLE
C     CONSISTS OF SCORES 81, 90, 36, 45 AND 83.
C
C     TERPSTRA-JONCKHEERE EXACT PERMUTATION RESULTS
C     -------------------------------------------------------------------
C     HEADING: OUTPUT OF RESULTS FOR EXAMPLE 1 (EXACT SOLUTION)
C
C     INPUT HAS    12 OBSERVATIONS
C     THERE ARE     3 SAMPLES OF SIZES:   3   4   5
C
C     OBSERVED TERPSTRA-JONCKHEERE STATISTIC  =    36.000000
C     MEAN OF TERPSTRA-JONCKHEERE STATISTIC   =    23.500000
C     VARIANCE OF T-J STATISTIC               =    45.916667
C     TOTAL NUMBER OF PERMUTATIONS            =    27720
C     NUMBER OF STATISTICS => OBSERVED VALUE  =     1051
C     EXACT P-VALUE OF OBSERVED T-J STATISTIC =    .37914863E-01
C     -------------------------------------------------------------------
C
C     EXAMPLE 2:
C          CONSIDER A PROBLEM WITH G = 4 SAMPLES OF SIZES M(1) = 12,
C     M(2) = 9, M(3) = 8, AND M(4) = 6.  THE DATA ARE LISTED BELOW AND
C     ARE TAKEN FROM S. SIEGEL AND N. J. CASTELLAN, NONPARAMETRIC
C     STATISTICS FOR THE BEHAVIORAL SCIENCES (2ND EDITION), MCGRAW-HILL,
C     NEW YORK, 1988, PAGE 219.
C
C                              8.82
C                             11.27
C                             15.78
C                             17.39
C                             24.99
C                             39.05
C                             47.54
C                             48.85
C                             71.66
C                             72.77
C                             90.38
C                            103.13
C                             13.53
C                             28.42
C                             48.11
C                             48.64
C                             51.40
C                             59.91
C                             67.98
C                             79.13
C                            103.05
C                             19.23
C                             67.83
C                             73.68
C                             75.22
C                             77.71
C                             83.67
C                             86.83
C                             93.25
C                             73.51
C                             85.25
C                             85.82
C                             88.88
C                             90.33
C                            118.11
C
C     WHERE THE FIRST SAMPLE CONSISTS OF THE 12 SCORES FROM 8.82 TO 103.13,
C     THE SECOND SAMPLE CONSISTS OF THE 9 SCORES FROM 13.53 TO 103.05, ETC.
C
C     TERPSTRA-JONCKHEERE NORMAL APPROXIMATION RESULTS
C     -------------------------------------------------------------------
C     HEADING: OUTPUT OF RESULTS FOR EXAMPLE 2 (NORMAL APPROXIMATION)
C
C     INPUT HAS    35 OBSERVATIONS
C     THERE ARE     4 SAMPLES OF SIZES:  12   9   8   6
C
C     OBSERVED TERPSTRA-JONCKHEERE STATISTIC  =    337.00000
C     MEAN OF TERPSTRA-JONCKHEERE STATISTIC   =    225.00000
C     VARIANCE OF T-J STATISTIC               =    1140.0000
C     P-VALUE OF OBSERVED T-J STATISTIC       =    .47945857E-03
C     -------------------------------------------------------------------
C
C     THE CURRENT DIMENSIONS FOR PROGRAM TJTEST ARE SET FOR G = 10,
C     N = 1000, AND ANY M(I) <= 400.  TO CHANGE THE SIZE OF THE DIMENSIONS,
C     REPLACE THE VALUES IN THE 'DOUBLE PRECISION' AND THE 'INTEGER'
C     STATEMENTS BELOW.
C     PROGRAM TJTEST COMPUTES EXACT PROBABILITY VALUES FOR ANALYSES
C     WITH NO MORE THAN 300,000 PERMUTATIONS, AT WHICH POINT THE PROGRAM
C     USES THE NORMAL APPROXIMATION TO OBTAIN THE PROBABILITY VALUE.  TO
C     CHANGE THE CUTOFF POINT, CHANGE THE VALUE OF 'CUTOFF' (3.0D5) IN THE
C     'PARAMETER' STATEMENT BELOW.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION  M(11),Y(1000),X(400,11)
      CHARACTER  FIN*40,FOUT*40,HEAD*60
      PARAMETER  (CUTOFF = 3.0D5)
C
C  IF G > 10, N > 1000 OR AN M(I) > 400, REVISE THE ABOVE DIMENSION STATEMENT.
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,'(A)') FIN
      WRITE(*,*) 'ENTER: FILE NAME FOR OUTPUT RESULTS'
      READ(*,'(A)') FOUT
      WRITE(*,*) 'ENTER: PROBLEM HEADING (UP TO 60 CHARACTERS)'
      READ(*,'(A)') HEAD
      WRITE(*,*) 'ENTER: G, THE NUMBER OF SAMPLES'
      READ(*,*) NG
      WRITE(*,*) 'ENTER: [M(1),...,M(G)], THE NUMBER OF OBSERVATIONS ',
     &           'IN EACH SAMPLE'
      READ(*,*) (M(I),I=1,NG)
      M(NG+1) = 0
      IF(NG.LT.2) THEN
        WRITE(*,*) 'PROGRAM TERMINATED SINCE G IS LESS THAN 2'
        GO TO 300
      ENDIF
      N = 0
      L2 = 0
      L3 = 0
      DO 10 I = 1,NG
        N = N+M(I)
        L2 = L2+M(I)**2
        L3 = L3+M(I)**3
   10 CONTINUE
      AN = ONE*N
      AN2 = ONE*L2
      AN3 = ONE*L3
      EW = (AN**2-AN2)/4.0D0
      VW = (AN*AN*(AN*2+3)-AN3*2-AN2*3)/72.0D0
      OPEN(5,FILE=FIN,STATUS='OLD')
      READ(5,*,END=280) (Y(I),I = 1,N)
      CLOSE(5)
   17 K = 0
      DO 19 I = 1,NG
        DO 18 J = 1,M(I)
          K = K+1
          X(J,I) = Y(K)
   18   CONTINUE
   19 CONTINUE
C
C     CALCULATE OBSERVED TERPSTRA-JONCKHERE W STATISTIC
C
      W1 = ZERO
      DO 100 I = 1,NG-1
        DO 90 J = I+1,NG
          DO 80 K = 1,M(I)
            DO 70 L = 1,M(J)
              IF(X(K,I).LT.X(L,J)) W1 = W1+1
   70       CONTINUE
   80     CONTINUE
   90   CONTINUE
  100 CONTINUE
C
C     DETERMINE EXACT OR APPROXIMATE SOLUTIONS
C
      C1 = 0.5D0*DLOG(DATAN(ONE)*8)
      C2 = C1+(AN+0.5D0)*DLOG(AN+ONE)-(AN+ONE)
      C3 = ZERO
      DO 110 I = 1,NG
        C3 = C3+C1+(M(I)+0.5D0)*DLOG(M(I)+ONE)-(M(I)+ONE)
  110 CONTINUE
      CHECK = DLOG(CUTOFF)
C
C     SELECT SOLUTION BASED ON ALL POSSIBLE PERMUTATIONS
C
      W1C = W1-0.5D0
      Z = (W1C-EW)/DSQRT(VW)
      CALL PVALUE(Z,PVAL1)
      IF(C2-C3.GT.CHECK) GO TO 130
  120 W1 = W1*(ONE-1.0D-12)
C
C     OUTPUT EXACT RESULTS BASED ON ALL POSSIBLE PERMUTATIONS
C
      WRITE(*,125) 'COMPUTING EXACT SOLUTION -- PLEASE WAIT . . . '
  125 FORMAT(/,T2,A,/)
      CALL PERMUT(N,NG,M,KOUNT1,KOUNT2,W1)
      PVAL2 = ONE*KOUNT2/KOUNT1
  130 OPEN(6,FILE=FOUT,STATUS='NEW')
      IF(C2-C3.GT.CHECK) THEN
        WRITE(6,140) 'TERPSTRA-JONCKHEERE NORMAL APPROXIMATION RESULTS'
      ELSE
        WRITE(6,140) 'TERPSTRA-JONCKHEERE EXACT PERMUTATION RESULTS'
      END IF
  140 FORMAT(T3,A)
      WRITE(6,150) '--------------------------------------------------',
     &             '---------------'
  150 FORMAT(T3,A,A)
      WRITE(6,160) 'HEADING: ',HEAD
  160 FORMAT(T3,A,A,/)
      WRITE(6,170) 'INPUT HAS ',N,' OBSERVATIONS'
  170 FORMAT(T3,A,I5,A)
      WRITE(6,180) 'THERE ARE ',NG,' SAMPLES OF SIZES:',(M(I),I = 1,NG)
  180 FORMAT(T3,A,I5,A,10I4)
      WRITE(6,200) 'OBSERVED TERPSTRA-JONCKHEERE STATISTIC   = ',W1
  200 FORMAT(/,T3,A,G16.8)
      WRITE(6,250) 'MEAN OF TERPSTRA-JONCKHEERE STATISTIC    = ',EW
  250 FORMAT(T3,A,G16.8)
      WRITE(6,250) 'VARIANCE OF T-J STATISTIC                = ',VW
      WRITE(6,250) 'NORMAL P-VALUE OF OBSERVED T-J STATISTIC = ',PVAL1
      IF(C2-C3.GT.CHECK) GO TO 270
      WRITE(6,260) 'TOTAL NUMBER OF PERMUTATIONS             = ',KOUNT1
      WRITE(6,260) 'NUMBER OF STATISTICS => OBSERVED VALUE   = ',KOUNT2
      WRITE(6,250) 'EXACT P-VALUE OF OBSERVED T-J STATISTIC  = ',PVAL2
  260 FORMAT(T3,A,I8)
  270 WRITE(6,150) '--------------------------------------------------',
     &             '---------------'
      GO TO 300
  280 WRITE(*,290) FIN
  290 FORMAT(/,T2,'E R R O R:  END-OF-FILE ENCOUNTERED: ',A,/)
  300 END
C
C
      SUBROUTINE PERMUT(N,G,B,KOUNT1,KOUNT2,W1)
C
C     A SINGLE CALL TO SUBROUTINE PERM GENERATES ALL POSSIBLE
C     PERMUTATIONS OF N OBJECTS PARTITIONED INTO G NON-EMPTY SUBSETS
C
      DOUBLE PRECISION  ZERO,W1
      INTEGER  G,B(11),P(1000),A(11,400)
C
      ZERO = 0.0D0
      WS1 = ZERO
      WS2 = ZERO
      K = G
      IF(B(G+1).NE.0) K = K+1
      L = N
      DO 50 I = 1,K-1
         DO 30 J = 1,B(I)
            A(I,J) = I
   30    CONTINUE
         DO 40 J = B(I)+1,L
            A(I,J) = I+1
   40    CONTINUE
         L = L-B(I)
   50 CONTINUE
      KOUNT1 = 0
      KOUNT2 = 0
   60 I = K-1
      NMARKS = B(K)+B(I)
      IF(KOUNT1.EQ.0) GO TO 130
   70 DO 80 J = 1,NMARKS-1
         IF(A(I,J).EQ.I.AND.A(I,J+1).EQ.I+1) THEN
            LIMIT = J-2
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)-1
            IF(LIMIT.LE.0) GO TO 130
            GO TO 110
         END IF
   80 CONTINUE
      IF(NMARKS.EQ.1) GO TO 100
      DO 90 J = 1,NMARKS/2
         ITEMP = A(I,J)
         A(I,J) = A(I,NMARKS-J+1)
         A(I,NMARKS-J+1) = ITEMP
   90 CONTINUE
  100 I = I-1
      IF(I.EQ.0) RETURN
      NMARKS = NMARKS+B(I)
      GO TO 70
  110 IFLAG = 0
      DO 120 J = 1,LIMIT
         IF(A(I,J).EQ.I+1.AND.A(I,J+1).EQ.I) THEN
            A(I,J) = A(I,J+1)
            A(I,J+1) = A(I,J)+1
            IFLAG = 1
         END IF
  120 CONTINUE
      IF(IFLAG.EQ.1) GO TO 110
  130 DO 140 J = 1,N
         A(K,J) = A(1,J)
  140 CONTINUE
      IF(K.EQ.2) GO TO 170
      DO 160 L = 2,K-1
         M = 1
         DO 150 J = 1,N
            IF(A(K,J).EQ.L) THEN
               A(K,J) = A(L,M)
               M = M+1
            END IF
  150    CONTINUE
  160 CONTINUE
  170 LL = 1
      DO 190 II = 1,K
         DO 180 JJ = 1,N
            IF(A(K,JJ).EQ.II) THEN
               P(LL) = JJ
               LL = LL+1
            END IF
  180    CONTINUE
  190 CONTINUE
      CALL STAT(G,B,P,KOUNT2,W1)
      KOUNT1 = KOUNT1+1
      GO TO 60
      END
C
C
      SUBROUTINE STAT(NG,M,P,KOUNT2,W1)
C
C     SUBROUTINE STAT COMPUTES THE TERPSTRA-JONCKHEERE W STATISTIC
C     FOR EACH PERMUTATION OF THE INPUT DATA SET
C
      DOUBLE PRECISION  ZERO,ONE,W,W1,X(400,11)
      INTEGER  M(11),P(1000)
C
      ZERO = 0.0D0
      ONE = 1.0D0
   17 K = 0
      DO 19 I = 1,NG
        DO 18 J = 1,M(I)
          K = K+1
          X(J,I) = ONE*P(K)
   18   CONTINUE
   19 CONTINUE
      W = ZERO
      DO 100 I = 1,NG-1
        DO 90 J = I+1,NG
          DO 80 K = 1,M(I)
            DO 70 L = 1,M(J)
              IF(X(K,I).LT.X(L,J)) W = W+ONE
   70       CONTINUE
   80     CONTINUE
   90   CONTINUE
  100 CONTINUE
      IF(W.GE.W1) KOUNT2 = KOUNT2+1
      RETURN
      END
C
C
      SUBROUTINE PVALUE(Z,PROB)
C
C     SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C     Z IS GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PROB,ZERO,ONE,PI,A,B,C,G,H,U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 20
      PI = DATAN(ONE)*4
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*DEXP(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = DEXP(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.LT.ZERO) PROB = ONE-PROB
      RETURN
      END
