      PROGRAM X3W
      IMPLICIT REAL*8(A-H,O-Z)
C
C   PROGRAM X3W INPUTS THE NECESSARY PARAMETERS TO DRIVE SUBROUTINES
C   RWAY AND XCONT3.  IN ADDITION, PROGRAM X3W PRINTS OUT THE RESULTS
C   OF THE ANALYSIS AND STORES THE RESULTS ON A FILE.
C   THE PARAMETERS ARE:
C
C        (1) THE NAME OF THE INPUT FILE CONTAINING THE RAW DATA
C        (2) THE NAME OF THE OUTPUT FILE TO CONTAIN THE RESULTS
C        (3) THE DESCRIPTIVE HEADING FOR THE R-WAY ANALYSIS
C        (4) THE NUMBER OF DIMENSIONS (R) IN THE R-WAY ANALYSIS
C        (5) THE SIZES OF THE R DIMENSIONS (N1, N2, ..., NR)
C
C   THE DATA MUST BE ARRANGED AS FOLLOWS:
C
C      CONSIDER A 3-WAY CONTINGENCY TABLE AND LET
C      M(I(1), I(2), I(3)) DENOTE THE TABLE ENTRY OF
C      THE (I(1), I(2), I(3))TH CELL'S FREQUENCY WHERE
C      I(J) = 1, ..., N(J) FOR J = 1, 2, 3.  THE DATA ARE
C      TO BE FED INTO PROGRAM X3W IN A LONG STRING, WITH ONE
C      CELL'S FREQUENCY PER LINE.
C
C         M(  1,    1,    1 )
C         .
C         .
C         .
C         M(  1,    1,  N(3))
C         M(  1,    2,    1 )
C         .
C         .
C         .
C         M(  1,  N(2), N(3))
C         M(  2,    1,    1 )
C         .
C         .
C         .
C         M(N(1), N(2), N(3))
C
C   THE CURRENT DIMENSIONS FOR PROGRAM DRIVER ARE SET FOR R = 3
C   AND THE MAXIMUM SIZE OF N1, N2, & N3 = 20.
C
C   PROGRAM MODIFIED 5/20/2006
C
      CHARACTER  HEAD*60,FIN*10,FOUT*10,PRE*10
      INTEGER    R
      DIMENSION  U(17),N(10),INDX(10),MM1(20),MM2(20),MM3(20),
     +           X(10,25),A(10,20),C(10,20,6)
      DATA       PRE /'ENTER THE '/
C
    1 FORMAT(A)
      WRITE(*,*) PRE,'NAME OF THE INPUT FILE CONTAINING THE DATA:'
      READ(*,1) FIN
      WRITE(*,*) PRE,'FILE NAME FOR THE OUTPUT OF RESULTS:'
      READ(*,1) FOUT
      WRITE(*,*) PRE,'PROBLEM HEADING (UP TO 60 CHARACTERS):'
      READ(*,1) HEAD
      R = 3
      WRITE(*,*) PRE,'DIMENSION SIZES (N1, N2, N3):'
      READ(*,*) (N(I), I = 1,R)
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      OPEN(UNIT = 4,FILE = FIN,STATUS = 'OLD')
C
C         FIND THE MAXIMUM OF N(1), N(2), ..., N(R).
C
      MX = N(1)
      DO 2 I = 2,R
         MX = MAX(MX,N(I))
    2 CONTINUE
C
      CALL RWAY(R,N,MX,A,INDX,C,X,TOT,T,ET,VT,GT,ZT,S,ES,VS,GS,ZS,
     +          MM1,MM2,MM3,CELLFAC,TLR,IERR)
      CLOSE(UNIT = 4,STATUS = 'KEEP')
      IF(IERR.EQ.1) GO TO 223
      CALL PVALUE(ZT,GT,PT)
      CALL PVALUE(ZS,GS,PS)
C
C         OUTPUT THE RESULTS OF THE R-WAY ANALYSIS ONTO TAPE 'FOUT'.
C
      OPEN(UNIT = 8,FILE = FOUT,STATUS = 'NEW')
      WRITE(8,3) 'RESULTS OF THE R-WAY CONTINGENCY TABLE ANALYSIS'
    3 FORMAT(T5,A,/,T5,68('-'))
      WRITE(8,4) HEAD,'NUMBER OF DIMENSIONS (R) IN THE TABLE = ',R
    4 FORMAT(T5,'HEADING: ',A,/,T5,A,I4)
      DO 6 I = 1,R
         WRITE(8,5) 'SIZE OF DIMENSION',I,' =',N(I)
    5    FORMAT(T8,A,I2,A,I3)
    6 CONTINUE
      NT = IDNINT(TOT)
      WRITE(8,7) 'TOTAL NUMBER OF OBSERVATIONS IN TABLE = ',NT
    7 FORMAT(T5,A,I4)
      WRITE(8,8) 'OBSERVED CONTINGENCY TABLE STATISTIC T =',T,
     +              'EXPECTED VALUE OF THE TEST STATISTIC T =',ET,
     +              'EXACT VARIANCE OF THE TEST STATISTIC T =',VT,
     +              'EXACT SKEWNESS OF THE TEST STATISTIC T =',GT,
     +              'STANDARDIZED VALUE OF TEST STATISTIC T =',ZT,
     +              'PROBABILITY VALUE FOR TEST STATISTIC T =',PT,
     +              'OBSERVED CONTINGENCY TABLE STATISTIC S =',S,
     +              'EXPECTED VALUE OF THE TEST STATISTIC S =',ES,
     +              'EXACT VARIANCE OF THE TEST STATISTIC S =',VS,
     +              'EXACT SKEWNESS OF THE TEST STATISTIC S =',GS,
     +              'STANDARDIZED VALUE OF TEST STATISTIC S =',ZS,
     +              'PROBABILITY VALUE FOR TEST STATISTIC S =',PS
    8 FORMAT(6(/,T5,A,E12.5),/,6(/,T5,A,E12.5),/,T5,68('-'))
      NP = 1
      NS = 0
      DO 9 I = 1,R
         NP = NP*N(I)
         NS = NS+N(I)
    9 CONTINUE
      CS = T*NT**(R-1)-NT
      NDF = NP-NS+R-1
      CALL CHISQ(CS,NDF,PCS)
      CZ = S*NT**(R-1)+NP-NT
      CALL CHISQ(CZ,NDF,PCZ)
      CALL CHISQ(TLR,NDF,PLR)
      WRITE(8,7) 'DEGREES-OF-FREEDOM FOR ASYMPTOTIC TESTS = ',NDF
      WRITE(8,10) 'VALUE OF CHI-SQUARED TEST STATISTIC = ',CS
   10 FORMAT(T5,A,E12.5)
      WRITE(8,10) 'ASYMPTOTIC P-VALUE OF CHI-SQUARED TEST = ',PCS
      WRITE(8,10) 'VALUE OF ZELTERMAN TEST STATISTIC = ',CZ
      WRITE(8,10) 'ASYMPTOTIC P-VALUE OF ZELTERMAN TEST = ',PCZ
      WRITE(8,10) 'VALUE OF LIKELIHOOD-RATIO TEST STATISTIC = ',TLR
      WRITE(8,11) 'ASYMPTOTIC P-VALUE OF LIKELIHOOD-RATIO TEST = ',PLR
   11 FORMAT(T5,A,E12.5,/,T5,68('-'))
C
C         CONVERT TO THE ALTERNATIVE NOTATION.
C
      N1 = N(1)
      N2 = N(2)
      N3 = N(3)
C
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 18 II = 1,17
         SU = 0.0D0
         TU = 0.5D0
         DO 17 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) SU = SU+TU
            TU = TU/2
   17    CONTINUE
         U(II) = SU
   18 CONTINUE
C
      PSTART = 0.0D0
      DO 20 I = 1,N1
         PSTART = PSTART+FAC(MM1(I))
   20 CONTINUE
      DO 25 J = 1,N2
         PSTART = PSTART+FAC(MM2(J))
   25 CONTINUE
      DO 30 K = 1,N3
         PSTART = PSTART+FAC(MM3(K))
   30 CONTINUE
      PSTART = PSTART-FAC(NT)*(R-1)
      THISP = PSTART-CELLFAC
C
      CR = RZ(NT)*(R-1)
      DO 40 I = 1,N1
         CR = CR-RZ(MM1(I))
   40 CONTINUE
      DO 45 J = 1,N2
         CR = CR-RZ(MM2(J))
   45 CONTINUE
      DO 50 K = 1,N3
         CR = CR-RZ(MM3(K))
   50 CONTINUE
C
      FEP = DEXP(THISP)
      THISP = THISP*(1.0D0-1.0D-12)
      CST = CS*(1.0D0-1.0D-12)
      IF(CST.LT.1.0D-11) CST = -1.0D-11
      CZT = CZ*(1.0D0-1.0D-12)
      IF(CZT.LT.1.0D-11) CZT = -1.0D-11
      CLT = TLR*(1.0D0-1.0D-12)
      IF(CLT.LT.1.0D-11) CLT = -1.0D-11
C
      IFE = 0
      ICS = 0
      ICZ = 0
      ICL = 0
C
C  DETERMINE THE PROBABILITY AND STATISTICS FOR OBSERVED TABLE.
C
      DO 110 IS = 1,MS
         CALL XCONT3(NT,N1,N2,N3,CF,CS,CZ,CL,CR,U,MM1,MM2,MM3)
         CP = PSTART-CF
         IF(CP.LT.THISP) IFE = IFE+1
         IF(CS.GT.CST) ICS = ICS+1
         IF(CZ.GT.CZT) ICZ = ICZ+1
         IF(CL.GT.CLT) ICL = ICL+1
  110 CONTINUE
C
      PFE = 1.0D0*IFE/MS
      PCS = 1.0D0*ICS/MS
      PCZ = 1.0D0*ICZ/MS
      PCL = 1.0D0*ICL/MS
C
      WRITE(8,130) 'THREE-WAY TABLE RESAMPLING ANALYSES '
      WRITE(8,140) ISEED, ' = RANDOM NUMBER GENERATOR SEED'
      WRITE(8,140) MS, ' = NUMBER OF RANDOM TABLES GENERATED'
      WRITE(8,150) FEP, ' = PROBABILITY OF OBSERVED 3-WAY TABLE'
      WRITE(8,150) PFE, ' = SIMULATED FISHER''S EXACT TEST P-VALUE'
      IF(CST.LE.0.0D0) CST = 0.0D0
      WRITE(8,150) CST, ' = OBSERVED 3-WAY CHI-SQUARE STATISTIC T'
      WRITE(8,150) PCS, ' = SIMULATED CHI-SQUARE TEST P-VALUE'
      IF(CZT.LE.0.0D0) CZT = 0.0D0
      WRITE(8,150) CZT, ' = OBSERVED 3-WAY ZELTERMAN STATISTIC'
      WRITE(8,150) PCZ, ' = SIMULATED ZELTERMAN TEST P-VALUE'
      IF(CLT.LE.0.0D0) CLT = 0.0D0
      WRITE(8,150) CLT, ' = OBSERVED 3-WAY LIKELIHOOD-RATIO STATISTIC'
      WRITE(8,150) PCL, ' = SIMULATED LIKELIHOOD-RATIO TEST P-VALUE'
  130 FORMAT(T5,A,/)
  140 FORMAT(T5,I10,A)
  150 FORMAT(T5,G15.8,A)
      WRITE(*,222) FOUT
  222 FORMAT(/,T2,'N O T E:  RESULTS ARE IN OUTPUT FILE: ',A,/)
      CLOSE(UNIT = 8,STATUS = 'KEEP')
      GO TO 225
  223 CLOSE(UNIT = 8,STATUS = 'DELETE')
      WRITE(*,224) FIN
  224 FORMAT(/,T2,'E R R O R:  END-OF-FILE FOUND ON FILE: ',A,/)
  225 END
C
C
      SUBROUTINE XCONT3(NT,N1,N2,N3,CF,CS,CZ,CL,CR,U,MM1,MM2,MM3)
      IMPLICIT REAL*8(A-H,O-Z)
C
C    GENERATES RANDOM THREE-WAY TABLE WITH FIXED MARGINAL TOTALS
C
      DIMENSION  U(17),L1(20),L2(20),L3(20),MC(20,20,20),
     +   MM1(20),MM2(20),MM3(20),P1(20),P2(20),P3(20),
     +   W1(20),W2(20),W3(20)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      LT = NT
      DO 1 I = 1,N1
         L1(I) = MM1(I)
    1 CONTINUE
      DO 2 I = 1,N2
         L2(I) = MM2(I)
    2 CONTINUE
      DO 3 I = 1,N3
         L3(I) = MM3(I)
    3 CONTINUE
      DO 15 I = 1,N1
         DO 14 J = 1,N2
            DO 13 K = 1,N3
               MC(I,J,K) = 0
   13       CONTINUE
   14    CONTINUE
   15 CONTINUE
C
      DO 200 K = 1,NT
C
         DO 20 I = 1,N1
            P1(I) = ONE*L1(I)/LT
   20    CONTINUE
         W1(1) = P1(1)
         DO 25 I = 2,N1-1
            W1(I) = W1(I-1)+P1(I)
   25    CONTINUE
         W1(N1) = ONE
C
         DO 30 I = 1,N2
            P2(I) = ONE*L2(I)/LT
   30    CONTINUE
         W2(1) = P2(1)
         DO 35 I = 2,N2-1
            W2(I) = W2(I-1)+P2(I)
   35    CONTINUE
         W2(N2) = ONE
C
         DO 40 I = 1,N3
            P3(I) = ONE*L3(I)/LT
   40    CONTINUE
         W3(1) = P3(1)
         DO 45 I = 2,N3-1
            W3(I) = W3(I-1)+P3(I)
   45    CONTINUE
         W3(N3) = ONE
C
         Z = UNI(U)
         DO 80 I = 1,N1
            IF(Z.LE.W1(I)) THEN
               L1(I) = L1(I)-1
               I1 = I
               GO TO 85
            END IF
   80    CONTINUE
C
   85    Z = UNI(U)
         DO 90 I = 1,N2
            IF(Z.LE.W2(I)) THEN
               L2(I) = L2(I)-1
               I2 = I
               GO TO 95
            END IF
   90    CONTINUE
C
   95    Z = UNI(U)
         DO 100 I = 1,N3
            IF(Z.LE.W3(I)) THEN
               L3(I) = L3(I)-1
               I3 = I
               GO TO 105
            END IF
  100    CONTINUE
C
  105    LT = LT-1
         MC(I1,I2,I3) = MC(I1,I2,I3)+1
  200 CONTINUE
C
      CF = 0.0D0
      CS = 0.0D0
      CZ = 0.0D0
      CL = 0.0D0
      DO 230 I = 1,N1
        DO 225 J = 1,N2
          DO 220 K = 1,N3
             CF = CF+FAC(MC(I,J,K))
             CS = CS+(ONE*MC(I,J,K))**2/MM1(I)/MM2(J)/MM3(K)
             CZ = CZ+ONE*MC(I,J,K)*(MC(I,J,K)-1)/MM1(I)/MM2(J)/MM3(K)
             CL = CL+RZ(MC(I,J,K))
  220     CONTINUE
  225   CONTINUE
  230 CONTINUE
      CS = CS*NT**2-NT
      CZ = CZ*NT**2+N1*N2*N3-NT
      CL = (CL+CR)*2
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(5000)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION RZ(I)
C  FIND I(LOG(I))
      DOUBLE PRECISION X
      X = 1.0D0*I
      IF(X.GT.1.5D0) THEN
         RZ = X*DLOG(X)
      ELSE
         RZ = 0.0D0
      ENDIF
      RETURN
      END
C  ...........................................................................
C
C
      SUBROUTINE RWAY(R,N,MX,A,INDX,C,X,TOT,T,ET,VT,GT,ZT,
     +    S,ES,VS,GS,ZS,MM1,MM2,MM3,CELLFAC,TLR,IERR)
      IMPLICIT REAL*8(A-H,O-Z)
C
C   DESCRIPTION OF VARIABLES IN CALL LIST:
C
C        R    - INPUT, NUMBER OF DIMENSIONS TO BE ANALYZED
C        N    - INPUT, ARRAY OF SIZES OF THE R DIMENSIONS
C        MX   - INPUT, MAXIMUM LENGTH OF THE R DIMENSIONS
C
C        INDX - WORKING ARRAY, CONTAINING INDICES FOR THE R DIMENSIONS
C        C    - WORKING ARRAY, CONTAINING THE C-SUPER-M VALUES
C        A    - WORKING ARRAY, CONTAINING THE R MARGINAL DISTRIBUTIONS
C        X    - WORKING ARRAY, CONTAINING THE <I> INTERMEDIATE VALUES
C
C        TOT  - OUTPUT, TOTAL NUMBER OF OBSERVATIONS IN THE TABLE
C        T    - OUTPUT, OBSERVED VALUE OF TEST STATISTIC, T
C        ET   - OUTPUT, EXPECTED VALUE OF TEST STATISTIC, T
C        VT   - OUTPUT, EXACT VARIANCE OF THE T DISTRIBUTION
C        GT   - OUTPUT, EXACT SKEWNESS OF THE T DISTRIBUTION
C        ZT   - OUTPUT, STANDARDIZED VALUE OF TEST STATISTIC, T
C        S    - OUTPUT, OBSERVED VALUE OF TEST STATISTIC, S
C        ES   - OUTPUT, EXPECTED VALUE OF TEST STATISTIC, S
C        VS   - OUTPUT, EXACT VARIANCE OF THE S DISTRIBUTION
C        GS   - OUTPUT, EXACT SKEWNESS OF THE S DISTRIBUTION
C        ZS   - OUTPUT, STANDARDIZED VALUE OF TEST STATISTIC, S
C   CELLFAC   - OUTPUT, PART OF FISHER'S EXACT TEST PROBABILITY, CELLFAC
C        TLR  - OUTPUT, OBSERVED LIKELIHOOD RATIO TEST STATISTIC, TLR
C
C   USAGE:
C
C        SUBROUTINE RWAY CALCULATES THE VALUES FOR TEST STATISTICS
C        T AND S.  IN ADDITION, EXACT VALUES FOR THE CORRESPONDING
C        MEAN, VARIANCE, AND SKEWNESS TERMS OF EACH DISTRIBUTION
C        ARE COMPUTED.  STANDARDIZED VALUES OF T AND S ARE RETURNED
C        TO THE DRIVER PROGRAM.  STATISTICS T AND S AND THE REQUIRED
C        PARAMETERS ARE DESCRIBED IN MIELKE AND BERRY, "CUMULANT
C        METHODS FOR ANALYZING INDEPENDENCE OF R-WAY CONTINGENCY
C        TABLES AND GOODNESS-OF-FIT FREQUENCY DATA," BIOMETRIKA,
C        VOLUME 75 (1989), PAGES 790 - 793.
C
      INTEGER    R
      DIMENSION  C(R,MX,6),Q(6,2),X(R,25),A(R,MX),P(22),N(R),INDX(R),
     +           MM1(20),MM2(20),MM3(20)
C
C         INITIALIZE THE MARGINAL ARRAYS.
C
      ONE = 1.0D0
      ZERO = 0.0D0
      DO 2 J = 1,MX
         DO 1 I = 1,R
            A(I,J) = ZERO
    1    CONTINUE
    2 CONTINUE
C
C         ACCUMULATE THE MARGINAL DISTRIBUTIONS.
C
      I = 1
      IERR = 0
      INDX(1) = 1
    3 IF(I.EQ.1.AND.INDX(I).GT.N(1)) GO TO 5
      IF(INDX(I).GT.N(I)) THEN
         I = I-1
         INDX(I) = INDX(I)+1
         GO TO 3
      END IF
      IF(I.EQ.R) THEN
         READ(4,*,END = 34) M
         DO 4 J = 1,R
            A(J,INDX(J)) = A(J,INDX(J))+M
    4    CONTINUE
         INDX(I) = INDX(I)+1
         GO TO 3
      END IF
      I = I+1
      INDX(I) = 1
      GO TO 3
C
    5 DO 6 I = 1,N(1)
         MM1(I) = IDNINT(A(1,I))
    6 CONTINUE
      DO 7 I = 1,N(2)
         MM2(I) = IDNINT(A(2,I))
    7 CONTINUE
      DO 8 I = 1,N(3)
         MM3(I) = IDNINT(A(3,I))
    8 CONTINUE
C
C         CALCULATE THE TOTAL NUMBER OF CASES AND COMPUTE THE Q(1,1)
C         TO Q(6,2) VALUES, WHICH ARE N(1,1) TO N(6,2), IN THE PAPER.
C
      TOT = ZERO
      DO 16 I = 1,N(1)
         TOT = TOT+A(1,I)
   16 CONTINUE
      Q(1,1) = TOT-3
      DO 17 I = 2,4
         Q(I,1) = Q(I-1,1)*(TOT-4+I)
   17 CONTINUE
      Q(1,2) = TOT-5
      DO 18 I = 2,6
         Q(I,2) = Q(I-1,2)*(TOT-6+I)
   18 CONTINUE
C
C         COMPUTE THE C-SUPER-M INTERMEDIATE VALUES.
C
      DO 21 I = 1,R
         DO 20 J = 1,N(I)
            C(I,J,1) = A(I,J)
            DO 19 K = 1,5
               C(I,J,K+1) = C(I,J,K)*(C(I,J,1)-K)
   19       CONTINUE
   20    CONTINUE
   21 CONTINUE
C
C         CALCULATE THE <I> INTERMEDIATE VALUES.
C
      DO 28 I = 1,R
         FCAT = ONE*N(I)
         X(I,1) = FCAT*(FCAT-1)
         X(I,2) = (TOT-FCAT)*(FCAT-1)
         X(I,3) = X(I,1)*(FCAT-2)
         X(I,4) = X(I,2)*(FCAT-2)
         DO 22 J = 5,25
            X(I,J) = ZERO
   22    CONTINUE
         DO 26 K = 1,N(I)
            CAT0 = TOT-A(I,K)*2-FCAT+2
            CAT1 = TOT-A(I,K)-FCAT+1
            CAT2 = A(I,K)**2
            CAT3 = A(I,K)**3
            DO 23 J = 5,8
               X(I,J) = X(I,J)+C(I,K,J-4)/CAT2
   23       CONTINUE
            DO 24 J = 9,14
               X(I,J) = X(I,J)+C(I,K,J-8)/CAT3
   24       CONTINUE
            DO 25 J = 15,18
               X(I,J) = X(I,J)+C(I,K,J-14)*CAT1/CAT2
   25       CONTINUE
            X(I,19) = X(I,19)+(A(I,K)-1)*CAT1
            X(I,20) = X(I,20)+(A(I,K)-1)*CAT0*CAT1
   26    CONTINUE
         DO 27 J = 21,24
            X(I,J) = X(I,J)+X(I,J-16)*(FCAT-1)
   27    CONTINUE
         X(I,25) = X(I,25)+X(I,19)*(FCAT-2)
   28 CONTINUE
C
C         COMPUTE THE 22 PRODUCT TERMS FROM THE <I> VALUES.
C
      DO 29 I = 1,22
         P(I) = ONE
   29 CONTINUE
      DO 30 I = 1,R
         P(1)  = P(1)*(TOT-N(I))
         P(2)  = P(2)*N(I)
         P(3)  = P(3)*(X(I,8)+X(I,19))
         P(4)  = P(4)*X(I,7)
         P(5)  = P(5)*(X(I,7)+X(I,2))
         P(6)  = P(6)*X(I,6)
         P(7)  = P(7)*(X(I,6)+X(I,1))
         P(8)  = P(8)*X(I,5)
         P(9)  = P(9)*(X(I,14)+X(I,18)*3+X(I,20))
         P(10) = P(10)*(X(I,13)+X(I,17))
         P(11) = P(11)*(X(I,13)+X(I,17)*2+X(I,24)+X(I,25))
         P(12) = P(12)*X(I,12)
         P(13) = P(13)*(X(I,12)+X(I,16))
         P(14) = P(14)*(X(I,12)+X(I,23))
         P(15) = P(15)*(X(I,12)+X(I,16)+X(I,23)*2+X(I,4))
         P(16) = P(16)*X(I,11)
         P(17) = P(17)*(X(I,11)+X(I,15))
         P(18) = P(18)*(X(I,11)+X(I,22))
         P(19) = P(19)*(X(I,11)+X(I,22)*3+X(I,3))
         P(20) = P(20)*X(I,10)
         P(21) = P(21)*(X(I,10)+X(I,21))
         P(22) = P(22)*X(I,9)
   30 CONTINUE
C
C         CALCULATE THE TEST STATISTICS: T, S, AND TLR.
C
      REWIND 4
      T = ZERO
      S = ZERO
      CELLFAC = ZERO
      TLR = ZERO
      I = 1
      INDX(1) = 1
   31 IF(I.EQ.1.AND.INDX(I).GT.N(1)) GO TO 33
      IF(INDX(I).GT.N(I)) THEN
         I = I-1
         INDX(I) = INDX(I)+1
         GO TO 31
      END IF
      IF(I.EQ.R) THEN
         READ(4,*) M
         BIN = ONE
         DO 32 J = 1, R
            BIN = BIN*A(J,INDX(J))
   32    CONTINUE
         T = T+ONE*M*M/BIN
         S = S+ONE*M*(M-1)/BIN
         CELLFAC = CELLFAC+FAC(M)
         IF(M.GT.0) THEN
            TLR = TLR+ONE*M*DLOG(TOT**(R-1)*M/BIN)
         ELSE
            TLR = TLR+ZERO
         END IF
         INDX(I) = INDX(I)+1
         GO TO 31
      END IF
      I = I+1
      INDX(I) = 1
      GO TO 31
C
C         COMPUTE THE EXACT PARAMETERS OF DISTRIBUTIONS OF T AND S.
C
   33 TLR = TLR*2
      L  = R-1
      ET = (P(1)+(TOT-1)**L*P(2))/(TOT*(TOT-1))**L
      T2 = (P(3)+Q(1,1)**L*(P(4)*2+P(5))*2+Q(2,1)**L*(P(6)*6+
     +     P(7))+Q(3,1)**L*P(8))/Q(4,1)**L
      T3 = (P(9)+Q(1,2)**L*(P(10)*4+P(11))*3+Q(2,2)**L*(P(12)*32+
     +     P(13)*18+P(14)*12+P(15)*3)+Q(3,2)**L*(P(16)*68+
     +     P(17)*3+P(18)*18+P(19))+Q(4,2)**L*(P(20)*28+
     +     P(21)*3)+Q(5,2)**L*P(22))/Q(6,2)**L
      VT = T2-ET*ET
      GT = (T3-ET*T2*3+ET*ET*ET*2)/DSQRT(VT**3)
      ES = P(1)/(TOT*(TOT-1))**L
      S2 = (P(3)+Q(1,1)**L*P(4)*4+Q(2,1)**L*P(6)*2)/Q(4,1)**L
      S3 = (P(9)+Q(1,2)**L*P(10)*12+Q(2,2)**L*(P(13)*6+P(12)*32)+
     +     Q(3,2)**L*P(16)*32+Q(4,2)**L*P(20)*4)/Q(6,2)**L
      VS = S2-ES*ES
      GS = (S3-ES*S2*3+ES*ES*ES*2)/DSQRT(VS**3)
C
C         CALCULATE THE STANDARDIZED VALUES OF TWO STATISTICS T AND S.
C
      ZT = (T-ET)/DSQRT(VT)
      ZS = (S-ES)/DSQRT(VS)
      RETURN
   34 IERR = 1
      RETURN
      END
C
C
      SUBROUTINE PVALUE(Z,GAM,PROB)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE
C        OF Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE
C        OF Z.
C
      DOUBLE PRECISION  Z,GAM,PROB,ZERO,ONE,PI,A,B,C,D,E,F,G,H,R,
     +   U,W,X,Y,E1,E2,E3,E4,E5,G1,G2,G3,H0,H1,H2,H3,H4
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      IF(DABS(GAM).LT.0.01D0) GO TO 50
      R = ONE*2/DABS(GAM)
      D = R*R
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      F = R*R+10
      U = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)-
     +    ONE/(F*F*F*360)
      G1 = 0.0375D0
      G2 = 0.075D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      A  = R*R-1
      B  = R*R*(DLOG(R)-1)-U
      W  = -1.99D0/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF(Z.LT.W) THEN
         PROB = ONE
         RETURN
      END IF
      X = Z
      Y = Z+15
      DO 20 I = 1,199
         G = A*DLOG(R+X+G1*(I*2-1))-R*(X+G1*(I*2-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2 = H2+E(G)
   20 CONTINUE
      G = A*DLOG(R+X)-R*X+B
      H0 = E(G)
      G = A*DLOG(R+Y)-R*Y+B
      H3 = E(G)
      G = A*DLOG(R+X+G1*399)-R*(X+G1*399)+B
      H4 = E(G)
      PROB = G3*(H0+(H1+H4)*4+H2*2+H3)
      RETURN
   30 IF(Z.GT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = Z-15
      Y = Z
      DO 40 I = 1,199
         G = A*DLOG(R-X-G1*(I*2-1))+R*(X+G1*(I*2-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2 = H2+E(G)
   40 CONTINUE
      G = A*DLOG(R-X)+R*X+B
      H0 = E(G)
      G = A*DLOG(R-Y)+R*Y+B
      H3 = E(G)
      G = A*DLOG(R-X-G1*399)+R*(X+G1*399)+B
      H4 = E(G)
      PROB = ONE-G3*(H0+(H1+H4)*4+H2*2+H3)
      RETURN
   50 PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-738.0D0) GO TO 70
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 60
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 70
   60 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   70 IF(Z.LT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-738.0D0) E = DEXP(G)
      RETURN
      END
C
C
      SUBROUTINE CHISQ(T,N,P)
      DOUBLE PRECISION T,ZERO,ONE,V,X,P,PI,PX,A,B,E1,E2,E3,E4,E5,H,W
      ZERO = 0.0D0
      ONE = 1.0D0
      IF(T.LE.ZERO) THEN
         P = ONE
         GO TO 60
      END IF
      IF(N.EQ.1) THEN
         X = DSQRT(T)
         CALL PVN(X,PX)
         P = PX*2
         GO TO 60
      END IF
      V = ONE*N
      IF(V/2-N/2.GT.0.1D0) GO TO 30
      P = DEXP(-T/2)
      IF(N.LT.3) GO TO 60
      K = (N-2)/2
      B = ONE
      DO 20 J = 1,K
         L = J
         A = ONE
         DO 10 I = 1,L
            A = A*T/(2*I)
   10    CONTINUE
         B = B+A
   20 CONTINUE
      P = P*B
      GO TO 60
   30 PI = DATAN(ONE)*4
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*DSQRT(T)+1)
      P = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*DEXP(-T/2)/DSQRT(PI/2)
      IF(N.LT.2) GO TO 60
      K = (N-1)/2
      B = ZERO
      DO 50 J = 1,K
         L = J
         A = ONE
         DO 40 I = 1,L
            A = A*T/(2*I-1)
   40    CONTINUE
         B = B+A
   50 CONTINUE
      P = P+B*DEXP(-T/2)/DSQRT(PI*T/2)
   60 RETURN
      END
C
C
      SUBROUTINE PVN(Z,PROB)
C
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY OF A VALUE OF
C        Z BEING GREATER THAN OR EQUAL TO THE OBSERVED VALUE OF Z.
C
      DOUBLE PRECISION  Z,PROB,ZERO,ONE,PI,G,A,B,C,EE,H,
     +   U,W,E1,E2,E3,E4,E5
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      PROB = ZERO
      G = -Z*Z/2
      IF(G.LT.-709.1D0) GO TO 20
      A = 4.2D0
      U = DABS(Z)
      IF(U.GT.A) GO TO 10
      E1 =  0.31938153D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H  =  0.2316419D0
      W  =  ONE/(H*DABS(Z)+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*EE(G)/DSQRT(PI*2)
      GO TO 20
   10 B = 2.2D0
      C = 4.9D0
      PROB = EE(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   20 IF(Z.LT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION EE(G)
C
      DOUBLE PRECISION G
C
      EE = 0.0D0
      IF(G.GE.-709.1D0) EE = DEXP(G)
      RETURN
      END
