      PROGRAM DRIVER
C
C   THIS DRIVER PROGRAM INPUTS THE NECESSARY PARAMETERS AND THE DATA
C   TO DRIVE SUBROUTINE ASTAND.  SUBROUTINE ASTAND COMPUTES A MULTI-
C   VARIATE MEASURE OF AGREEMENT BETWEEN A SET OF RATERS AND A STAND-
C   DARD (OR CORRECT SET) OF RESPONSES.  IN ADDITION, PROGRAM DRIVER
C   PRINTS OUT THE RESULTS OF THE ANALYSIS.
C   THE PARAMETERS ARE:
C
C        (1) THE DESCRIPTIVE HEADING FOR THE AGREEMENT ANALYSIS
C        (2) THE NUMBER OF OBSERVERS OR RATERS (M)
C        (3) THE NUMBER OF OBJECTS OBSERVED BY EACH RATER (N)
C        (4) THE NUMBER OF DIMENSIONS/RESPONSES FOR EACH OBJECT (R)
C        (5) THE NAME OF THE INPUT FILE CONTAINING THE RAW DATA
C
C   THE DATA MUST BE ARRANGED AS FOLLOWS, WHERE X IS A DATA POINT, M IS
C   THE NUMBER OF RATERS/OBSERVERS, N IS THE NUMBER OF OBJECTS OBSERVED,
C   R IS THE NUMBER OF DIMENSIONS/RESPONSES FOR EACH OF THE N OBJECTS,
C   AND THE DATA ARE IN FREE-FIELD FORMAT (I.E., THE COLUMNS OF DATA ARE
C   SEPARATED BY A COMMA OR BY ONE OR MORE SPACES):
C
C        X(1,1,1), X(1,1,2), X(1,1,3), ..., X(1,1,R)
C        X(1,2,1), X(1,2,2), X(1,2,3), ..., X(1,2,R)
C          . . .     . . .     . . .          . . .
C        X(1,M,1), X(1,M,2), X(1,M,3), ..., X(1,M,R)
C        X(2,1,1), X(2,1,2), X(2,1,3), ..., X(2,1,R)
C        X(2,2,1), X(2,2,2), X(2,2,3), ..., X(2,2,R)
C          . . .     . . .     . . .          . . .
C        X(2,M,1), X(2,M,2), X(2,M,3), ..., X(2,M,R)
C          . . .     . . .     . . .          . . .
C        X(N,1,1), X(N,1,2), X(N,1,3), ..., X(N,1,R)
C        X(N,2,1), X(N,2,2), X(N,2,3), ..., X(N,2,R)
C          . . .     . . .     . . .          . . .
C        X(N,M,1), X(N,M,2), X(N,M,3), ..., X(N,M,R)
C
C   --------------------------------------------------------------------
C
C   EXAMPLE:
C
C      CONSIDER A PROBLEM WHERE EACH OF M = 3 RATERS ESTIMATES THE HEIGHT,
C   WIDTH, AND DEPTH (I.E., R = 3) OF N = 5 OBJECTS, IN CENTIMETERS.  THE
C   DATA ARE GIVEN BELOW, WHERE THE RATER 1 ESTIMATES THE HEIGHT, WIDTH,
C   AND DEPTH OF OBJECT 1 AS 8.0, 9.2, AND 6.0, RESPECTIVELY.  RATER 2
C   ESTIMATES THE HEIGHT, WIDTH, AND DEPTH OF OBJECT 1 AS 8.2, 9.0, AND
C   6.5, RESPECTIVELY.  RATER 3 ESTIMATES THE HEIGHT, WIDTH, AND DEPTH OF
C   OBJECT 1 AS 8.2, 9.0, AND 6.5, RESPECTIVELY.  AND RATER 1 ESTIMATES
C   THE HEIGHT, WIDTH, AND DEPTH OF OBJECT 2 AS 10.5, 2.5, AND 11.0,
C   RESPECTIVELY.  AND SO ON.  RATER NUMBER 1 IS CONSIDERED THE STANDARD
C   OR CORRECT SET OF RESPONSES AGAINST WHICH RATERS 2 AND 3 ARE COMPARED.
C
C         [HEIGHT]  [WIDTH]  [DEPTH]
C
C           8.0       9.2      6.0         [OBJECT 1; RATER 1]
C           8.2       9.0      6.5         [OBJECT 1; RATER 2]
C           8.2       9.0      6.5         [OBJECT 1; RATER 3]
C          10.5       2.5     11.0         [OBJECT 2; RATER 1]
C          11.2       3.0     11.5         [OBJECT 2; RATER 2]
C           9.5       2.8     12.5         [OBJECT 2; RATER 3]
C          17.6       4.5     13.0         [OBJECT 3; RATER 1]
C          20.0       4.5     15.0         [OBJECT 3; RATER 2]
C          21.4       4.5     17.0         [OBJECT 3; RATER 3]
C           9.0      12.0     14.2         [OBJECT 4; RATER 1]
C           9.0      12.5     14.0         [OBJECT 4; RATER 2]
C           9.5      13.5     14.4         [OBJECT 4; RATER 3]
C          14.6       6.0      7.5         [OBJECT 5; RATER 1]
C          14.2       6.0      8.0         [OBJECT 5; RATER 2]
C          14.5       5.5      9.2         [OBJECT 5; RATER 3]
C
C   --------------------------------------------------------------------
C
C                RESULTS OF THE AGREEMENT ANALYSIS:
C
C   HEADING: OUTPUT OF RESULTS FOR THE EXAMPLE
C
C   NUMBER OF RATERS (M)    =  3
C   NUMBER OF OBJECTS (N)   =  5
C   NUMBER OF RESPONSES (R) =  3
C
C   COMPUTED VALUE OF DELTA =   .343204E+01
C   EXPECTED VALUE OF DELTA =   .162184E+02
C   VARIANCE OF DELTA       =   .636389E+01
C   SKEWNESS OF DELTA       =  -.492177E+00
C   AGREEMENT MEASURE (R)   =   .788386E+00
C   PROBABILITY VALUE (P)   =   .528116E-04
C
C   --------------------------------------------------------------------
C
C   THE CURRENT DIMENSIONS FOR PROGRAM DRIVER ARE SET FOR N = 50,
C   M = 35, AND R = 10.  TO CHANGE THE DIMENSIONS, REPLACE THE VALUES
C   IN THE 'DOUBLE PRECISION' STATEMENT BELOW; BE SURE THAT 'D' IS
C   DIMENSIONED AT (2*N,2*N) AND 'TDATA' IS DIMENSIONED AT (N*M*R).
C
C   FINALLY, NOTE THAT IN THE PROGRAMS M = KM, N = KN, AND R = KR.
C
C   --------------------------------------------------------------------
C
      CHARACTER         HEAD*60, FIN*20
      INTEGER           KM, KN, KR, I, J, K
      DOUBLE PRECISION  C1(35), C2(35), C3(35), D(100,100), DEL(35),
     &                  DATA(50,2,10), SJ1(50,2,2), SJ2(50,2,2),
     &                  SJ3(50,2,2), TDATA(17500), UJ(50,2,2), DELTA,
     &                  EDEL, GAM, PROB, RHO, VAR
C
    1 FORMAT (A)
      WRITE  (*, *) 'ENTER THE PROBLEM HEADING (UP TO 60 CHARACTERS):'
      READ   (*, 1)  HEAD
      WRITE  (*, *) 'ENTER THE NUMBER OF OBSERVERS/RATERS (M):'
      READ   (*, *)  KM
      WRITE  (*, *) 'ENTER THE NUMBER OF OBJECTS OBSERVED (N):'
      READ   (*, *)  KN
      WRITE  (*, *) 'ENTER THE NUMBER OF DIMENSIONS/RESPONSES (R):'
      READ   (*, *)  KR
      WRITE  (*, *) 'ENTER THE NAME OF THE DATA INPUT FILE:'
      READ   (*, 1)  FIN
      OPEN   (5, FILE = FIN, STATUS = 'OLD')
      DO 3 I = 1,KN
         DO 2 J = 1,KM
            READ (5,*,END = 8) (TDATA(KN*KM*(K-1)+KN*(J-1)+I),K = 1,KR)
    2    CONTINUE
    3 CONTINUE
      CLOSE  (5)
C
C   CALL SUBROUTINE ASTAND
C
      CALL ASTAND (KM, KN, KR, TDATA, D, C1, C2, C3, DEL, DATA, SJ1,
     &             SJ2, SJ3, UJ, DELTA, EDEL, VAR, GAM, RHO, PROB)
C
      WRITE  (*, 4)
    4 FORMAT (//, T5, 'RESULTS OF THE AGREEMENT ANALYSIS', /,
     &            T5, 68('-'))
      WRITE  (*, 5) HEAD, KM, KN, KR
    5 FORMAT (T5, 'HEADING: ',A,/,
     &        T5, 'NUMBER OF RATERS (M)    = ', I2, /,
     &        T5, 'NUMBER OF OBJECTS (N)   = ', I2, /,
     &        T5, 'NUMBER OF RESPONSES (R) = ', I2, /)
      IF (VAR .NE. 0.0) THEN
         WRITE  (*, 6) DELTA, EDEL, VAR, GAM, RHO, PROB
    6    FORMAT (T5, 'COMPUTED VALUE OF DELTA = ', E13.6, /,
     &           T5, 'EXPECTED VALUE OF DELTA = ', E13.6, /,
     &           T5, 'VARIANCE OF DELTA       = ', E13.6, /,
     &           T5, 'SKEWNESS OF DELTA       = ', E13.6, /,
     &           T5, 'AGREEMENT MEASURE (R)   = ', E13.6, /,
     &           T5, 'PROBABILITY VALUE (P)   = ', E13.6, /,
     &           T5, 68('-'),/)
         GO TO 10
      ELSE
         WRITE  (*, 7) DELTA, EDEL, RHO
    7    FORMAT (T5, 'COMPUTED VALUE OF DELTA = ', E13.6, /,
     &           T5, 'EXPECTED VALUE OF DELTA = ', E13.6, /,
     &           T5, 'AGREEMENT MEASURE (R)   = ', E13.6, /,
     &           T5, 'NOTE: THE VARIANCE IS ZERO', /,
     &           T5, 'THE PROBABILITY IS, THEREFORE, 1.0', /,
     &           T5, 68('-'),/)
         GO TO 10
      END IF
    8 WRITE  (*, 9) FIN
    9 FORMAT (/, T2, 'E R R O R: END-OF-FILE FOUND ON FILE: ', A, /)
   10 END
C
C
C
      SUBROUTINE ASTAND (KM, KN, KR, TDATA, D, C1, C2, C3, DEL, DATA,
     &                   SJ1, SJ2, SJ3, UJ, DELTA, EDEL, VAR, GAM,
     &                   RHO, PROB)
C
C   DESCRIPTION OF VARIABLES IN CALL LIST:
C        KM    - INPUT, NUMBER OF OBSERVERS/RATERS
C        KN    - INPUT, NUMBER OF OBJECTS OBSERVED
C        KR    - INPUT, NUMBER OF DIMENSIONS/RESPONSES
C        TDATA - INPUT, MATRIX CONTAINING THE RAW SCORE VALUES
C        D      - WORKING ARRAY, CONTAINING THE DIFFERENCE SCORES
C        C1     - WORKING ARRAY, CONTAINING C1 INTERMEDIATE VALUES
C        C2     - WORKING ARRAY, CONTAINING C2 INTERMEDIATE VALUES
C        C3     - WORKING ARRAY, CONTAINING C3 INTERMEDIATE VALUES
C        DEL    - WORKING ARRAY, CONTAINING DEL INTERMEDIATE VALUES
C        DATA   - WORKING ARRAY, CONTAINING ADJUSTED DATA VALUES
C        SJ1    - WORKING ARRAY, CONTAINING SJ1 INTERMEDIATE VALUES
C        SJ2    - WORKING ARRAY, CONTAINING SJ2 INTERMEDIATE VALUES
C        SJ3    - WORKING ARRAY, CONTAINING SJ3 INTERMEDIATE VALUES
C        UJ     - WORKING ARRAY, CONTAINING UJ INTERMEDIATE VALUES
C        DELTA  - OUTPUT, OBSERVED (REALIZED) VALUE OF DELTA
C        EDEL   - OUTPUT, EXPECTED (MEAN) VALUE OF DELTA
C        VAR    - OUTPUT, VARIANCE OF THE DELTA DISTRIBUTION
C        GAM    - OUTPUT, SKEWNESS OF THE DELTA DISTRIBUTION
C        RHO    - OUTPUT, DELTA-BASED AGREEMENT COEFFICIENT
C        PROB   - OUTPUT, PROBABILITY OF AGREEMENT COEFFICIENT
C   USAGE:
C        SUBROUTINE ASTAND CALCULATES THE VALUE FOR DELTA AND THE
C        VALUE FOR THE COEFFICIENT OF AGREEMENT, R.  IN ADDITION,
C        EXACT VALUES FOR THE MEAN (EDEL), VARIANCE (VAR), AND
C        SKEWNESS (GAM) OF THE DELTA DISTRIBUTION ARE COMPUTED.
C
      INTEGER           KM, KN, KR, I, J, K
      DOUBLE PRECISION  C1(KM), C2(KM), C3(KM), D(2*KN,2*KN),
     &                  DATA(KN,2,KR), DEL(KM), SJ1(KN,2,2),
     &                  SJ2(KN,2,2), SJ3(KN,2,2), TDATA(KN,KM,KR),
     &                  UJ(KN,2,2), C11, C22, C33, CUM1, CUM2, CUM3,
     &                  DELTA, EDEL, GAM, PROB, RHO, T, VAR, ZERO
C
      ZERO = 0.0D0
      DO 2 I = 1,KN
         DO 1 J = 1,KR
            DATA(I,1,J) = TDATA(I,1,J)
    1    CONTINUE
    2 CONTINUE
      DO 5 I = 2,KM
         DO 4 J = 1,KN
            DO 3 K = 1,KR
               DATA(J,2,K) = TDATA(J,I,K)
    3       CONTINUE
    4    CONTINUE
C
C   CALL SUBROUTINE CALC
C
         CALL CALC (KN, KR, D, DATA, SJ1, SJ2, SJ3, UJ, CUM1, CUM2,
     &              CUM3, DELTA)
C
         DEL(I) = DELTA
         C1(I)  = CUM1
         C2(I)  = CUM2
         C3(I)  = CUM3
    5 CONTINUE
      DELTA = ZERO
      C11 = ZERO
      C22 = ZERO
      C33 = ZERO
      DO 6 I = 2,KM
         DELTA = DELTA+DEL(I)
         C11 = C11+C1(I)
         C22 = C22+C2(I)
         C33 = C33+C3(I)
    6 CONTINUE
      EDEL = C11
      VAR = C22
      GAM = C33/DSQRT(VAR**3)
      T = (DELTA-EDEL)/DSQRT(VAR)
      RHO = 1.0D0-(DELTA/EDEL)
C
C   CALL SUBROUTINE PVALUE
C
      CALL PVALUE (T, GAM, PROB)
      RETURN
      END
C
C
C
      SUBROUTINE CALC (KN, KR, D, DATA, SJ1, SJ2, SJ3, UJ, CUM1, CUM2,
     &                 CUM3, DELTA)
C
C   DESCRIPTION OF VARIABLES IN CALL LIST:
C        KN    - INPUT, NUMBER OF OBJECTS OBSERVED
C        KR    - INPUT, NUMBER OF DIMENSIONS/RESPONSES
C        D      - WORKING ARRAY, CONTAINING THE DIFFERENCE SCORES
C        DATA   - WORKING ARRAY, CONTAINING ADJUSTED DATA VALUES
C        SJ1    - WORKING ARRAY, CONTAINING SJ1 INTERMEDIATE VALUES
C        SJ2    - WORKING ARRAY, CONTAINING SJ2 INTERMEDIATE VALUES
C        SJ3    - WORKING ARRAY, CONTAINING SJ3 INTERMEDIATE VALUES
C        UJ     - WORKING ARRAY, CONTAINING UJ INTERMEDIATE VALUES
C        VI     - WORKING ARRAY, CONTAINING VI INTERMEDIATE VALUES
C        CUM1   - OUTPUT, INITIAL SUM FOR THE FIRST CUMULANT
C        CUM2   - OUTPUT, INITIAL SUM FOR THE SECOND CUMULANT
C        CUM3   - OUTPUT, INITIAL SUM FOR THE THIRD CUMULANT
C        DELTA  -
C   USAGE:
C        SUBROUTINE CALC COMPUTES THE INITIAL VALUES FOR SUBROUTINE
C        ASTAND, PRELIMINARY TO CALCULATING THE COEFFICIENT OF
C        AGREEMENT (R), THE EXACT MEAN (EDEL), VARIANCE (VAR), AND
C        SKEWNESS (GAM) OF THE DELTA DISTRIBUTION.
C
      INTEGER           I, IJ, IR, IRR, IS, J, JSS, K, KN, KL, KR, LO, M
      DOUBLE PRECISION  D(2*KN,2*KN), DATA(KN,2,KR), SIJ1(2,2),
     &                  SIJ2(2,2), SIJ3(2,2), TIJ2(2,2), TIJ3(2,2),
     &                  SJ1(KN,2,2), SJ2(KN,2,2), SJ3(KN,2,2),
     &                  UIJ(2,2), UJ(KN,2,2), VI(2,2), C1, C2,
     &                  C3, CUM1, CUM2, CUM3, DELTA, T1, T2, ZERO
C
      ZERO = 0.0D0
      DO 2 I = 1,2*KN
         DO 1 J = 1,2*KN
            D(I,J) = ZERO
    1    CONTINUE
    2 CONTINUE
      DO 7 I = 1,KN
         DO 6 J = 1,2
            DO 5 K = I,KN
               LO = 1
               IF (I.EQ.K) LO = J
               DO 4 L = LO,2
                  IJ = 2*(I-1)+J
                  KL = 2*(K-1)+L
                  D(IJ,KL) = ZERO
                  DO 3 M = 1,KR
                     D(IJ,KL) = D(IJ,KL)+(DATA(I,J,M)-DATA(K,L,M))**2
    3             CONTINUE
                  D(IJ,KL) = D(IJ,KL)**0.5
                  D(KL,IJ) = D(IJ,KL)
    4          CONTINUE
    5       CONTINUE
    6    CONTINUE
    7 CONTINUE
      DO 11 IS = 1,2
         DO 10 IR = 1,2
            IF (IR.EQ.IS) GO TO 10
            SIJ1(IR,IS) = ZERO
            SIJ2(IR,IS) = ZERO
            SIJ3(IR,IS) = ZERO
            DO 9 I = 1,KN
               SJ1(I,IR,IS) = ZERO
               SJ2(I,IR,IS) = ZERO
               SJ3(I,IR,IS) = ZERO
               DO 8 J = 1,KN
                  IRR = (I-1)*2+IR
                  JSS = (J-1)*2+IS
                  SJ1(I,IR,IS) = SJ1(I,IR,IS)+D(IRR,JSS)
                  SJ2(I,IR,IS) = SJ2(I,IR,IS)+D(IRR,JSS)**2
                  SJ3(I,IR,IS) = SJ3(I,IR,IS)+D(IRR,JSS)**3
    8          CONTINUE
               SIJ1(IR,IS) = SIJ1(IR,IS)+SJ1(I,IR,IS)
               SIJ2(IR,IS) = SIJ2(IR,IS)+SJ2(I,IR,IS)
               SIJ3(IR,IS) = SIJ3(IR,IS)+SJ3(I,IR,IS)
    9       CONTINUE
   10    CONTINUE
   11 CONTINUE
      T2 = ZERO
      TIJ2(1,2) = ZERO
      TIJ3(1,2) = ZERO
      VI(1,2)   = ZERO
      UIJ(1,2)  = ZERO
      DO 13 I = 1,KN
         TIJ2(1,2) = TIJ2(1,2)+SJ1(I,1,2)**2+SJ1(I,2,1)**2
         TIJ3(1,2) = TIJ3(1,2)+SJ1(I,1,2)**3+SJ1(I,2,1)**3
         VI(1,2)   = VI(1,2)+SJ1(I,1,2)*SJ2(I,1,2)+SJ1(I,2,1)*SJ2(I,2,1)
         UJ(I,1,2) = ZERO
         DO 12 J = 1,KN
            IRR = 2*(I-1)+1
            JSS = 2*(J-1)+2
            UJ(I,1,2) = UJ(I,1,2)+D(IRR,JSS)*SJ1(I,1,2)*SJ1(J,2,1)
   12    CONTINUE
         UIJ(1,2) = UIJ(1,2)+UJ(I,1,2)
   13 CONTINUE
      T1 = ZERO
      IF (KN.GT.2) THEN
         T1 = T1+4.0D0*SIJ1(1,2)**3-SIJ1(1,2)*TIJ2(1,2)*6.0D0*
     &        KN+UIJ(1,2)*6.0D0*KN*KN+TIJ3(1,2)*2.0D0*KN*KN+
     &        SIJ1(1,2)*SIJ2(1,2)*3.0D0*KN*KN-VI(1,2)*3.0D0*
     &        KN*KN*KN+SIJ3(1,2)*KN**4
      END IF
      IF (KN.GT.2) T1 = T1/DBLE(KN-2)
      C1 = 1.0D0/DBLE(KN*KN)
      C2 = C1*C1
      C3 = C2*C1
      CUM1 = C1*SIJ1(1,2)
      CUM2 = (SIJ1(1,2)*SIJ1(1,2)-TIJ2(1,2)*KN+SIJ2(1,2)*KN*KN)*C2/
     &        DBLE(KN-1)
      CUM3 = C3*(T1-T2)/DBLE(KN-1)
      DELTA = ZERO
      DO 14 I = 1,KN
         IRR = (I-1)*2+1
         ISS = (I-1)*2+2
         DELTA = DELTA+D(IRR,ISS)
   14 CONTINUE
      DELTA = DELTA/DBLE(KN)
      RETURN
      END
C
C
C
      SUBROUTINE PVALUE (T, GAM, PROB)
C
C   DESCRIPTION OF VARIABLES IN CALL LIST:
C        T    - INPUT,  STANDARDIZED TEST STATISTIC
C        GAM  - INPUT,  SKEWNESS OF THE DELTA DISTRIBUTION
C        PROB - OUTPUT, PROBABILITY OF THE TEST STATISTIC
C   USAGE:
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C        T IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF T.
C
      DOUBLE PRECISION A, B, D, E, E1, E2, E3, E4, E5, F, G, G1, G2,
     &                 G3, GAM, H, H0, H1, H1N, H2, H3, PHI, PI, PROB,
     &                 R, T, U, W, X, Y, ZERO
      INTEGER          I
C
      ZERO = 0.0D0
      PI = 4.0D0*DATAN(1.0D0)
      IF (DABS(GAM).LT.0.01D0) GO TO 5
      R = 2.0D0/DABS(GAM)
      D = R*R
      F = D+10.0D0
      DO 1 I = 1,9
         D = D*(R*R+I)
    1 CONTINUE
      U = (2.0D0*F-1.0D0)*DLOG(F)/2.0D0-F+DLOG(2.0D0*PI)/2.0D0-DLOG(D)
     &    +1.0D0/(12.0D0*F)-1.0D0/(360.0D0*F**3)
      A = R*R-1.0D0
      B = R*R*(DLOG(R)-1.0D0)-U
      G1 = 0.0375D0
      G2 = 0.0750D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      W = -1.99D0/GAM
      IF (GAM.LT.ZERO) GO TO 3
      IF (T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+15.0D0
      DO 2 I = 1,199
         PHI = A*DLOG(R+X+G1*(2.0D0*I-1.0D0))-R*(X+G1*(2.0D0*I-1.0D0))+B
         H1  = H1+E(PHI)
         PHI = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2  = H2+E(PHI)
    2 CONTINUE
      PHI = A*DLOG(R+X+G1*399.0D0)-R*(X+G1*399.0D0)+B
      H1N = E(PHI)
      PHI = A*DLOG(R+X)-R*X+B
      H0  = E(PHI)
      PHI = A*DLOG(R+Y)-R*Y+B
      H3  = E(PHI)
      PROB = 1.0D0-G3*(H0+(H1+H1N)*4.0D0+H2*2.0D0+H3)
      RETURN
    3 IF (T.GT.W) THEN
         PROB = 1.0D0
         RETURN
      END IF
      X = T-15.0D0
      Y = T
      DO 4 I = 1,199
         PHI = A*DLOG(R-X-G1*(2.0D0*I-1.0D0))+R*(X+G1*(2.0D0*I-1.0D0))+B
         H1  = H1+E(PHI)
         PHI = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2  = H2+E(PHI)
    4 CONTINUE
      PHI = A*DLOG(R-X-G1*399.0D0)+R*(X+G1*399.0D0)+B
      H1N = E(PHI)
      PHI = A*DLOG(R-X)+R*X+B
      H0  = E(PHI)
      PHI = A*DLOG(R-Y)+R*Y+B
      H3  = E(PHI)
      PROB = G3*(H0+(H1+H1N)*4.0D0+H2*2+H3)
      RETURN
    5 PROB = ZERO
      G = -T*T/2.0D0
      IF (G.LT.-709.1D0) GO TO 7
      A = 4.2D0
      U = DABS(T)
      IF (U.GT.A) GO TO 6
      E1 =  0.319381530D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H =   0.231641900D0
      W = 1.0D0/(H*U+1.0D0)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(2.0D0*PI)
      GO TO 7
    6 B = 2.2D0
      PROB = E(G)*(1.0D0/U-1.0D0/U**3+2.2D0/U**4.9)/DSQRT(2.0D0*PI)
    7 IF (T.GT.ZERO) PROB = 1.0D0-PROB
      RETURN
      END
C
C
C
      DOUBLE PRECISION FUNCTION E(G)
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF (G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
