      PROGRAM DIFFER
c
C   PROGRAM DIFFER COMPUTES THE PROBABILITY OF AN OBSERVED DIFFERENCE
C   BETWEEN TWO MEASURES OF AGREEMENT (R1 AND R2) OBTAINED FROM TWO
C   INDEPENDENT SETS OF RATERS.  THE VALUES TO BE INPUT INTO PROGRAM
C   DIFFER ARE GENERATED FROM PROGRAM AGREE.  SEE BERRY & MIELKE, A
C   GENERALIZED AGREEMENT MEASURE, EDUCATIONAL AND PSYCHOLOGICAL
C   MEASUREMENT, 50 (1990), PAGES 122-125.
C
C   EXAMPLE SUITABLE FOR TESTING PROGRAM DIFFER.  THE DATA LISTED
C   BELOW WERE GATHERED FROM 8 RATERS ON 40 OBJECTS IN 6 RESPONSES.
C
C          INPUT         GROUP 1        GROUP 2
C          ----------------------------------------------
C          R VALUES:     0.11578        0.19780
C          MEANS:        1.27050        1.60240
C          VARIANCES:    0.4678D-03     0.1010D-02
C          SKEWNESS:    -0.34145       -0.28425
C
C          OUTPUT
C          ----------------------------------------------
C          AGREEMENT VALUE (R1) =  .115780D+00
C          AGREEMENT VALUE (R2) =  .197800D+00
C          DIFFERENCE VALUE (D) = -.820200D-01
C          STANDARDIZED D (T) =   -.313804D+01
C          EXPECTED VALUE OF D =   .000000D+00
C          VARIANCE OF D =         .683159D-03
C          SKEWNESS OF D =        -.298469D-01
C          PROBABILITY OF D =      .196608D-02
C
      DOUBLE PRECISION  R1, R2, MU1, MU2, VAR1, VAR2, GAM1, GAM2,
     &                  D, T, MUD, SIG1, SIG2, SIGD, VARD, GAMD,
     &                  PROBD
C
      WRITE  (*, 1) ' ENTER THE AGREEMENT VALUES FOR GROUPS 1 AND 2',
     &              ' [R(1) AND R(2)]:'
    1 FORMAT (A,A)
      READ   (*, *)   R1, R2
      WRITE  (*, 1) ' ENTER THE AVERAGE VALUES FOR GROUPS 1 AND 2',
     &              ' [MEAN(1) AND MEAN(2)]:'
      READ   (*, *)   MU1, MU2
      WRITE  (*, 1) ' ENTER THE VARIANCES FOR GROUPS 1 AND 2',
     &              ' [VARIANCE(1) AND VARIANCE(2)]:'
      READ   (*, *)   VAR1, VAR2
      WRITE  (*, 1) ' ENTER THE SKEWNESS VALUES FOR GROUPS 1 AND 2',
     &              ' [SKEWNESS(1) AND SKEWNESS(2)]:'
      READ   (*, *)   GAM1, GAM2
C
      D = R1-R2
      MUD = 0.0D0
      VARD = (MU1**2*VAR2+MU2**2*VAR1)/(MU1**2*MU2**2)
      SIG1 = DSQRT(VAR1)
      SIG2 = DSQRT(VAR2)
      SIGD = DSQRT(VARD)
      GAMD = (MU1**3*SIG2**3*GAM2-MU2**3*SIG1**3*GAM1)/
     &       (MU1**3*MU2**3*SIGD**3)
      T = D/SIGD
C
C   CALL SUBROUTINE PVALUE
C
      CALL PVALUE (T, GAMD, PROBD)
C
      IF (PROBD.GT.0.5D0) THEN
         PROBD = 2.0D0*(1.0D0-PROBD)
      ELSE
         PROBD = 2.0D0*PROBD
      END IF
C
C   BEGIN OUTPUT OF RESULTS
C
      WRITE  (*, 2) 'O U T P U T   O F   P R O G R A M   D I F F E R'
    2 FORMAT (//,T17,A,/)
      WRITE  (*, 3) R1, R2, D, T, MUD, VARD, GAMD, PROBD
    3 FORMAT (T23,'AGREEMENT VALUE (R1) = ',D12.6,/,
     &        T23,'AGREEMENT VALUE (R2) = ',D12.6,/,
     &        T23,'DIFFERENCE VALUE (D) = ',D12.6,/,
     &        T23,'STANDARDIZED D (T) =   ',D12.6,/,
     &        T23,'EXPECTED VALUE OF D =  ',D12.6,/,
     &        T23,'VARIANCE OF D =        ',D12.6,/,
     &        T23,'SKEWNESS OF D =        ',D12.6,/,
     &        T23,'PROBABILITY OF D =     ',D12.6,/)
      END
C
C
C
      SUBROUTINE PVALUE (T, GAM, PROB)
C
C   DESCRIPTION OF VARIABLES IN CALL LIST:
C        T    - INPUT,  STANDARDIZED TEST STATISTIC
C        GAM  - INPUT,  SKEWNESS OF THE DELTA DISTRIBUTION
C        PROB - OUTPUT, PROBABILITY OF THE TEST STATISTIC
C   USAGE:
C        SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C        T IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF T.
C
      DOUBLE PRECISION A, B, D, E, E1, E2, E3, E4, E5, F, G, G1, G2,
     &                 G3, GAM, H, H0, H1, H1N, H2, H3, PHI, PI, PROB,
     &                 R, T, U, W, X, Y, ZERO
      INTEGER          I
C
      ZERO = 0.0D0
      PI = 4.0D0*DATAN(1.0D0)
      IF (DABS(GAM).LT.0.01D0) GO TO 5
      R = 2.0D0/DABS(GAM)
      D = R*R
      F = D+10.0D0
      DO 1 I = 1,9
         D = D*(R*R+I)
    1 CONTINUE
      U = (2.0D0*F-1.0D0)*DLOG(F)/2.0D0-F+DLOG(2.0D0*PI)/2.0D0-DLOG(D)
     &    +1.0D0/(12.0D0*F)-1.0D0/(360.0D0*F**3)
      A = R*R-1.0D0
      B = R*R*(DLOG(R)-1.0D0)-U
      G1 = 0.0375D0
      G2 = 0.0750D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      W = -1.99D0/GAM
      IF (GAM.LT.ZERO) GO TO 3
      IF (T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+15.0D0
      DO 2 I = 1,199
         PHI = A*DLOG(R+X+G1*(2.0D0*I-1.0D0))-R*(X+G1*(2.0D0*I-1.0D0))+B
         H1  = H1+E(PHI)
         PHI = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2  = H2+E(PHI)
    2 CONTINUE
      PHI = A*DLOG(R+X+G1*399.0D0)-R*(X+G1*399.0D0)+B
      H1N = E(PHI)
      PHI = A*DLOG(R+X)-R*X+B
      H0  = E(PHI)
      PHI = A*DLOG(R+Y)-R*Y+B
      H3  = E(PHI)
      PROB = 1.0D0-G3*(H0+(H1+H1N)*4.0D0+H2*2.0D0+H3)
      RETURN
    3 IF (T.GT.W) THEN
         PROB = 1.0D0
         RETURN
      END IF
      X = T-15.0D0
      Y = T
      DO 4 I = 1,199
         PHI = A*DLOG(R-X-G1*(2.0D0*I-1.0D0))+R*(X+G1*(2.0D0*I-1.0D0))+B
         H1  = H1+E(PHI)
         PHI = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2  = H2+E(PHI)
    4 CONTINUE
      PHI = A*DLOG(R-X-G1*399.0D0)+R*(X+G1*399.0D0)+B
      H1N = E(PHI)
      PHI = A*DLOG(R-X)+R*X+B
      H0  = E(PHI)
      PHI = A*DLOG(R-Y)+R*Y+B
      H3  = E(PHI)
      PROB = G3*(H0+(H1+H1N)*4.0D0+H2*2+H3)
      RETURN
    5 PROB = ZERO
      G = -T*T/2.0D0
      IF (G.LT.-709.1D0) GO TO 7
      A = 4.2D0
      U = DABS(T)
      IF (U.GT.A) GO TO 6
      E1 =  0.319381530D0
      E2 = -0.356563782D0
      E3 =  1.781477937D0
      E4 = -1.821255978D0
      E5 =  1.330274429D0
      H =   0.231641900D0
      W = 1.0D0/(H*U+1.0D0)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(2.0D0*PI)
      GO TO 7
    6 B = 2.2D0
      PROB = E(G)*(1.0D0/U-1.0D0/U**3+2.2D0/U**4.9)/DSQRT(2.0D0*PI)
    7 IF (T.GT.ZERO) PROB = 1.0D0-PROB
      RETURN
      END
C
C
C
      DOUBLE PRECISION FUNCTION E(G)
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF (G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
