      PROGRAM AGREECI
      IMPLICIT REAL*8(A-H,O-Z)
C
      CHARACTER  HEAD*70,FIN*40,FOUT*40
      DIMENSION  X(500),Y(500),U(17),W(1000000)
C
C    PROGRAM DRCI PROVIDES A POINT ESTIMATE AND SIMULATED QUANTILE INTERVAL
C                     FOR THE AGREEMENT MEASURE
C
C          PRESENT SIZE LIMITS ON PROGRAM AGREECI ARE THE FOLLOWING:
C             N = 500 OBSERVED CASES & MS = 1000000 SIMULATIONS
C
      ONE = 1.0D0
      ZERO = 0.0D0
      TOLER = 1.0D-9
      WRITE(*,10) '*** PROGRAM AGREECI ***'
   10 FORMAT(T19,A)
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA:'
      READ(*,12) FIN
   12 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,12) FOUT
      WRITE(*,*) 'ENTER # OF SIMULATIONS FOR TESTING NULL MODEL:'
      READ(*,*) MS
      WRITE(*,*) 'ENTER RANDOM # GENERATOR SEED (ANY NON-ZERO INTEGER):'
      READ(*,*) ISEED
      WRITE(*,*) 'ENTER DATA DESCRIPTION:'
      READ(*,12) HEAD
      OPEN(4,FILE=FIN)
      OPEN(7,FILE=FOUT,STATUS='NEW')
C
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 100 II = 1,17
        S = 0.0D0
        T = 0.5D0
        DO 99 JJ = 1,K
          M1 = MOD(MOD(I1*J1,179)*K1,179)
          I1 = J1
          J1 = K1
          K1 = M1
          L1 = MOD(53*L1+1,169)
          IF(MOD(L1*M1,64).GE.32) S = S+T
          T = T/2
   99   CONTINUE
        U(II) = S
  100 CONTINUE
C
      READ(4,*) N
      READ(4,*) (X(I),Y(I),I=1,N)
      CLOSE(4)
      ODEL = ZERO
      EDEL = ZERO
      DO 200 I = 1,N
         ODEL = ODEL+DABS(X(I)-Y(I))
  200 CONTINUE
      ODEL = ODEL/N
      DO 300 I = 1,N
         DO 250 J = 1,N
            EDEL = EDEL+DABS(X(I)-Y(J))
  250    CONTINUE
  300 CONTINUE
      EDEL = EDEL/N/N
      AOBS = (EDEL-ODEL)/EDEL
C
      WRITE(7,400)
  400 FORMAT(T20,'*** OUTPUT OF AGREECI ANALYSIS ***',/)
      WRITE(7,450) HEAD
  450 FORMAT(T2,'DATA: ',A,/)
      WRITE(7,500) 'RANDOM # GENERATOR SEED = ',ISEED
  500 FORMAT(T6,A,I8)
      WRITE(7,500) 'NUMBER OF SIMULATIONS OF TEST STATISTIC = ',MS
      WRITE(7,550) 'OBSERVED VALUE OF DELTA STATISTIC = ',ODEL
  550 FORMAT(T6,A,G16.8)
      WRITE(7,550) 'EXPECTED VALUE OF DELTA STATISTIC = ',EDEL
      WRITE(7,550) 'OBSERVED VALUE OF AGREEMENT STATISTIC = ',AOBS
      DO 1000 J = 1,MS
         CALL SAMPWOR(N,N,U,X)
         SDEL = ZERO
         DO 600 I = 1,N
            SDEL = SDEL+DABS(X(I)-Y(I))
  600    CONTINUE
         SDEL = SDEL/N
         W(J) = (EDEL-SDEL)/EDEL
 1000 CONTINUE
C
      WRITE(7,2000)
 2000 FORMAT(T10,/,'QUANTILES OF SIMULATED AGREEMENT STATISTICS:',/)
      CALL QUANTILE(MS,W)
C
      WRITE(7,2600)
 2600 FORMAT(/,T21,'*** END OF AGREECI ANALYSIS ***')
      END
C
C
      SUBROUTINE SAMPWOR(M,N,U,X)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),X(500)
C
C         SUBROUTINE SAMPWOR DRAWS A SAMPLE OF SIZE N FROM
C            A POPULATION OF SIZE M WITHOUT REPLACEMENT
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     X: REAL INPUT WHICH CONTAINS X(1),...,X(M) POPULATION VALUES
C     UNI(U): UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 10 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         BIN = X(J)
         X(J) = X(I)
         X(I) = BIN
   10 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
C
C
      SUBROUTINE QUANTILE(N,Z)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  SELECTIONS OF QUANTILES ARE OBTAINED FOR A SET OF RESIDUAL DATA ORDER
C     TO ESTIMATE EMPIRICAL CONFIDENCE BANDS FOR A PREDICTED VALUE.
C  NOTE:  THE RESIDUAL DATA SET SIZE MUST BE AT LEAST 2
C
      DIMENSION  Z(1000000)
C
      ZERO = 0.0D0
      J = N
      K = J/2+1
   10 K = K-1
      W = Z(K)
      GO TO 30
   20 W = Z(J)
      Z(J) = Z(1)
      J = J-1
   30 II = K
   40 L = 2*II
      IF(L-J) 50,60,70
   50 IF(Z(L+1).GE.Z(L)) L = L+1
   60 IF(W.GE.Z(L)) GO TO 70
      Z(II) = Z(L)
      II = L
      GO TO 40
   70 Z(II) = W
      IF(K.GT.1) GO TO 10
      IF(J.GE.2) GO TO 20
      WRITE(7,76) '0.0-TH SAMPLE QUANTILE (MINIMUM VALUE) = ',Z(1)
   76 FORMAT(T5,A,G16.10)
      K = MAX(1,IDNINT(0.00005D0*N+0.5D0))
      WRITE(7,76) '0.00005-TH SAMPLE QUANTILE = ',Z(K)
      K = MAX(1,IDNINT(0.0005D0*N+0.5D0))
      WRITE(7,76) '0.0005-TH SAMPLE QUANTILE = ',Z(K)
      K = MAX(1,IDNINT(0.005D0*N+0.5D0))
      WRITE(7,76) '0.005-TH SAMPLE QUANTILE = ',Z(K)
      K = MAX(1,IDNINT(0.05D0*N+0.5D0))
      WRITE(7,76) '0.05-TH SAMPLE QUANTILE = ',Z(K)
      K = IDINT(0.5D0*N+0.51D0)
      Y = Z(K)
      L = N/2
      IF(K.EQ.L) Y = (Z(K)+Z(K+1))/2
      WRITE(7,78) '0.5-TH SAMPLE QUANTILE (MEDIAN VALUE) = ',Y
   78 FORMAT(/,T5,A,G16.10,/)
      K = MIN(N,IDNINT(0.95D0*N+0.5D0))
      WRITE(7,76) '0.95-TH SAMPLE QUANTILE = ',Z(K)
      K = MIN(N,IDNINT(0.995D0*N+0.5D0))
      WRITE(7,76) '0.995-TH SAMPLE QUANTILE = ',Z(K)
      K = MIN(N,IDNINT(0.9995D0*N+0.5D0))
      WRITE(7,76) '0.9995-TH SAMPLE QUANTILE = ',Z(K)
      K = MIN(N,IDNINT(0.99995D0*N+0.5D0))
      WRITE(7,76) '0.99995-TH SAMPLE QUANTILE = ',Z(K)
      WRITE(7,80) '1.0-TH SAMPLE QUANTILE (MAXIMUM VALUE) = ',Z(N)
   80 FORMAT(T5,A,G16.10)
      RETURN
      END
