      PROGRAM EMVPTMP
      IMPLICIT REAL*8(A-H,O-Z)
C
C    ONE-SAMPLE OPTION: (KW = 1)
C    THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C    EXACT P-VALUE FOR A ONE-SAMPLE MULTIVARIATE (PERMUTATION) TEST WITH
C    HYPOTHESIZED MEAN VECTOR OF THE R RESPONSES.  THE MAXIMUM VALUES
C    OF B AND R CAN BE CHANGED FOR ANY EXAMPLE.  THE  PRESENT MAXIMUM
C    VALUES OF B AND R IN THIS PROGRAM ARE RESPECTIVELY 20 AND 30.  THIS
C    PROGRAM CAN INVOKE A HOTELLING TYPE COMMENSURATION (LZ = 1 & B > R).
C    THIS PROGRAM CAN ALSO INVOKE AVERAGE PAIR-WISE DISTANCE FUNCTION
C    (D.F.) COMMENSURATION (IC = 1).
C
C    THE DATA MATRIX FOR ONE-SAMPLE OPTION MUST BE IN THE FOLLOWING SEQUENCE:
C    THE FIRST LINE IS A DESCRIPTIVE TITLE OF THE DATA
C    THE SECOND LINE ARE THE ENTRIES V, B, R, KW, LZ AND IC
C    THE THIRD LINE IS THE HYPOTHESIZED R MEAN (MEDIAN) RESPONSE VALUES
C        U(1),    U(2),    ...,U(R)
C    THE JTH OBJECT'S R RESPONSE VALUES ARE THEN ON THE (3+J)TH LINE
C    WHERE J = 1,...,B (THE # OF OBJECTS).
C
C        A(1,1,1),A(1,1,2),...,A(1,1,R)
C        A(1,2,1),A(1,2,2),...,A(1,2,R)
C                 ...
C        A(1,B,1),A(1,B,2),...,A(1,B,R)
C
C    THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C    B IS # OF BLOCKS (IN THIS CASE OBJECTS), AND R IS # OF RESPONSES.
C
C
C    MATCHED-PAIRS OPTION: (KW = 2)
C    THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED EXACT
C    P-VALUE FOR A MATCHED-PAIRS MULTIVARIATE (PERMUTATION) TEST.  THE
C    MAXIMUM VALUES OF B AND R CAN BE CHANGED FOR ANY EXAMPLE.  THE PRESENT
C    MAXIMUM VALUES OF B AND R IN THIS PROGRAM ARE RESPECTIVELY 20 AND 30.
C    THIS PROGRAM CAN INVOKE A HOTELLING TYPE COMMENSURATION (LZ = 1 & B > R).
C    THIS PROGRAM CAN ALSO INVOKE AVERAGE PAIR-WISE DISTANCE FUNCTION (D.F.)
C    COMMENSURATION (IC = 1).
C
C    THE DATA MATRIX MUST BE IN THE FOLLOWING SEQUENCE:
C    THE FIRST LINE IS A DESCRIPTIVE TITLE OF THE DATA
C    THE SECOND LINE ARE THE ENTRIES V, B, R, KW, LZ AND IC
C    THE JTH PAIR'S 1ST R RESPONSE VALUES ARE ON THE (2+J)TH LINE
C    AND THE 2ND R RESPONSE VALUES ARE ON THE (2+B+J)TH LINE WHERE
C    J = 1,...,B (THE # OF PAIRS).
C
C        A(1,1,1),A(1,1,2),...,A(1,1,R)
C                 ...
C        A(1,B,1),A(1,B,2),...,A(1,B,R)
C        A(2,1,1),A(2,1,2),...,A(2,1,R)
C                 ...
C        A(2,B,1),A(2,B,2),...,A(2,B,R)
C
C    THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C    B IS # OF BLOCKS (IN THIS CASE PAIRS), AND R IS # OF RESPONSES.
C
C    NOTE #1: KB > 3 IS REQUIRED FOR PROGRAM TO EXECUTE.
C
C    NOTE #2: ASSOCIATE B AND R WITH KB AND KR IN THIS PROGRAM.
C
C    NOTE #3: HOTELLING TYPE COMMENSURATION IS (NOT) INVOKED IF KB > KR > 1
C             AND LZ = 1 (0).
C
C    NOTE #4: AVERAGE PAIRWISE D.F. COMMENSURATION IS (NOT) INVOKED IF
C             KR > 1, LZ = 0 AND IC = 1 (0).
C
C
C    PROGRAM MODIFIED 9/30/99
C
      CHARACTER  HEAD*80,FIN*40,FOUT*40
      DIMENSION  U(30),X(30),AD(30),DIFF(30),W(30,30),Y(30,30),
     1           DT(20,30),RAW(2,20,30)
C
      ONE = 1.0D0
      ZERO = 0.0D0
      LZ = 0
      IC = 0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,1) FIN
    1 FORMAT(A)
      WRITE(*,*) 'ENTER: FILE NAME OF OUTPUT RESULTS'
      READ(*,1) FOUT
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
    2 READ(3,1,END=4) HEAD
      GO TO 5
    4 GO TO 200
    5 READ(3,*) V,KB,KR,KW,LZ,IC
      IF(KB.LT.4) STOP 'NUMBER OF BLOCKS LESS THAN FOUR'
      IF(KB.LE.KR) LZ = 0
      IF(KR.EQ.1) THEN
         LZ = 0
         IC = 0
      END IF
      IF(KW.EQ.2) GO TO 21
      READ(3,*) (U(K),K=1,KR)
      DO 7 J = 1,KB
         READ(3,*,END=170) (RAW(1,J,K),K=1,KR)
    7 CONTINUE
      WRITE(6,1) HEAD
      WRITE(6,8) V,KR,KB
    8 FORMAT('EXACT ONE-SAMPLE MULTIVARIATE PERMUTATION TEST:'
     1     /4X,'DISTANCE EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES AND ',I3,' BLOCKS.')
      IF(LZ.EQ.1) THEN
         WRITE(6,1) 'HOTELLING TYPE COMMENSURATION IS INVOKED'
         IC = 0
         GO TO 14
      END IF
      DO 12 J = 1,KB
         DO 10 K = 1,KR
            DIFF(K) = RAW(1,J,K)-U(K)
   10    CONTINUE
         DO 11 K = 1,KR
            DT(J,K) = DIFF(K)
   11    CONTINUE
   12 CONTINUE
      GO TO 30
   14 DO 20 J = 1,KB
         DO 16 K = 1,KR
            DIFF(K) = RAW(1,J,K)-U(K)
   16    CONTINUE
         DO 18 K = 1,KR
            RAW(1,J,K) = DIFF(K)
   18    CONTINUE
   20 CONTINUE
      GO TO 65
   21 DO 23 I = 1,2
         DO 22 J = 1,KB
            READ(3,*,END=170) (RAW(I,J,K),K=1,KR)
   22    CONTINUE
   23 CONTINUE
      WRITE(6,1) HEAD
      WRITE(6,25) V,KR,KB
   25 FORMAT('EXACT MATCHED-PAIRS MULTIVARIATE PERMUTATION TEST:'
     1     /4X,'DISTANCE EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES AND ',I3,' BLOCKS.')
      IF(LZ.EQ.1) THEN
         WRITE(6,1) 'HOTELLING TYPE COMMENSURATION IS INVOKED'
         IC = 0
         GO TO 54
      END IF
      DO 28 J = 1,KB
         DO 26 K = 1,KR
            DIFF(K) = RAW(1,J,K)-RAW(2,J,K)
   26    CONTINUE
         DO 27 K = 1,KR
            DT(J,K) = DIFF(K)
   27    CONTINUE
   28 CONTINUE
   30 IF(IC.EQ.0) GO TO 165
      WRITE(6,1) 'AVERAGE D.F. COMMENSURATION IS INVOKED'
      DO 35 K = 1,KR
         AD(K) = ZERO
         DO 34 J1 = 2,KB
            DO 33 J2 = 1,J1-1
               AD(K) = AD(K)+((DABS(DT(J1,K)-DT(J2,K)))**V
     1                 +(DABS(DT(J1,K)+DT(J2,K)))**V)*2.0D0
   33       CONTINUE
   34    CONTINUE
         AD(K) = AD(K)**(ONE/V)
   35 CONTINUE
      DO 45 J = 1,KB
         DO 44 K = 1,KR
            DT(J,K) = DT(J,K)/AD(K)
   44    CONTINUE
   45 CONTINUE
      GO TO 165
   54 DO 60 J = 1,KB
         DO 56 K = 1,KR
            DIFF(K) = RAW(1,J,K)-RAW(2,J,K)
   56    CONTINUE
         DO 58 K = 1,KR
            RAW(1,J,K) = DIFF(K)
   58    CONTINUE
   60 CONTINUE
   65   DO 70 I = 1,KR
         X(I) = ZERO
         DO 69 J = 1,KB
            X(I) = X(I)+DABS(RAW(1,J,I))
   69    CONTINUE
         X(I) = X(I)/KB
   70 CONTINUE
      DO 80 I = 1,KR
         DO 79 J = I,KR
            W(I,J) = ZERO
            DO 78 K = 1,KB
               W(I,J) = W(I,J)+(DABS(RAW(1,K,I))-X(I))
     1                        *(DABS(RAW(1,K,J))-X(J))
   78       CONTINUE
            W(I,J) = W(I,J)/KB
            W(J,I) = W(I,J)
   79    CONTINUE
   80 CONTINUE
      N = KR
      CALL INVERT(N,W)
      Y(1,1) = DSQRT(W(1,1))
      DO 90 I = 2,KR
         Y(1,I) = W(1,I)/Y(1,1)
         Y(I,1) = ZERO
   90 CONTINUE
      DO 130 I = 2,KR
         Y(I,I) = W(I,I)
         DO 100 J = 1,I-1
            Y(I,I) = Y(I,I)-Y(J,I)*Y(J,I)
  100    CONTINUE
         Y(I,I) = DSQRT(Y(I,I))
         DO 120 J = I+1,KR
            Y(I,J) = W(I,J)
            Y(J,I) = ZERO
            DO 110 K = 1,I-1
               Y(I,J) = Y(I,J)-Y(K,I)*Y(K,J)
  110       CONTINUE
            Y(I,J) = Y(I,J)/Y(I,I)
  120    CONTINUE
  130 CONTINUE
      DO 160 I = 1,KB
         DO 150 J = 1,KR
            DT(I,J) = ZERO
            DO 140 K = J,KR
               DT(I,J) = DT(I,J)+Y(J,K)*RAW(1,I,K)
  140       CONTINUE
  150    CONTINUE
  160 CONTINUE
  165 CALL COMB(V,KB,KR,DT)
      GO TO 2
  170 WRITE(*,180)
  180 FORMAT(' E R R O R: END-OF-FILE ENCOUNTERED.')
      WRITE(*,190) KB,KR
  190 FORMAT(' CHECK THESE:  B = ',I3,' R = ',I3)
  200 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE INVERT(N,W)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION KP(30),KS(30),KC(30),W(30,30)
C
      ZERO = 0.0D0
      DO 5 I = 1,N
         KP(I) = 0
    5 CONTINUE
      DO 45 I = 1,N
         T = ZERO
         DO 20 J = 1,N
            IF(KP(J).NE.1) THEN
               DO 15 K = 1,N
                  IF(KP(K)-1) 10,15,60
   10             IF(T.LT.DABS(W(J,K))) THEN
                     IS = J
                     IC = K
                     T = DABS(W(J,K))
                  END IF
   15          CONTINUE
            END IF
   20    CONTINUE
         KP(IC) = KP(IC)+1
         IF(IS.NE.IC) THEN
            DO 25 L = 1,N
               T = W(IS,L)
               W(IS,L) = W(IC,L)
               W(IC,L) = T
   25       CONTINUE
         END IF
         KS(I) = IS
         KC(I) = IC
         T = W(IC,IC)
         W(IC,IC) = 1.0D0
         DO 30 L = 1,N
            W(IC,L) = W(IC,L)/T
   30    CONTINUE
         DO 40 M = 1,N
            IF(M.NE.IC) THEN
               T = W(M,IC)
               W(M,IC) = ZERO
               DO 35 L = 1,N
                  W(M,L) = W(M,L)-W(IC,L)*T
   35          CONTINUE
            END IF
   40    CONTINUE
   45 CONTINUE
      DO 55 I = 1,N
         J = N-I+1
         IF(KS(J).NE.KC(J)) THEN
            K = KS(J)
            L = KC(J)
            DO 50 M = 1,N
               T = W(M,K)
               W(M,K) = W(M,L)
               W(M,L) = T
   50       CONTINUE
         END IF
   55 CONTINUE
   60 RETURN
      END
C
C
      SUBROUTINE COMB(V,KB,KR,DT)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION M(20),DT(20,30)
C
      ONE = 1.0D0
      DO 10 I = 1,KB
         M(I) = 1
   10 CONTINUE
      KOUNT1 = 0
      KOUNT2 = 0
      KOUNT3 = 0
      NMINUS = 0
      LAST = 0
      GO TO 50
   20 J = 0
   30 J = J+1
      IF(MOD(KOUNT3,2).EQ.1) GO TO 40
      KOUNT3 = KOUNT3/2
      GO TO 30
   40 LAST = M(J)
      M(J) = -LAST
   50 KOUNT1 = KOUNT1+1
      KOUNT3 = KOUNT1
      NMINUS = NMINUS+LAST
      CALL XACT(V,KB,KR,KOUNT1,KOUNT2,DELTA1,M,DT)
      IF(NMINUS.EQ.1.AND.M(KB).EQ.-1) GO TO 60
      GO TO 20
   60 PROB = ONE*KOUNT2/KOUNT1
      WRITE(6,70) 'OBSERVED DELTA =', DELTA1
   70 FORMAT(T3,A,G20.8)
      WRITE(6,80) 'NUMBER OF DELTA VALUES  OBSERVED DELTA =', KOUNT2
   80 FORMAT(T3,A,I8)
      WRITE(6,80) 'TOTAL NUMBER OF DELTA VALUES =', KOUNT1
      WRITE(6,70) 'P-VALUE OF A DELTA  OBSERVED DELTA =', PROB
      WRITE(6,90)
   90 FORMAT('********************************************************')
      RETURN
      END
C
C
      SUBROUTINE XACT(V,KB,KR,KOUNT1,KOUNT2,DELTA1,M,DT)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION M(20),DT(20,30)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      B = ONE*KB
      PREC = 1.0D-12
      SUM = ZERO
      DO 30 I = 1,KB-1
         DO 20 J = I+1,KB
            PSUM = ZERO
            DO 10 K = 1,KR
               PSUM = PSUM+(DT(I,K)*M(I)-DT(J,K)*M(J))**2
   10       CONTINUE
            SUM = SUM+PSUM**(V/2)
   20    CONTINUE
   30 CONTINUE
      DELTA = SUM*2/(B*(B-1))
      IF(KOUNT1.EQ.1) DELTA1 = DELTA*(ONE+PREC)
      IF(DELTA.LE.DELTA1) KOUNT2 = KOUNT2+1
      RETURN
      END
