      PROGRAM MVPTMP
      IMPLICIT REAL*8(A-H,O-Z)
C
C    ONE-SAMPLE OPTION: (KW = 1)
C    THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C    P-VALUE FOR A ONE-SAMPLE MULTIVARIATE (PERMUTATION) TEST WITH
C    HYPOTHESIZED MEAN VECTOR OF THE R RESPONSES.  AN AGREEMENT MEASURE
C    CORRESPONDING TO A PEARSON TYPE CORRELATION COEFFICIENT IS ALSO
C    PROVIDED.  THE MAXIMUM VALUES OF B AND R CAN BE CHANGED FOR ANY
C    EXAMPLE.  THE PRESENT MAXIMUM VALUES OF B AND R IN THIS PROGRAM
C    ARE RESPECTIVELY 20 AND 15.  THIS PROGRAM CAN INVOKE A HOTELLING
C    TYPE COMMENSURATION (LZ = 1 & B > R).  THIS PROGRAM CAN ALSO INVOKE
C    AVERAGE PAIR-WISE DISTANCE FUNCTION (D.F.) COMMENSURATION (IC = 1).
C
C    THE DATA MATRIX FOR ONE-SAMPLE OPTION MUST BE IN THE FOLLOWING SEQUENCE:
C    THE FIRST LINE IS A DESCRIPTIVE TITLE OF THE DATA
C    THE SECOND LINE ARE THE ENTRIES V, B, R, KW, LZ AND IC
C    THE THIRD LINE IS THE HYPOTHESIZED R MEAN (MEDIAN) RESPONSE VALUES
C        U(1),    U(2),    ...,U(R)
C    THE JTH OBJECT'S R RESPONSE VALUES ARE THEN ON THE (3+J)TH LINE
C    WHERE J = 1,...,B (THE # OF OBJECTS).
C
C        A(1,1,1),A(1,1,2),...,A(1,1,R)
C        A(1,2,1),A(1,2,2),...,A(1,2,R)
C                 ...
C        A(1,B,1),A(1,B,2),...,A(1,B,R)
C
C    THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C    B IS # OF BLOCKS (IN THIS CASE OBJECTS), AND R IS # OF RESPONSES.
C
C
C    MATCHED-PAIRS OPTION: (KW = 2)
C    THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C    P-VALUE FOR A MATCHED-PAIRS MULTIVARIATE (PERMUTATION) PROCEDURES.
C    AN AGREEMENT MEASURE CORRESPONDING TO A PEARSON TYPE CORRELATION
C    COEFFICIENT IS ALSO PROVIDED.  THE MAXIMUM VALUES OF B AND R CAN
C    BE CHANGED FOR ANY EXAMPLE.  THE PRESENT MAXIMUM VALUES OF B AND R
C    IN THIS PROGRAM ARE RESPECTIVELY 20 AND 15.  THIS PROGRAM CAN INVOKE
C    A HOTELLING TYPE COMMENSURATION (LZ = 1 & B > R).  THIS PROGRAM CAN
C    ALSO INVOKE AVERAGE PAIR-WISE DISTANCE FUNCTION (D.F.) COMMENSURATION
C    (IC = 1).
C
C    THE DATA MATRIX MUST BE IN THE FOLLOWING SEQUENCE:
C    THE FIRST LINE IS A DESCRIPTIVE TITLE OF THE DATA
C    THE SECOND LINE ARE THE ENTRIES V, B, R, KW, LZ AND IC
C    THE JTH PAIR'S 1ST R RESPONSE VALUES ARE ON THE (2+J)TH LINE
C    AND THE 2ND R RESPONSE VALUES ARE ON THE (2+B+J)TH LINE WHERE
C    J = 1,...,B (THE # OF PAIRS).
C
C        A(1,1,1),A(1,1,2),...,A(1,1,R)
C                 ...
C        A(1,B,1),A(1,B,2),...,A(1,B,R)
C        A(2,1,1),A(2,1,2),...,A(2,1,R)
C                 ...
C        A(2,B,1),A(2,B,2),...,A(2,B,R)
C
C    THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C    B IS # OF BLOCKS (IN THIS CASE PAIRS), AND R IS # OF RESPONSES.
C
C    NOTE #1: KB > 3 IS REQUIRED FOR PROGRAM TO EXECUTE.
C
C    NOTE #2: ASSOCIATE B AND R WITH KB AND KR IN THIS PROGRAM.
C
C    NOTE #3: HOTELLING TYPE COMMENSURATION IS (NOT) INVOKED IF KB > KR > 1
C             AND LZ = 1 (0).
C
C    NOTE #4: AVERAGE PAIRWISE D.F. COMMENSURATION IS (NOT) INVOKED IF
C             KR > 1, LZ = 0 AND IC = 1 (0).
C
C    NOTE #5: PROGRAM MVPTMP IS CAPABLE OF MULTIPLE RUNS.  HOWEVER, LEAVE NO
C             BLANK LINES BETWEEN DATA SETS SINCE THE FIRST LINE FOLLOWING
C             THE LAST DATA ENTRY MAY BE READ AS A BLANK PROBLEM HEAD FOR THE
C             NEXT DATA SET.
C
C    PROGRAM MODIFIED 4/25/01
C
      CHARACTER  FIN*40,FOUT*40,HEAD*80
      DIMENSION U(15),X(15),AD(15),DIFF(15),Y(15,15),W(15,15),
     1    DT(2,20,15),RAW(2,20,15)
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,1) FIN
    1 FORMAT(A)
      WRITE(*,*) 'ENTER: FILE NAME OF OUTPUT RESULTS'
      READ(*,1) FOUT
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
    2 READ(3,1,END=4) HEAD
      GO TO 5
    4 GO TO 200
    5 READ(3,*) V,KB,KR,KW,LZ,IC
      IF(KB.LT.4) STOP 'NUMBER OF BLOCKS LESS THAN FOUR'
      IF(KB.LE.KR) LZ = 0
      IF(KR.EQ.1) THEN
         LZ = 0
         IC = 0
      END IF
      IF(KW.EQ.2) GO TO 21
      READ(3,*) (U(K),K=1,KR)
      DO 7 J = 1,KB
         READ(3,*,END=170) (RAW(1,J,K),K=1,KR)
    7 CONTINUE
      WRITE(6,1) HEAD
      WRITE(6,8) V,KR,KB
    8 FORMAT('ONE-SAMPLE MULTIVARIATE PERMUTATION TEST:'
     1     /4X,'DISTANCE EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES AND ',I3,' BLOCKS.')
      IF(LZ.EQ.1) THEN
         WRITE(6,1) 'HOTELLING TYPE COMMENSURATION IS INVOKED'
         IC = 0
         GO TO 14
      END IF
      DO 12 J = 1,KB
         DO 10 K = 1,KR
            DIFF(K) = RAW(1,J,K)-U(K)
   10    CONTINUE
         DO 11 K = 1,KR
            DT(1,J,K) = DIFF(K)
            DT(2,J,K) = ZERO-DIFF(K)
   11    CONTINUE
   12 CONTINUE
      GO TO 30
   14 DO 20 J = 1,KB
         DO 16 K = 1,KR
            DIFF(K) = RAW(1,J,K)-U(K)
   16    CONTINUE
         DO 18 K = 1,KR
            RAW(1,J,K) = DIFF(K)
   18    CONTINUE
   20 CONTINUE
      GO TO 65
   21 DO 23 I = 1,2
         DO 22 J = 1,KB
            READ(3,*,END=170) (RAW(I,J,K),K=1,KR)
   22    CONTINUE
   23 CONTINUE
      WRITE(6,1) HEAD
      WRITE(6,25) V,KR,KB
   25 FORMAT('MATCHED-PAIRS MULTIVARIATE PERMUTATION TEST:'
     1     /4X,'DISTANCE EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES AND ',I3,' BLOCKS.')
      IF(LZ.EQ.1) THEN
         WRITE(6,1) 'HOTELLING TYPE COMMENSURATION IS INVOKED'
         IC = 0
         GO TO 54
      END IF
      DO 28 J = 1,KB
         DO 26 K = 1,KR
            DIFF(K) = RAW(1,J,K)-RAW(2,J,K)
   26    CONTINUE
         DO 27 K = 1,KR
            DT(1,J,K) = DIFF(K)
            DT(2,J,K) = ZERO-DIFF(K)
   27    CONTINUE
   28 CONTINUE
   30 IF(IC.EQ.0) GO TO 165
      WRITE(6,1) 'AVERAGE D.F. COMMENSURATION IS INVOKED'
      DO 38 K = 1,KR
        AD(K) = ZERO
        DO 37 J1 = 2,KB
          DO 36 J2 = 1,J1-1
            AD(K) = AD(K)+((DABS(DT(1,J1,K)-DT(1,J2,K)))**V
     1              +(DABS(DT(1,J1,K)+DT(1,J2,K)))**V)*2.0D0
   36     CONTINUE
   37   CONTINUE
        AD(K) = AD(K)**(ONE/V)
   38 CONTINUE
      DO 46 I = 1,2
         DO 45 J = 1,KB
            DO 44 K = 1,KR
               DT(I,J,K) = DT(I,J,K)/AD(K)
   44       CONTINUE
   45    CONTINUE
   46 CONTINUE
      GO TO 165
   54 DO 60 J = 1,KB
         DO 56 K = 1,KR
            DIFF(K) = RAW(1,J,K)-RAW(2,J,K)
   56    CONTINUE
         DO 58 K = 1,KR
            RAW(1,J,K) = DIFF(K)
   58    CONTINUE
   60 CONTINUE
   65 DO 70 I = 1,KR
         X(I) = ZERO
         DO 69 J = 1,KB
            X(I) = X(I)+DABS(RAW(1,J,I))
   69    CONTINUE
         X(I) = X(I)/KB
   70 CONTINUE
      DO 80 I = 1,KR
         DO 79 J = I,KR
            W(I,J) = ZERO
            DO 78 K = 1,KB
               W(I,J) = W(I,J)+(DABS(RAW(1,K,I))-X(I))
     1                        *(DABS(RAW(1,K,J))-X(J))
   78       CONTINUE
            W(I,J) = W(I,J)/KB
            W(J,I) = W(I,J)
   79    CONTINUE
   80 CONTINUE
      N = KR
      CALL INVERT(N,W)
      Y(1,1) = DSQRT(W(1,1))
      DO 90 I = 2,KR
         Y(1,I) = W(1,I)/Y(1,1)
         Y(I,1) = ZERO
   90 CONTINUE
      DO 130 I = 2,KR
         Y(I,I) = W(I,I)
         DO 100 J = 1,I-1
            Y(I,I) = Y(I,I)-Y(J,I)*Y(J,I)
  100    CONTINUE
         Y(I,I) = DSQRT(Y(I,I))
         DO 120 J = I+1,KR
            Y(I,J) = W(I,J)
            Y(J,I) = ZERO
            DO 110 K = 1,I-1
               Y(I,J) = Y(I,J)-Y(K,I)*Y(K,J)
  110       CONTINUE
            Y(I,J) = Y(I,J)/Y(I,I)
  120    CONTINUE
  130 CONTINUE
      DO 160 I = 1,KB
         DO 150 J = 1,KR
            DT(1,I,J) = ZERO
            DO 140 K = J,KR
               DT(1,I,J) = DT(1,I,J)+Y(J,K)*RAW(1,I,K)
  140       CONTINUE
            DT(2,I,J) = ZERO-DT(1,I,J)
  150    CONTINUE
  160 CONTINUE
  165 CALL CALC(V,KB,KR,KW,DT)
      GO TO 2
  170 WRITE(*,180)
  180 FORMAT(' E R R O R: END-OF-FILE ENCOUNTERED.')
      WRITE(*,190) KB,KR
  190 FORMAT(' CHECK THESE:  B = ',I3,' R = ',I3)
  200 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE INVERT(N,W)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION KP(15),KS(15),KC(15),W(15,15)
C
      ZERO = 0.0D0
      DO 5 I = 1,N
         KP(I) = 0
    5 CONTINUE
      DO 45 I = 1,N
         T = ZERO
         DO 20 J = 1,N
            IF(KP(J).NE.1) THEN
               DO 15 K = 1,N
                  IF(KP(K)-1) 10,15,60
   10             IF(T.LT.DABS(W(J,K))) THEN
                     IS = J
                     IC = K
                     T = DABS(W(J,K))
                  END IF
   15          CONTINUE
            END IF
   20    CONTINUE
         KP(IC) = KP(IC)+1
         IF(IS.NE.IC) THEN
            DO 25 L = 1,N
               T = W(IS,L)
               W(IS,L) = W(IC,L)
               W(IC,L) = T
   25       CONTINUE
         END IF
         KS(I) = IS
         KC(I) = IC
         T = W(IC,IC)
         W(IC,IC) = 1.0D0
         DO 30 L = 1,N
            W(IC,L) = W(IC,L)/T
   30    CONTINUE
         DO 40 M = 1,N
            IF(M.NE.IC) THEN
               T = W(M,IC)
               W(M,IC) = ZERO
               DO 35 L = 1,N
                  W(M,L) = W(M,L)-W(IC,L)*T
   35          CONTINUE
            END IF
   40    CONTINUE
   45 CONTINUE
      DO 55 I = 1,N
         J = N-I+1
         IF(KS(J).NE.KC(J)) THEN
            K = KS(J)
            L = KC(J)
            DO 50 M = 1,N
               T = W(M,K)
               W(M,K) = W(M,L)
               W(M,L) = T
   50       CONTINUE
         END IF
   55 CONTINUE
   60 RETURN
      END
C
C
      SUBROUTINE CALC(V,KB,KR,KW,DT)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION D(40,40),SIJ(20,20),SIJ2(20,20),TIJ2(20,20),
     1    DT(2,20,15),SJ(2,20,20),SJ2(2,20,20),WI(20,20,20),
     2    YIJ(20,20,20),ZIJK(20,20,20)
C
      ONE=1.0D0
      ZERO=0.0D0
      DO 10 I=1,2
         DO 9 J=1,KB
            DO 8 K=I,2
               LO=1
               IF(I.EQ.K) LO=J
               DO 7 L=LO,KB
                  IJ=KB*(I-1)+J
                  KL=KB*(K-1)+L
                  D(IJ,KL)=ZERO
                  DO 6 M=1,KR
                     D(IJ,KL)=D(IJ,KL)+(DT(I,J,M)-DT(K,L,M))**2
    6             CONTINUE
                  D(IJ,KL)=D(IJ,KL)**(V/2)
                  D(KL,IJ)=D(IJ,KL)
    7          CONTINUE
    8       CONTINUE
    9    CONTINUE
   10 CONTINUE
      DO 30 IS=1,KB
         DO 29 IR=1,KB
            IF(IR.EQ.IS) GO TO 28
               SIJ(IR,IS)=ZERO
               SIJ2(IR,IS)=ZERO
               DO 27 I=1,2
                  SJ(I,IR,IS)=ZERO
                  SJ2(I,IR,IS)=ZERO
                  DO 26 J=1,2
                     IRR=(I-1)*KB+IR
                     JSS=(J-1)*KB+IS
                     SJ(I,IR,IS)=SJ(I,IR,IS)+D(IRR,JSS)
                     SJ2(I,IR,IS)=SJ2(I,IR,IS)+D(IRR,JSS)**2
   26             CONTINUE
                  SIJ(IR,IS)=SIJ(IR,IS)+SJ(I,IR,IS)
                  SIJ2(IR,IS)=SIJ2(IR,IS)+SJ2(I,IR,IS)
   27          CONTINUE
   28       CONTINUE
   29    CONTINUE
   30 CONTINUE
      T2=ZERO
      IF(KB.LE.2) GO TO 51
      DO 50 IT=3,KB
         IT1=IT-1
         DO 49 IS=2,IT1
            IS1=IS-1
            DO 48 IR=1,IS1
               WI(IR,IS,IT)=ZERO
               WI(IS,IR,IT)=ZERO
               WI(IT,IR,IS)=ZERO
               YIJ(IR,IS,IT)=ZERO
               ZIJK(IR,IS,IT)=ZERO
               DO 47 I=1,2
                  WI(IR,IS,IT)=WI(IR,IS,IT)+SJ(I,IR,IS)*SJ(I,IR,IT)
                  WI(IS,IR,IT)=WI(IS,IR,IT)+SJ(I,IS,IR)*SJ(I,IS,IT)
                  WI(IT,IR,IS)=WI(IT,IR,IS)+SJ(I,IT,IR)*SJ(I,IT,IS)
                  DO 46 J=1,2
                     IRR=KB*(I-1)+IR
                     JSS=KB*(J-1)+IS
                     JTT=KB*(J-1)+IT
                     ISS=KB*(I-1)+IS
                     YIJ(IR,IS,IT)=YIJ(IR,IS,IT)+
     1                 D(IRR,JSS)*SJ(I,IR,IT)*SJ(J,IS,IT)+
     2                 D(IRR,JTT)*SJ(I,IR,IS)*SJ(J,IT,IS)+
     3                 D(ISS,JTT)*SJ(I,IS,IR)*SJ(J,IT,IR)
                     DO 45 K=1,2
                        KTT=(K-1)*KB+IT
                        ZIJK(IR,IS,IT)=ZIJK(IR,IS,IT)+
     1                    D(IRR,JSS)*D(IRR,KTT)*D(JSS,KTT)
   45                CONTINUE
   46             CONTINUE
   47          CONTINUE
               T2=T2+SIJ(IR,IS)*SIJ(IR,IT)*SIJ(IS,IT)-
     1           SIJ(IS,IT)*WI(IR,IS,IT)*2-
     2           SIJ(IR,IT)*WI(IS,IR,IT)*2-
     3           SIJ(IR,IS)*WI(IT,IR,IS)*2+
     4           YIJ(IR,IS,IT)*4-ZIJK(IR,IS,IT)*8
   48       CONTINUE
   49    CONTINUE
   50 CONTINUE
      T2=T2*6
   51 DO 100 IS=2,KB
         IS1=IS-1
         DO 99 IR=1,IS1
            TIJ2(IR,IS)=ZERO
            DO 98 I=1,2
               TIJ2(IR,IS)=TIJ2(IR,IS)+SJ(I,IR,IS)**2+
     1           SJ(I,IS,IR)**2
   98       CONTINUE
   99    CONTINUE
  100 CONTINUE
      EDEL=ZERO
      VAR=ZERO
      DO 200 IS=2,KB
         IS1=IS-1
         DO 199 IR=1,IS1
            EDEL=EDEL+SIJ(IR,IS)
            VAR=VAR+SIJ(IR,IS)*SIJ(IR,IS)-TIJ2(IR,IS)*2+SIJ2(IR,IS)*4
  199    CONTINUE
  200 CONTINUE
      C0=ONE*KB*(KB-1)
      C1=ONE/C0/2
      EDEL=C1*EDEL
      VAR=VAR*C1*C1
      GAM=-C1*C1*C1*T2
      GAM=GAM/DSQRT(VAR**3)
      DELTA=ZERO
      DO 300 IS=2,KB
         IS1=IS-1
         DO 299 IR=1,IS1
            DO 298 I=1,2
               IRR=(I-1)*KB+IR
               ISS=(I-1)*KB+IS
               DELTA=DELTA+D(IRR,ISS)
  298       CONTINUE
  299    CONTINUE
  300 CONTINUE
      DELTA=DELTA/C0
      T=(DELTA-EDEL)/DSQRT(VAR)
      RHO=ONE-(DELTA/EDEL)
      IF(KW.EQ.2) GO TO 570
      WRITE(6,560)
  560 FORMAT(/'RESULTS OF MULTIVARIATE ONE-SAMPLE PERMUTATION TEST:')
      GO TO 590
  570 WRITE(6,580)
  580 FORMAT(/'RESULTS OF MULTIVARIATE MATCHED-PAIRS PERMUTATION TEST:')
  590 WRITE(6,600) DELTA
  600 FORMAT(11X,'DELTA(D)',T29,':  ',G16.7)
      WRITE(6,610) EDEL
  610 FORMAT(11X,'EXPECTED DELTA',T29,':  ',G16.7)
      WRITE(6,620) VAR
  620 FORMAT(11X,'VARIANCE OF DELTA',T29,':  ',G16.7)
      WRITE(6,630) GAM
  630 FORMAT(11X,'SKEWNESS OF DELTA',T29,':   ',G16.7)
      WRITE(6,640) RHO
  640 FORMAT(11X,'AGREEMENT MEASURE',T29,':  ',G16.7)
      WRITE(6,650) T
  650 FORMAT(11X,'(D-E[D])/STDEV(D)',T29,':  ',G16.7)
      CALL PVALUE(T,GAM,PROB)
      WRITE(6,670) PROB
  670 FORMAT(11X,'P-VALUE',T29,':  ',G16.7)
      WRITE(6,680)
  680 FORMAT('********************************************************')
      RETURN
      END
C
C
      SUBROUTINE PVALUE(T,GAM,PROB)
      IMPLICIT REAL*8(A-H,O-Z)
C
C         SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C         T IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF T.
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      IF(DABS(GAM).LT.0.01D0) GO TO 50
      R = ONE*2/DABS(GAM)
      D = R*R
      F = R*R+10
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      U = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)
     +        -ONE/(F*F*F*360)
      A = R*R-1
      B = R*R*(DLOG(R)-1)-U
      G1 = 0.0375D0
      G2 = 0.075D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      W = -1.99D0/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF(T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+15
      DO 20 I = 1,199
         PHI = A*DLOG(R+X+G1*(2*I-1))-R*(X+G1*(2*I-1))+B
         H1 = H1+E(PHI)
         PHI = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2 = H2+E(PHI)
   20 CONTINUE
      PHI = A*DLOG(R+X+G1*399)-R*(X+G1*399)+B
      H1N = E(PHI)
      PHI = A*DLOG(R+X)-R*X+B
      H0 = E(PHI)
      PHI = A*DLOG(R+Y)-R*Y+B
      H3 = E(PHI)
      PROB = ONE-G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   30 IF(T.GT.W) THEN
         PROB = ONE
         RETURN
      END IF
      X = T-15
      Y = T
      DO 40 I = 1,199
         PHI = A*DLOG(R-X-G1*(2*I-1))+R*(X+G1*(2*I-1))+B
         H1 = H1+E(PHI)
         PHI = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2 = H2+E(PHI)
   40 CONTINUE
      PHI = A*DLOG(R-X-G1*399)+R*(X+G1*399)+B
      H1N = E(PHI)
      PHI = A*DLOG(R-X)+R*X+B
      H0 = E(PHI)
      PHI = A*DLOG(R-Y)+R*Y+B
      H3 = E(PHI)
      PROB = G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   50 PROB = ZERO
      G = -T*T/2
      IF(G.LT.-709.1D0) GO TO 70
      A = 4.2D0
      U = DABS(T)
      IF(U.GT.A) GO TO 60
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 70
   60 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   70 IF(T.GT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
      IMPLICIT REAL*8(A-H,O-Z)
C
      E = 0.0D0
      IF(G.GE.-709.1D0) E = DEXP(G)
      RETURN
      END
