      PROGRAM RMRBP
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C         P-VALUE FOR AN ANALYSIS OF A RANDOMIZED BLOCK EXPERIMENT (MRBP3).
C         THE CORRESPONDENCE BETWEEN A CORRELATION ANALYSIS AND A
C         RANDOMIZED BLOCK EXPERIMENT CAN BE USED TO GET A CORRELATION
C         COEFFICIENT AS WELL. THE MAXIMUM VALUES OF G, B AND R CAN BE
C         CHANGED FOR ANY EXAMPLE.  THE PRESENT MAXIMUM VALUES OF G, B
C         AND R IN THIS PROGRAM ARE RESPECTIVELY 10, 12 AND 15.
C
C       THIS PROGRAM IS CAPABLE OF PERFORMING (1) REPEATED MRBP ANALYSES
C       IN ONE OPERATION, (2) ALIGNMENT WITHIN BLOCKS, (3) DISTANCE
C       FUNCTION COMMENSURATION, AND (4) C(G,H) RANKS TEST.
C
C         PROGRAM MODIFIED 2/8/2003
C
C       THE DATA MATRIX MUST BE IN THE FOLLOWING SEQUENCE WITH EACH
C       OBJECT'S R RESPONSE VALUES ON A SEPARATE LINE AS FOLLOWS:
C
C         A(1,1,1),A(1,1,2),...,A(1,1,R)
C         A(1,2,1),A(1,2,2),...,A(1,2,R)
C                  ...
C         A(1,B,1),A(1,B,2),...,A(1,B,R)
C         A(2,1,1),A(2,1,2),...,A(2,1,R)
C                  ...
C         A(2,B,1),A(2,B,2),...,A(2,B,R)
C                  ...
C         A(G,1,1),A(G,1,2),...,A(G,1,R)
C                  ...
C         A(G,B,1),A(G,B,2),...,A(G,B,R)
C
C   THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C   G IS # OF GROUPS, B IS # OF BLOCKS, R IS # OF RESPONSES, IA = 1 IMPLIES
C   ALIGNMENT, IC = 1 IMPLIES COMMENSURATION, AND LR = 1 IMPLIES C(G,H)
C   RANKS TEST.   NOTE: ASSOCIATE G, B AND R WITH KG, KB AND KR IN PROGRAM.
C
      CHARACTER HEAD*80,FIN*40,FOUT*40
      DIMENSION U(17),AD(15),XM(12,15),DATA(10,12,15),X(10,12,15)
C
      ONE=1.0D0
      ZERO=0.0D0
      WRITE(*,*) 'ENTER FILE NAME OF INPUT DATA'
      READ(*,'(A)') FIN
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,'(A)') FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 2 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 1 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
    1    CONTINUE
         U(II) = S
    2 CONTINUE
      OPEN(5,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
    4 READ(5,'(A)',END=5) HEAD
      GO TO 10
    5 GO TO 100
   10 READ(5,*) V,KG,KB,KR,IA,IC,LR,H
      DO 20 I=1,KG
         DO 15 J=1,KB
            READ(5,*,END=75) (DATA(I,J,K),K=1,KR)
   15    CONTINUE
   20 CONTINUE
      WRITE(6,21) HEAD,V,KR,KB,KG
   21 FORMAT(' ANALYSIS OF RANDOMIZED BLOCK EXPERIMENT (MRBP): '/
     1     4X,A/4X,'DISTANCE FUNCTION EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES, ',I3,' BLOCKS, ',I3,' GROUPS.')
      IF(LR.EQ.1) THEN
         IA = 0
         IC = 0
         CALL RANK(KG,KB,KR,H,DATA)
         GO TO 60
      END IF
      IF(IA.EQ.0) GO TO 40
      DO 25 I=1,KG
         DO 24 J=1,KB
            DO 23 K=1,KR
               X(I,J,K)=DATA(I,J,K)
   23       CONTINUE
   24    CONTINUE
   25 CONTINUE
      DO 30 J=1,KB
         DO 29 K=1,KR
            A1=1.0D200
            SUM1=A1
            DO 28 I1=1,KG
               SUM=ZERO
               DO 27 I2=1,KG
                  SUM=SUM+DABS(DATA(I2,J,K)-X(I1,J,K))
   27          CONTINUE
               IF(SUM.LT.A1) THEN
                  DM1=X(I1,J,K)
                  A1=SUM
                  A2=A1*1.0000000001D0
               END IF
               IF(SUM.LT.A2.AND.DM1.NE.X(I1,J,K)) THEN
                  DM2=X(I1,J,K)
                  SUM1=SUM
               END IF
   28       CONTINUE
            IF(SUM1.GT.A2) DM2=DM1
            XM(J,K)=(DM1+DM2)/2
   29    CONTINUE
   30 CONTINUE
      DO 35 I=1,KG
         DO 34 J=1,KB
            DO 33 K=1,KR
               DATA(I,J,K)=DATA(I,J,K)-XM(J,K)
   33       CONTINUE
   34    CONTINUE
   35 CONTINUE
   40 IF(IC.EQ.0) GO TO 60
      IF(KR.EQ.1) GO TO 60
      DO 45 K=1,KR
         AD(K)=ZERO
         DO 44 I1=1,KG
            DO 43 I2=1,KG
               DO 42 J1=2,KB
                  DO 41 J2=1,J1-1
                     AD(K)=AD(K)+(DABS(DATA(I1,J1,K)-DATA(I2,J2,K)))**V
   41             CONTINUE
   42          CONTINUE
   43       CONTINUE
   44    CONTINUE
         AD(K)=AD(K)**(ONE/V)
   45 CONTINUE
      DO 50 I=1,KG
         DO 49 J=1,KB
            DO 48 K=1,KR
               DATA(I,J,K)=DATA(I,J,K)/AD(K)
   48       CONTINUE
   49    CONTINUE
   50 CONTINUE
   60 CALL CALC(V,KG,KB,KR,IA,IC,LR,H,ISEED,MS,U,DATA)
      GO TO 4
   75 WRITE(6,80)
   80 FORMAT(' E R R O R:  END-OF-FILE ENCOUNTERED.')
      WRITE(6,90) KG,KB,KR
   90 FORMAT(' CHECK THESE:  G = ',I3,' B = ',I3,' R = ',I3)
      STOP
  100 CLOSE(5)
      CLOSE(6)
      END
C
C
      SUBROUTINE RANK(KG,KB,KR,H,DATA)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION RKS(10),DATA(10,12,15)
C
      ZERO = 0.0D0
      ONE = 1.0D0
      YM = ONE*(KG+1)/2
      DO 80 J = 1,KB
         DO 70 K = 1,KR
            CL = 1.0D30
            DO 3 I = 1,KG
               IF(DATA(I,J,K).LT.CL) CL = DATA(I,J,K)
    3       CONTINUE
            DO 7 I = 1,KG
               DATA(I,J,K) = DATA(I,J,K)-CL+ONE
    7       CONTINUE
            PHI = ONE+1.0D-12
            A1 = ZERO
            A2 = ZERO
            A3 = ONE*KG-0.1D0
            B1 = ZERO-1.0D30
            B2 = 1.0D30
   10       IF(A2.GT.A3)  GO TO 50
            DO 20 I = 1,KG
               IF(DATA(I,J,K).GT.B1.AND.DATA(I,J,K).LT.B2)
     1           B2 = DATA(I,J,K)*PHI
   20       CONTINUE
            DO 30 I = 1,KG
               W = DABS(ONE-DATA(I,J,K)/B2)
               IF(W.LT.1.0D-11)  A1 = A1+1
   30       CONTINUE
            A4 = A2+(A1+1)/2
            DO 40 I = 1,KG
               W = DABS(ONE-DATA(I,J,K)/B2)
               IF(W.LT.1.0D-11)  RKS(I) = A4
   40       CONTINUE
            A2 = A2+A1
            A1 = ZERO
            B1 = B2
            B2 = 1.0D30
            GO TO 10
   50       DO 60 I = 1,KG
               W = DABS(RKS(I)-YM)
               IF(W.LT.0.001D0)  THEN
                  DATA(I,J,K) = ZERO
               ELSE
                  DATA(I,J,K) = (RKS(I)-YM)*DABS(RKS(I)-YM)**(H-1)
               END IF
   60       CONTINUE
   70    CONTINUE
   80 CONTINUE
      RETURN
      END
C
C
      SUBROUTINE CALC(V,KG,KB,KR,IA,IC,LR,H,ISEED,MS,U,DATA)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17),D(120,120),DT(10,15),DATA(10,12,15)
C
      ONE=1.0D0
      ZERO=0.0D0
      Y=ONE*V/2
      BC2=ONE*KB*(KB-1)/2
      KBG=KB*KG
      DO 4 I=1,KBG
         DO 3 J=1,KBG
            D(I,J)=ZERO
    3    CONTINUE
    4 CONTINUE
      DO 10 I=1,KG
         DO 9 J=1,KB
            DO 8 K=I,KG
               LO=1
               IF(I.EQ.K) LO=J
               DO 7 L=LO,KB
                  IJ=KB*(I-1)+J
                  KL=KB*(K-1)+L
                  D(IJ,KL)=ZERO
                  DO 6 M=1,KR
                     D(IJ,KL)=D(IJ,KL)+(DATA(I,J,M)-DATA(K,L,M))**2
    6             CONTINUE
                  D(IJ,KL)=D(IJ,KL)**Y
                  D(KL,IJ)=D(IJ,KL)
    7          CONTINUE
    8       CONTINUE
    9    CONTINUE
   10 CONTINUE
      DELTA=ZERO
      DO 30 IS=2,KB
         IS1=IS-1
         DO 29 IR=1,IS1
            DO 28 I=1,KG
               IRR=(I-1)*KB+IR
               ISS=(I-1)*KB+IS
               DELTA=DELTA+D(IRR,ISS)
   28       CONTINUE
   29    CONTINUE
   30 CONTINUE
      C0=BC2*KG
      DELTA=DELTA/C0
      DX = DELTA*(ONE+1.0D-12)
      MP = 0
      DO 200 IW = 1,MS
         DO 50 J = 2,KB
            DO 46 I = 1,KG
               DO 45 K = 1,KR
                  DT(I,K) = DATA(I,J,K)
   45          CONTINUE
   46       CONTINUE
            CALL SAMPWOR(KG,KG,KR,U,DT)
            DO 48 I = 1,KG
               DO 47 K = 1,KR
                  DATA(I,J,K) = DT(I,K)
   47          CONTINUE
   48       CONTINUE
   50    CONTINUE
         DO 54 I=1,KBG
            DO 53 J=1,KBG
               D(I,J)=ZERO
   53       CONTINUE
   54    CONTINUE
         DO 60 I=1,KG
            DO 59 J=1,KB
               DO 58 K=I,KG
                  LO=1
                  IF(I.EQ.K) LO=J
                  DO 57 L=LO,KB
                     IJ=KB*(I-1)+J
                     KL=KB*(K-1)+L
                     D(IJ,KL)=ZERO
                     DO 56 M=1,KR
                        D(IJ,KL)=D(IJ,KL)+(DATA(I,J,M)-DATA(K,L,M))**2
   56                CONTINUE
                     D(IJ,KL)=D(IJ,KL)**Y
                     D(KL,IJ)=D(IJ,KL)
   57             CONTINUE
   58          CONTINUE
   59       CONTINUE
   60    CONTINUE
         DZ=ZERO
         DO 80 IS=2,KB
            IS1=IS-1
            DO 79 IR=1,IS1
               DO 78 I=1,KG
                  IRR=(I-1)*KB+IR
                  ISS=(I-1)*KB+IS
                  DZ=DZ+D(IRR,ISS)
   78          CONTINUE
   79       CONTINUE
   80    CONTINUE
         DZ = DZ/C0
         IF(DZ.LT.DX) MP = MP+1
  200 CONTINUE
      PROB = ONE*MP/MS
      WRITE(6,580)
  580 FORMAT(/'   RESULTS OF RMRBP ANALYSIS:')
      WRITE(6,585) LR
  585 FORMAT(11X,'1 (0) INDICATES (NO) C(G,H) RANKS TEST:',I3)
      IF(LR.EQ.1) THEN
        WRITE(6,588) H
  588   FORMAT(11X,'C(G,H) RANKS TEST EXPONENT H =',G16.7)
      ELSE
        WRITE(6,590)
  590   FORMAT(' ')
      END IF
      WRITE(6,593) IA
  593 FORMAT(11X,'1 (0) INDICATES (NO) MEDIAN ALIGNMENT:',I3)
      IF(KR.EQ.1) THEN
        WRITE(6,590)
      ELSE
        WRITE(6,597) IC
  597   FORMAT(11X,'1 (0) INDICATES (NO) AVE. DIST. COMMENSURATION:',I3)
      END IF
      WRITE(6,600) 'RANDOM NUMBER GENERATOR SEED =', ISEED
  600 FORMAT(T10,A,I9)
      WRITE(6,600) 'NUMBER OF SIMULATIONS =', MS
      WRITE(6,650) DELTA
  650 FORMAT(11X,'DELTA(D)',T29,':  ',G16.7)
      WRITE(6,670) PROB
  670 FORMAT(11X,'P-VALUE',T29,':  ',G16.7)
      WRITE(6,680)
  680 FORMAT('********************************************************')
      RETURN
      END
C
C
      SUBROUTINE SAMPWOR(M,N,ID,U,DATA)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),BIN(15),DATA(10,15)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     DATA: REAL INPUT WHICH CONTAINS DATA(1,*),...,DATA(M,*) POPULATION VALUES
C     WITHOUT REPLACEMENT SAMPLE CONTAINED IN DATA(1,*),...,DATA(N,*)
C
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 20 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         DO 10 K = 1,ID
            BIN(K) = DATA(J,K)
            DATA(J,K) = DATA(I,K)
            DATA(I,K) = BIN(K)
   10    CONTINUE
   20 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
