      PROGRAM RMRPC
      IMPLICIT REAL*8(A-H,O-Z)
C
C         THE TEST STATISTIC ASSOCIATED WITH RESAMPLING MULTI-RESPONSE
C         PERMUTATION COMPARISONS (RMRPC) OF TWO SAMPLES IS CALCULATED
C         WITH THIS FORTRAN-77 PROGRAM TO ANALYZE MULTIVARIATE DATA AT
C         ORDINAL (OR HIGHER) LEVELS AND IS BASED ON THE AVERAGE STANDARDIZED
C         EUCLIDEAN DISTANCE BETWEEN OBJECT POSITIONS OF THE TWO SAMPLES
C         OF SIZES N1 AND N2, RESPECTIVELY, WITHIN G A PRIORI DISJOINT
C         FINITE GROUPS OF n1(I) > 0 AND n2(I) > 0 OBJECTS, RESPECTIVELY,
C         FOR I = 1,...,G WHERE EACH POSITION IS A POINT IN AN R-DIMENSIONAL
C         SPACE (THUS THE ARGUMENTS OF EACH R-DIMENSIONAL POINT CORRESPOND
C         TO THE R MEASURED RESPONSES OF EACH OBJECT IN THE FINITE POPULATION
C         OF OBJECTS).  THE PRESENT MAXIMUM VALUES OF G, N1, N2, AND R
C         IN THIS PROGRAM ARE 60, 1200, 1200, AND 15, RESPECTIVELY.  RMRPC
C         IS A RMRPC PROGRAM THAT CALCULATES THE SIMULATED MEAN, VARIANCE,
C         AND SKEWNESS BASED ON THE RESAMPLED DATA IN ORDER TO OBTAIN
C         THE PEARSON TYPE III P-VALUE (BECAUSE A SPECIFIC CASE OF MRPC
C         IS MRBP WITH TWO BLOCKS, THE NORMAL APPROXIMATION WILL NOT BE
C         SATISFACTORY FOR MANY CASES).  IN ADDITION, THIS PROGRAM IS
C         CAPABLE OF MULTIPLE RUNS.
C
C         PROGRAM MODIFIED 2/24/2008
C
C       EITHER OF THE FOUR FOLLOWING WEIGHTING FORMS MAY BE CHOSEN.
C            CFORM = 1:  C(I) = sqrt(n1(I)n2(I))/sum(sqrt(n1(I)n2(I)))
C            CFORM = 2:  C(I) = n1(I)n2(I)/sum(n1(I)n2(I))
C            CFORM = 3:  C(I) = (n1(I)+N2(I))/(N1+N2)
C            CFORM = 4:  C(I) = 1/sum(1)
C       CFORM IS PASSED AS THE FOURTH PARAMETER ON THE SECOND RECORD
C       IN THE DATA FILE.  WEIGHTS FOR EACH GROUP ARE CALCULATED AND
C       STORED IN ARRAY C(I).
C
      CHARACTER HEAD*80,FIN*40,FOUT*40
      DIMENSION U(17),L1(0:60),L2(0:60),M1(60),M2(60),J1(1200),J2(1200),
     +          C(60),ZO(60),Z(60),X1(1200,15),X2(1200,15),D(1200,1200)
      INTEGER   CFORM
C
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA:'
      READ(*,100) FIN
  100 FORMAT(A)
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,100) FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I11 = MOD(IABS(ISEED),177)+1
      J11 = MOD(IABS(ISEED),167)+1
      K11 = MOD(IABS(ISEED),157)+1
      L11 = MOD(IABS(ISEED),147)+1
      DO 104 II = 1,17
         S = 0.0D0
         T = 0.5D0
         DO 102 JJ = 1,K
            M11 = MOD(MOD(I11*J11,179)*K11,179)
            I11 = J11
            J11 = K11
            K11 = M11
            L11 = MOD(53*L11+1,169)
            IF(MOD(L11*M11,64).GE.32) S = S+T
            T = T/2
  102    CONTINUE
         U(II) = S
  104 CONTINUE
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
  110 READ(3,100,END=120) HEAD
      GO TO 130
  120 GO TO 2000
  130 READ(3,*) N1,N2,ID,CFORM,NGRPS
      READ(3,*) (M1(I),I = 1,NGRPS)
      READ(3,*) (M2(I),I = 1,NGRPS)
      DO 140 I = 1,N1
         READ(3,*) (X1(I,J),J = 1,ID)
  140 CONTINUE
      DO 150 I = 1,N2
         READ(3,*) (X2(I,J),J = 1,ID)
  150 CONTINUE
      GO TO (155,170,185,200) CFORM
  155 FK = ZERO
      DO 160 I = 1,NGRPS
         FK = FK+DSQRT(ONE*M1(I)*M2(I))
  160 CONTINUE
      DO 165 I = 1,NGRPS
         C(I) = DSQRT(ONE*M1(I)*M2(I))/FK
  165 CONTINUE
      GO TO 220
  170 FK = ZERO
      DO 175 I = 1,NGRPS
         FK = FK+ONE*M1(I)*M2(I)
  175 CONTINUE
      DO 180 I = 1,NGRPS
         C(I) = ONE*M1(I)*M2(I)/FK
  180 CONTINUE
      GO TO 220
  185 FK = ZERO
      DO 190 I = 1,NGRPS
         FK = FK+ONE*M1(I)+M2(I)
  190 CONTINUE
      DO 195 I = 1,NGRPS
         C(I) = (ONE*M1(I)+M2(I))/FK
  195 CONTINUE
      GO TO 220
  200 DO 205 I = 1,NGRPS
         C(I) = ONE/NGRPS
  205 CONTINUE
  220 L1(0) = 0
      L2(0) = 0
      DO 225 I = 1,NGRPS
         L1(I) = L1(I-1)+M1(I)
         L2(I) = L2(I-1)+M2(I)
  225 CONTINUE
      DO 230 I = 1,N1
         J1(I) = I
  230 CONTINUE
      DO 235 I = 1,N2
         J2(I) = I
  235 CONTINUE
      EW = ZERO
      DO 250 I = 1,N1
         DO 245 J = 1,N2
            D(I,J) = ZERO
            DO 240 K = 1,ID
               D(I,J) = D(I,J)+(X1(I,K)-X2(J,K))**2
  240       CONTINUE
            D(I,J) = DSQRT(D(I,J))
            EW = EW+D(I,J)
  245    CONTINUE
  250 CONTINUE
      EW = EW/(N1*N2)
      WO = ZERO
      DO 280 K = 1,NGRPS
         ZO(K) = ZERO
         DO 270 I = L1(K-1)+1,L1(K)
            DO 260 J = L2(K-1)+1,L2(K)
               ZO(K) = ZO(K)+D(I,J)
  260       CONTINUE
  270    CONTINUE
         ZO(K) = ZO(K)/(M1(K)*M2(K))
         WO = WO+C(K)*ZO(K)
  280 CONTINUE
      RO = ONE-WO/EW
      WOL = WO*(ONE+1.0D-10)
      WOU = WO*(ONE-1.0D-10)
      ML = 0
      MU = 0
      U1 = ZERO
      U2 = ZERO
      U3 = ZERO
      DO 330 IW = 1,MS
         CALL SAMPWOR(N1,N1,U,J1)
         CALL SAMPWOR(N2,N2,U,J2)
         W = ZERO
         DO 320 K = 1,NGRPS
            Z(K) = ZERO
            DO 310 I = L1(K-1)+1,L1(K)
               DO 300 J = L2(K-1)+1,L2(K)
                  Z(K) = Z(K)+D(J1(I),J2(J))
  300          CONTINUE
  310       CONTINUE
            W = W+C(K)*Z(K)/(M1(K)*M2(K))
  320    CONTINUE
         IF(W.LT.WOL) ML = ML+1
         IF(W.GT.WOU) MU = MU+1
         U1 = U1+W
         U2 = U2+W**2
         U3 = U3+W**3
  330 CONTINUE
      U1 = U1/MS
      U2 = U2/MS
      U3 = U3/MS
      PL = ONE*ML/MS
      PU = ONE*MU/MS
      VAR = U2-U1**2
      GAM = (U3-U2*U1*3+U1**3*2)/VAR**(1.5D0)
      T = (WO-EW)/DSQRT(VAR)
      CALL PVALUE(T,GAM,PROB)
      CALL PVALUE(T,ZERO,PN)
C
      WRITE(6,100) HEAD
      HEAD = 'TWO-SAMPLE MULTI-RESPONSE PERMUTATION COMPARISON'
      WRITE(6,1100) HEAD
 1100 FORMAT(T2,A)
      WRITE(6,1200) 'INPUT HAS ',N1,' AND ',N2,' CASES IN SAMPLES ',
     +              '#1 AND #2, RESPECTIVELY'
 1200 FORMAT(T2,A,I4,A,I4,A,A)
      WRITE(6,1250) 'THERE WERE ',ID,' DIMENSION(S)'
 1250 FORMAT(T2,A,I3,A)
      WRITE(6,1300) 'IN THE ', NGRPS,' GROUPS OF SAMPLE #1 AND'
 1300 FORMAT(T2,A,I2,A)
      WRITE(6,1300) 'THE ', NGRPS,' CORRESPONDING GROUPS OF SAMPLE #2'
      GO TO (1410,1420,1430,1440) CFORM
 1410 WRITE(6,1450) 'C(I) = sqrt(n1(I)n2(I))/sum(sqrt(n1(J)n2(J)))'
      GO TO 1500
 1420 WRITE(6,1450) 'C(I) = n1(I)n2(I)/sum(n1(J)n2(J))'
      GO TO 1500
 1430 WRITE(6,1450) 'C(I) = (n1(I)+n2(I))/(N1+N2)'
      GO TO 1500
 1440 WRITE(6,1450) 'C(I) = 1/sum(1)'
 1450 FORMAT(T10,A)
 1500 WRITE(6,100) ' '
      DO 1700 I = 1,NGRPS
         WRITE(6,1600) 'GROUP NUMBER ', I, ' OF SIZES ', M1(I),' AND ',
     +                 M2(I),' HAS AN AVERAGE DISTANCE =', ZO(I)
 1600    FORMAT(T2,A,I2,A,I4,A,I4,A,G16.8)
 1700 CONTINUE
      WRITE(6,100) ' '
      WRITE(6,1710) 'RANDOM NUMBER GENERATOR SEED = ',ISEED
      WRITE(6,1710) 'NUMBER OF SIMULATIONS = ',MS
 1710 FORMAT(T10,A,I9)
 1800 FORMAT(T10,A,G16.8E3)
      WRITE(6,1800) 'RESAMPLING LOWER TAIL P-VALUE OF OBSERVED W = ',PL
      WRITE(6,1800) 'RESAMPLING UPPER TAIL P-VALUE OF OBSERVED W = ',PU
      WRITE(6,1800) 'APPROXIMATE PEARSON TYPE III P-VALUE = ',PROB
      WRITE(6,1800) 'APPROXIMATE NORMAL P-VALUE = ',PN
      WRITE(6,1800) 'OBSERVED VALUE OF W = ', WO
      WRITE(6,1800) 'EXACT MEAN OF W = ', EW
      WRITE(6,1800) 'AGREEMENT MEASURE ASSOCIATED WITH WO = ',RO
      WRITE(6,1800) 'APPROXIMATE STANDARDIZED STATISTIC = ', T
      WRITE(6,1800) 'APPROXIMATE MEAN OF W = ',U1
      WRITE(6,1800) 'APPROXIMATE VARIANCE OF W = ',VAR
      WRITE(6,1800) 'APPROXIMATE SKEWNESS OF W = ', GAM
      WRITE(6,1850)
 1850 FORMAT('********************************************************')
      GO TO 110
 2000 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE SAMPWOR(M,N,U,L)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17),L(1200)
C
C     M: SIZE OF POPULATION
C     N: SIZE OF SAMPLE DRAWN WITHOUT REPLACEMENT (1 </= N </= M)
C     X: REAL INPUT WHICH CONTAINS L(1),...,L(M) POPULATION VALUES
C     WITHOUT REPLACEMENT VALUES CONTAINED IN L(1),...,L(N)
C     UNI(U) IS UNIFORM RANDOM NUMBER GENERATOR ON (0,1)
C
      DO 10 I = 1,N
         J = INT(I+(M-I+1)*UNI(U))
         LBIN = L(J)
         L(J) = L(I)
         L(I) = LBIN
   10 CONTINUE
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
C
C
      SUBROUTINE PVALUE(T,GAM,PROB)
C
C         SUBROUTINE PVALUE CALCULATES THE PROBABILITY THAT A VALUE OF
C         T IS LESS THAN OR EQUAL TO THE OBSERVED VALUE OF T.
C
      DOUBLE PRECISION  T,GAM,PROB,ZERO,ONE,PI,A,B,C,D,E,F,G,H,R,
     +   U,W,X,Y,E1,E2,E3,E4,E5,G1,G2,G3,H0,H1,H1N,H2,H3
C
      ZERO = 0.0D0
      ONE = 1.0D0
      PI = DATAN(ONE)*4
      IF(DABS(GAM).LT.0.01D0) GO TO 50
      R = ONE*2/DABS(GAM)
      D = R*R
      F = R*R+10
      DO 10 I = 1,9
         D = D*(R*R+I)
   10 CONTINUE
      U = (F*2-1)*DLOG(F)/2-F+DLOG(PI*2)/2-DLOG(D)+ONE/(F*12)
     +        -ONE/(F*F*F*360)
      A = R*R-1
      B = R*R*(DLOG(R)-1)-U
      G1 = 0.0375D0
      G2 = 0.075D0
      G3 = 0.0125D0
      H1 = ZERO
      H2 = ZERO
      W = -1.99D0/GAM
      IF(GAM.LT.ZERO) GO TO 30
      IF(T.LT.W) THEN
         PROB = ZERO
         RETURN
      END IF
      X = T
      Y = T+15
      DO 20 I = 1,199
         G = A*DLOG(R+X+G1*(2*I-1))-R*(X+G1*(2*I-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R+X+G2*I)-R*(X+G2*I)+B
         H2 = H2+E(G)
   20 CONTINUE
      G = A*DLOG(R+X+G1*399)-R*(X+G1*399)+B
      H1N = E(G)
      G = A*DLOG(R+X)-R*X+B
      H0 = E(G)
      G = A*DLOG(R+Y)-R*Y+B
      H3 = E(G)
      PROB = ONE-G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   30 IF(T.GT.W) THEN
         PROB = ONE
         RETURN
      END IF
      X = T-15
      Y = T
      DO 40 I = 1,199
         G = A*DLOG(R-X-G1*(2*I-1))+R*(X+G1*(2*I-1))+B
         H1 = H1+E(G)
         G = A*DLOG(R-X-G2*I)+R*(X+G2*I)+B
         H2 = H2+E(G)
   40 CONTINUE
      G = A*DLOG(R-X-G1*399)+R*(X+G1*399)+B
      H1N = E(G)
      G = A*DLOG(R-X)+R*X+B
      H0 = E(G)
      G = A*DLOG(R-Y)+R*Y+B
      H3 = E(G)
      PROB = G3*(H0+(H1+H1N)*4+H2*2+H3)
      RETURN
   50 PROB = ZERO
      G = -T*T/2
      IF(G.LT.-738.0D0) GO TO 70
      A = 4.2D0
      U = DABS(T)
      IF(U.GT.A) GO TO 60
      E1 = 0.31938153D0
      E2 = -0.356563782D0
      E3 = 1.781477937D0
      E4 = -1.821255978D0
      E5 = 1.330274429D0
      H = 0.2316419D0
      W = ONE/(H*U+1)
      PROB = ((((E5*W+E4)*W+E3)*W+E2)*W+E1)*W*E(G)/DSQRT(PI*2)
      GO TO 70
   60 B = 2.2D0
      C = 4.9D0
      PROB = E(G)*(ONE/U-ONE/U**3+B/U**C)/DSQRT(PI*2)
   70 IF(T.GT.ZERO) PROB = ONE-PROB
      RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION E(G)
C
      DOUBLE PRECISION G
C
      E = 0.0D0
      IF(G.GE.-738.0D0) E = DEXP(G)
      RETURN
      END
