      PROGRAM RMVPTMP
      IMPLICIT REAL*8(A-H,O-Z)
C
C    ONE-SAMPLE OPTION: (KW = 1)
C    THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C    P-VALUE FOR A ONE-SAMPLE MULTIVARIATE (PERMUTATION) TEST WITH
C    HYPOTHESIZED MEAN VECTOR OF THE R RESPONSES.  AN AGREEMENT MEASURE
C    CORRESPONDING TO A PEARSON TYPE CORRELATION COEFFICIENT IS ALSO
C    PROVIDED.  THE MAXIMUM VALUES OF B AND R CAN BE CHANGED FOR ANY
C    EXAMPLE.  THE PRESENT MAXIMUM VALUES OF B AND R IN THIS PROGRAM
C    ARE RESPECTIVELY 20 AND 15.  THIS PROGRAM CAN INVOKE A HOTELLING
C    TYPE COMMENSURATION (LZ = 1 & B > R).  THIS PROGRAM CAN ALSO INVOKE
C    AVERAGE PAIR-WISE DISTANCE FUNCTION (D.F.) COMMENSURATION (IC = 1).
C
C    THE DATA MATRIX FOR ONE-SAMPLE OPTION MUST BE IN THE FOLLOWING SEQUENCE:
C    THE FIRST LINE IS A DESCRIPTIVE TITLE OF THE DATA
C    THE SECOND LINE ARE THE ENTRIES V, B, R, KW, LZ AND IC
C    THE THIRD LINE IS THE HYPOTHESIZED R MEAN (MEDIAN) RESPONSE VALUES
C        UM(1),    UM(2),    ...,UM(R)
C    THE JTH OBJECT'S R RESPONSE VALUES ARE THEN ON THE (3+J)TH LINE
C    WHERE J = 1,...,B (THE # OF OBJECTS).
C
C        A(1,1,1),A(1,1,2),...,A(1,1,R)
C        A(1,2,1),A(1,2,2),...,A(1,2,R)
C                 ...
C        A(1,B,1),A(1,B,2),...,A(1,B,R)
C
C    THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C    B IS # OF BLOCKS (IN THIS CASE OBJECTS), AND R IS # OF RESPONSES.
C
C
C    MATCHED-PAIRS OPTION: (KW = 2)
C    THIS FORTRAN PROGRAM COMPUTES THE TEST STATISTIC AND ASSOCIATED
C    P-VALUE FOR A MATCHED-PAIRS MULTIVARIATE (PERMUTATION) PROCEDURES.
C    AN AGREEMENT MEASURE CORRESPONDING TO A PEARSON TYPE CORRELATION
C    COEFFICIENT IS ALSO PROVIDED.  THE MAXIMUM VALUES OF B AND R CAN
C    BE CHANGED FOR ANY EXAMPLE.  THE PRESENT MAXIMUM VALUES OF B AND R
C    IN THIS PROGRAM ARE RESPECTIVELY 20 AND 15.  THIS PROGRAM CAN INVOKE
C    A HOTELLING TYPE COMMENSURATION (LZ = 1 & B > R).  THIS PROGRAM CAN
C    ALSO INVOKE AVERAGE PAIR-WISE DISTANCE FUNCTION (D.F.) COMMENSURATION
C    (IC = 1).
C
C    THE DATA MATRIX MUST BE IN THE FOLLOWING SEQUENCE:
C    THE FIRST LINE IS A DESCRIPTIVE TITLE OF THE DATA
C    THE SECOND LINE ARE THE ENTRIES V, B, R, KW, LZ AND IC
C    THE JTH PAIR'S 1ST R RESPONSE VALUES ARE ON THE (2+J)TH LINE
C    AND THE 2ND R RESPONSE VALUES ARE ON THE (2+B+J)TH LINE WHERE
C    J = 1,...,B (THE # OF PAIRS).
C
C        A(1,1,1),A(1,1,2),...,A(1,1,R)
C                 ...
C        A(1,B,1),A(1,B,2),...,A(1,B,R)
C        A(2,1,1),A(2,1,2),...,A(2,1,R)
C                 ...
C        A(2,B,1),A(2,B,2),...,A(2,B,R)
C
C    THE INPUT DATA ARE IN FREE FORMAT.  SPECIFICALLY, V IS DISTANCE EXPONENT,
C    B IS # OF BLOCKS (IN THIS CASE PAIRS), AND R IS # OF RESPONSES.
C
C    NOTE #1: KB > W IS REQUIRED FOR PROGRAM TO EXECUTE.
C
C    NOTE #2: ASSOCIATE B AND R WITH KB AND KR IN THIS PROGRAM.
C
C    NOTE #3: HOTELLING TYPE COMMENSURATION IS (NOT) INVOKED IF KB > KR > 1
C             AND LZ = 1 (0).
C
C    NOTE #4: AVERAGE PAIRWISE D.F. COMMENSURATION IS (NOT) INVOKED IF
C             KR > 1, LZ = 0 AND IC = 1 (0).
C
C    NOTE #5: PROGRAM MVPTMP IS CAPABLE OF MULTIPLE RUNS.  HOWEVER, LEAVE NO
C             BLANK LINES BETWEEN DATA SETS SINCE THE FIRST LINE FOLLOWING
C             THE LAST DATA ENTRY MAY BE READ AS A BLANK PROBLEM HEAD FOR THE
C             NEXT DATA SET.
C
C    PROGRAM MODIFIED 11/19/99
C
      CHARACTER  FIN*40,FOUT*40,HEAD*80
      DIMENSION U(17),UM(15),X(15),AD(15),DIFF(15),Y(15,15),W(15,15),
     1          D(40,40),RAW(2,20,15),DT(2,20,15)
C
      ONE = 1.0D0
      HALF = 0.5D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER: FILE NAME OF INPUT DATA'
      READ(*,1) FIN
    1 FORMAT(A)
      WRITE(*,*) 'ENTER: FILE NAME OF OUTPUT RESULTS'
      READ(*,1) FOUT
      WRITE(*,*) 'ENTER RANDOM # SEED & # OF SIMULATIONS:'
      READ(*,*) ISEED,MS
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 3 II = 1,17
         S = ZERO
         T = HALF
         DO 2 JJ = 1,K
            M1 = MOD(MOD(I1*J1,179)*K1,179)
            I1 = J1
            J1 = K1
            K1 = M1
            L1 = MOD(53*L1+1,169)
            IF(MOD(L1*M1,64).GE.32) S = S+T
            T = T/2
    2    CONTINUE
         U(II) = S
    3 CONTINUE
      OPEN(3,FILE=FIN)
      OPEN(6,FILE=FOUT,STATUS='NEW')
    4 READ(3,1,END=5) HEAD
      GO TO 6
    5 GO TO 800
    6 READ(3,*) V,KB,KR,KW,LZ,IC
      IF(KB.LT.4) STOP 'NUMBER OF BLOCKS LESS THAN FOUR'
      IF(KB.LE.KR) LZ = 0
      IF(KR.EQ.1) THEN
         LZ = 0
         IC = 0
      END IF
      IF(KW.EQ.2) GO TO 21
      READ(3,*) (UM(K),K=1,KR)
      DO 7 J = 1,KB
         READ(3,*,END=770) (RAW(1,J,K),K=1,KR)
    7 CONTINUE
      WRITE(6,1) HEAD
      WRITE(6,8) V,KR,KB
    8 FORMAT('ONE-SAMPLE MULTIVARIATE PERMUTATION TEST:'
     1     /4X,'DISTANCE EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES AND ',I3,' BLOCKS.')
      IF(LZ.EQ.1) THEN
         WRITE(6,1) 'HOTELLING TYPE COMMENSURATION IS INVOKED'
         IC = 0
         GO TO 14
      END IF
      DO 12 J = 1,KB
         DO 10 K = 1,KR
            DIFF(K) = RAW(1,J,K)-UM(K)
   10    CONTINUE
         DO 11 K = 1,KR
            DT(1,J,K) = DIFF(K)
            DT(2,J,K) = ZERO-DIFF(K)
   11    CONTINUE
   12 CONTINUE
      GO TO 30
   14 DO 20 J = 1,KB
         DO 16 K = 1,KR
            DIFF(K) = RAW(1,J,K)-UM(K)
   16    CONTINUE
         DO 18 K = 1,KR
            RAW(1,J,K) = DIFF(K)
   18    CONTINUE
   20 CONTINUE
      GO TO 65
   21 DO 23 I = 1,2
         DO 22 J = 1,KB
            READ(3,*,END=770) (RAW(I,J,K),K=1,KR)
   22    CONTINUE
   23 CONTINUE
      WRITE(6,1) HEAD
      WRITE(6,25) V,KR,KB
   25 FORMAT('MATCHED-PAIRS MULTIVARIATE PERMUTATION TEST:'
     1     /4X,'DISTANCE EXPONENT: ',F5.2/4X,'WITH ',I3,
     2     ' RESPONSES AND ',I3,' BLOCKS.')
      IF(LZ.EQ.1) THEN
         WRITE(6,1) 'HOTELLING TYPE COMMENSURATION IS INVOKED'
         IC = 0
         GO TO 54
      END IF
      DO 28 J = 1,KB
         DO 26 K = 1,KR
            DIFF(K) = RAW(1,J,K)-RAW(2,J,K)
   26    CONTINUE
         DO 27 K = 1,KR
            DT(1,J,K) = DIFF(K)
            DT(2,J,K) = ZERO-DIFF(K)
   27    CONTINUE
   28 CONTINUE
   30 IF(IC.EQ.0) GO TO 165
      WRITE(6,1) 'AVERAGE D.F. COMMENSURATION IS INVOKED'
      DO 38 K = 1,KR
        AD(K) = ZERO
        DO 37 J1 = 2,KB
          DO 36 J2 = 1,J1-1
            AD(K) = AD(K)+((DABS(DT(1,J1,K)-DT(1,J2,K)))**V
     1              +(DABS(DT(1,J1,K)+DT(1,J2,K)))**V)*2.0D0
   36     CONTINUE
   37   CONTINUE
        AD(K) = AD(K)**(ONE/V)
   38 CONTINUE
      DO 46 I = 1,2
         DO 45 J = 1,KB
            DO 44 K = 1,KR
               DT(I,J,K) = DT(I,J,K)/AD(K)
   44       CONTINUE
   45    CONTINUE
   46 CONTINUE
      GO TO 165
   54 DO 60 J = 1,KB
         DO 56 K = 1,KR
            DIFF(K) = RAW(1,J,K)-RAW(2,J,K)
   56    CONTINUE
         DO 58 K = 1,KR
            RAW(1,J,K) = DIFF(K)
   58    CONTINUE
   60 CONTINUE
   65 DO 70 I = 1,KR
         X(I) = ZERO
         DO 69 J = 1,KB
            X(I) = X(I)+DABS(RAW(1,J,I))
   69    CONTINUE
         X(I) = X(I)/KB
   70 CONTINUE
      DO 80 I = 1,KR
         DO 79 J = I,KR
            W(I,J) = ZERO
            DO 78 K = 1,KB
               W(I,J) = W(I,J)+(DABS(RAW(1,K,I))-X(I))
     1                        *(DABS(RAW(1,K,J))-X(J))
   78       CONTINUE
            W(I,J) = W(I,J)/KB
            W(J,I) = W(I,J)
   79    CONTINUE
   80 CONTINUE
      N = KR
      CALL INVERT(N,W)
      Y(1,1) = DSQRT(W(1,1))
      DO 90 I = 2,KR
         Y(1,I) = W(1,I)/Y(1,1)
         Y(I,1) = ZERO
   90 CONTINUE
      DO 130 I = 2,KR
         Y(I,I) = W(I,I)
         DO 100 J = 1,I-1
            Y(I,I) = Y(I,I)-Y(J,I)*Y(J,I)
  100    CONTINUE
         Y(I,I) = DSQRT(Y(I,I))
         DO 120 J = I+1,KR
            Y(I,J) = W(I,J)
            Y(J,I) = ZERO
            DO 110 K = 1,I-1
               Y(I,J) = Y(I,J)-Y(K,I)*Y(K,J)
  110       CONTINUE
            Y(I,J) = Y(I,J)/Y(I,I)
  120    CONTINUE
  130 CONTINUE
      DO 160 I = 1,KB
         DO 150 J = 1,KR
            DT(1,I,J) = ZERO
            DO 140 K = J,KR
               DT(1,I,J) = DT(1,I,J)+Y(J,K)*RAW(1,I,K)
  140       CONTINUE
            DT(2,I,J) = ZERO-DT(1,I,J)
  150    CONTINUE
  160 CONTINUE
  165 DO 210 I=1,2
         DO 209 J=1,KB
            DO 208 K=I,2
               LO=1
               IF(I.EQ.K) LO=J
               DO 207 L=LO,KB
                  IJ=KB*(I-1)+J
                  KL=KB*(K-1)+L
                  D(IJ,KL)=ZERO
                  DO 206 M=1,KR
                     D(IJ,KL)=D(IJ,KL)+(DT(I,J,M)-DT(K,L,M))**2
  206             CONTINUE
                  D(IJ,KL)=D(IJ,KL)**(V/2)
                  D(KL,IJ)=D(IJ,KL)
  207          CONTINUE
  208       CONTINUE
  209    CONTINUE
  210 CONTINUE
      DELTA=ZERO
      DO 300 IS=2,KB
         IS1=IS-1
         DO 299 IR=1,IS1
            DO 298 I=1,2
               IRR=(I-1)*KB+IR
               ISS=(I-1)*KB+IS
               DELTA=DELTA+D(IRR,ISS)
  298       CONTINUE
  299    CONTINUE
  300 CONTINUE
      DELTA=DELTA*(ONE+1.0D-12)
      MP = 0
      DO 500 IM=1,MS
         DO 303 I=1,KB
            IF(UNI(U).LT.HALF) THEN
               DO 302 J=1,KR
                  DT(2,I,J)=DT(1,I,J)
                  DT(1,I,J)=ZERO-DT(1,I,J)
  302          CONTINUE
            END IF
  303    CONTINUE
         DO 310 I=1,2
            DO 309 J=1,KB
               DO 308 K=I,2
                  LO=1
                  IF(I.EQ.K) LO=J
                  DO 307 L=LO,KB
                     IJ=KB*(I-1)+J
                     KL=KB*(K-1)+L
                     D(IJ,KL)=ZERO
                     DO 306 M=1,KR
                        D(IJ,KL)=D(IJ,KL)+(DT(I,J,M)-DT(K,L,M))**2
  306                CONTINUE
                     D(IJ,KL)=D(IJ,KL)**(V/2)
                     D(KL,IJ)=D(IJ,KL)
  307             CONTINUE
  308          CONTINUE
  309       CONTINUE
  310    CONTINUE
         DX=ZERO
         DO 400 IS=2,KB
            IS1=IS-1
            DO 399 IR=1,IS1
               DO 398 I=1,2
                  IRR=(I-1)*KB+IR
                  ISS=(I-1)*KB+IS
                  DX=DX+D(IRR,ISS)
  398          CONTINUE
  399       CONTINUE
  400    CONTINUE
         IF(DX.LT.DELTA) MP=MP+1
  500 CONTINUE
      PROB = ONE*MP/MS
      C0=ONE*KB*(KB-1)
      DELTA=DELTA/C0
      IF(KW.EQ.2) GO TO 570
      WRITE(6,560)
  560 FORMAT(/'RESULTS OF MULTIVARIATE ONE-SAMPLE PERMUTATION TEST:')
      GO TO 590
  570 WRITE(6,580)
  580 FORMAT(/'RESULTS OF MULTIVARIATE MATCHED-PAIRS PERMUTATION TEST:')
  590 WRITE(6,595) 'RANDOM NUMBER GENERATOR SEED =',ISEED
      WRITE(6,595) 'NUMBER OF SIMULATIONS =',MS
  595 FORMAT(T10,A,I9)
      WRITE(6,600) DELTA
  600 FORMAT(11X,'DELTA(D)',T29,':  ',G16.7)
      WRITE(6,670) PROB
  670 FORMAT(11X,'SIMULATED P-VALUE',T29,':  ',G16.7)
      WRITE(6,680)
  680 FORMAT('********************************************************')
      GO TO 4
  770 WRITE(*,780)
  780 FORMAT(' E R R O R: END-OF-FILE ENCOUNTERED.')
      WRITE(*,790) KB,KR
  790 FORMAT(' CHECK THESE:  B = ',I3,' R = ',I3)
  800 CLOSE(3)
      CLOSE(6)
      END
C
C
      SUBROUTINE INVERT(N,W)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION KP(15),KS(15),KC(15),W(15,15)
C
      ZERO = 0.0D0
      DO 5 I = 1,N
         KP(I) = 0
    5 CONTINUE
      DO 45 I = 1,N
         T = ZERO
         DO 20 J = 1,N
            IF(KP(J).NE.1) THEN
               DO 15 K = 1,N
                  IF(KP(K)-1) 10,15,60
   10             IF(T.LT.DABS(W(J,K))) THEN
                     IS = J
                     IC = K
                     T = DABS(W(J,K))
                  END IF
   15          CONTINUE
            END IF
   20    CONTINUE
         KP(IC) = KP(IC)+1
         IF(IS.NE.IC) THEN
            DO 25 L = 1,N
               T = W(IS,L)
               W(IS,L) = W(IC,L)
               W(IC,L) = T
   25       CONTINUE
         END IF
         KS(I) = IS
         KC(I) = IC
         T = W(IC,IC)
         W(IC,IC) = 1.0D0
         DO 30 L = 1,N
            W(IC,L) = W(IC,L)/T
   30    CONTINUE
         DO 40 M = 1,N
            IF(M.NE.IC) THEN
               T = W(M,IC)
               W(M,IC) = ZERO
               DO 35 L = 1,N
                  W(M,L) = W(M,L)-W(IC,L)*T
   35          CONTINUE
            END IF
   40    CONTINUE
   45 CONTINUE
      DO 55 I = 1,N
         J = N-I+1
         IF(KS(J).NE.KC(J)) THEN
            K = KS(J)
            L = KC(J)
            DO 50 M = 1,N
               T = W(M,K)
               W(M,K) = W(M,L)
               W(M,L) = T
   50       CONTINUE
         END IF
   55 CONTINUE
   60 RETURN
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END


