      PROGRAM EI222
C
C     PROGRAM EI222 IS A DRIVER PROGRAM, CALLING A SPECIFIC
C     SUBROUTINES TO COMPUTE FISHER'S EXACT PROBABILITY FOR
C     ALL FIRST AND SECOND ORDER INTERACTIONS OF A 2X2X2 TABLE.
C     THE SUBROUTINE CONSTANTS (INIT AND EPSLN) ARE MACHINE
C     DEPENDENT  VALUES TO COMPENSATE FOR ROUNDING (EPSLN) AND
C     TO PREVENT UNDER/OVER FLOW (INIT).  THESE VALUES MAY BE
C     CHANGED TO ACCOMMODATE SPECIFIC MACHINES.
C
      INTEGER    A(2,2),E(2,2,2)
      REAL*8     INIT,EPSLN,ONE,ZERO
      CHARACTER  FOUT*40
C
C     COMPUTE EXACT 1ST & 2ND ORDER INTERACTION P-VALUES FOR A 2X2X2 TABLE
C
      INIT = 1.0D-200
      EPSLN = 1.0D-12
      ONE = 1.0D0
      ZERO = 0.0D0
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS:'
      READ(*,2) FOUT
    2 FORMAT(A)
      WRITE(*,3) 'ENTER E111 AND E211 OF THE 2X2X2 TABLE:'
    3 FORMAT(T2,A)
      READ(*,*)  E(1,1,1),E(2,1,1)
      WRITE(*,3) 'ENTER E121 AND E221 OF THE 2X2X2 TABLE:'
      READ(*,*)  E(1,2,1),E(2,2,1)
      WRITE(*,3) 'ENTER E112 AND E212 OF THE 2X2X2 TABLE:'
      READ(*,*)  E(1,1,2),E(2,1,2)
      WRITE(*,3) 'ENTER E122 AND E222 OF THE 2X2X2 TABLE:'
      READ(*,*)  E(1,2,2),E(2,2,2)
      OPEN(7,FILE=FOUT,STATUS='NEW')
      WRITE(7,4) 'EXACT 1ST & 2ND ORDER 2X2X2 TABLE INTERACTION PVALUES'
    4 FORMAT(T2,A,/)
      WRITE(7,6) 'SECOND ORDER AXBXC INTERACTION:'
    6 FORMAT(/,T2,A)
      WRITE(7,8) 'E111 & E211 OF 2X2X2 TABLE: ',E(1,1,1),E(2,1,1)
    8 FORMAT(T5,A,2I8)
      WRITE(7,8) 'E121 & E221 OF 2X2X2 TABLE: ',E(1,2,1),E(2,2,1)
      WRITE(7,8) 'E112 & E212 OF 2X2X2 TABLE: ',E(1,1,2),E(2,1,2)
      WRITE(7,8) 'E122 & E222 OF 2X2X2 TABLE: ',E(1,2,2),E(2,2,2)
      CALL FE2X2X2(INIT,EPSLN,ONE,ZERO,E)
      WRITE(7,6) 'FIRST ORDER AXB INTERACTION:'
      A(1,1) = E(1,1,1)+E(1,1,2)
      A(1,2) = E(1,2,1)+E(1,2,2)
      A(2,1) = E(2,1,1)+E(2,1,2)
      A(2,2) = E(2,2,1)+E(2,2,2)
      CALL FET2X2(INIT,EPSLN,ONE,ZERO,A)
      WRITE(7,6) 'FIRST ORDER AXC INTERACTION:'
      A(1,1) = E(1,1,1)+E(1,2,1)
      A(1,2) = E(1,1,2)+E(1,2,2)
      A(2,1) = E(2,1,1)+E(2,2,1)
      A(2,2) = E(2,1,2)+E(2,2,2)
      CALL FET2X2(INIT,EPSLN,ONE,ZERO,A)
      WRITE(7,6) 'FIRST ORDER BXC INTERACTION:'
      A(1,1) = E(1,1,1)+E(2,1,1)
      A(1,2) = E(1,1,2)+E(2,1,2)
      A(2,1) = E(1,2,1)+E(2,2,1)
      A(2,2) = E(1,2,2)+E(2,2,2)
      CALL FET2X2(INIT,EPSLN,ONE,ZERO,A)
      END
C
C
      SUBROUTINE FET2X2(INIT,EPSLN,ONE,ZERO,E)
C
      INTEGER  E(2,2),X,R1,C1,Z,ULZ
      REAL*8   H,P,RR1,RC1,ZZ,RL,T,S,U,INIT,EPSLN,ONE,ZERO
C
      H = INIT
      T = INIT
      U = INIT
      S = ZERO
      X = E(1,1)
      R1 = E(1,1)+E(1,2)
      C1 = E(1,1)+E(2,1)
      N = R1+E(2,1)+E(2,2)
      K = R1+C1-N
      L = 1-K
      RR1 = ONE*R1
      RC1 = ONE*C1
      RL = ONE*L
      LLZ = MAX0(0,K)
      ULZ = MIN0(R1,C1)
      IF(X.GT.LLZ) THEN
         DO 10 Z = LLZ,X-1
            ZZ = ONE*Z
            U = U*(RR1-ZZ)*(RC1-ZZ)/(ONE+ZZ)/(RL+ZZ)
   10    CONTINUE
      END IF
      U = U*(ONE+EPSLN)
      IF(H.LE.U) S = H
      DO 20 Z = LLZ,ULZ-1
         ZZ = ONE*Z
         H = H*(RR1-ZZ)*(RC1-ZZ)/(ONE+ZZ)/(RL+ZZ)
         T = T+H
         IF(H.LE.U) S = S+H
   20 CONTINUE
      P = S/T
      WRITE(7,25) '1ST ROW OF 2 BY 2 TABLE:',E(1,1),E(1,2)
   25 FORMAT(T5,A,2I8)
      WRITE(7,25) '2ND ROW OF 2 BY 2 TABLE:',E(2,1),E(2,2)
      WRITE(7,35) 'FIRST ORDER INTERACTION P-VALUE FOR 2X2X2 TABLE = ',P
   35 FORMAT(/,T5,A,G16.6)
      RETURN
      END
C
C
      SUBROUTINE FE2X2X2(INIT,EPSLN,ONE,ZERO,E)
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        E     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        EPSLN -- INPUT,  EPSILON VALUE FOR ROUNDING ERROR
C        INIT  -- INPUT,  INITIAL VALUE FOR RECURSION PROCEDURE
C     USAGE:
C        SUBROUTINE FE2X2X2 COMPUTES THE FISHER EXACT PROBABILITY TEST FOR
C        SECOND ORDER INTERACTION OF 2X2X2 CROSS-CLASSIFICATION TABLES
C

      INTEGER  N,X,Z,A,B,C,AB,AC,BC,LLZ,ULZ,E(2,2,2)
      REAL*8   H,T,P,S,U,ZZ,RA,RB,RC,RAB,RAC,RBC,RN,
     1         WAB,WAC,WBC,WN,INIT,EPSLN,ONE,ZERO
C
C     INITIALIZE VARIABLES
C
      U  = INIT
      H  = INIT
      T  = INIT
      S  = ZERO
      X  = E(1,1,1)
      A  = E(1,1,1)+E(2,1,1)
      B  = E(1,1,1)+E(1,2,1)
      C  = E(1,1,1)+E(1,1,2)
      AB = A+E(1,2,1)+E(2,2,1)
      AC = A+E(1,1,2)+E(2,1,2)
      BC = B+E(1,1,2)+E(1,2,2)
      N  = AB+E(1,1,2)+E(2,1,2)+E(1,2,2)+E(2,2,2)
      RA = ONE*A
      RB = ONE*B
      RC = ONE*C
      RAB = ONE*AB
      RAC = ONE*AC
      RBC = ONE*BC
      RN  = ONE*N
      WAB = RAB-RA-RB+ONE
      WAC = RAC-RA-RC+ONE
      WBC = RBC-RB-RC+ONE
      WN  = RN-RAB-RAC-RBC+RA+RB+RC
      LLZ = MAX0(0,A+B-AB,A+C-AC,B+C-BC)
      ULZ = MIN0(A,B,C,N-AB-AC-BC+A+B+C)
C
C     BEGIN FIRST RECURSION LOOP
C
      IF(X.GT.LLZ) THEN
         DO 1 Z = LLZ,X-1
            ZZ = ONE*Z
            U  = U *(RA-ZZ)*(RB-ZZ)*(RC-ZZ)*(WN-ZZ)/(ZZ+ONE)
     1             /(WAB+ZZ)/(WAC+ZZ)/(WBC+ZZ)
   1     CONTINUE
      END IF
      U  = U*(ONE+EPSLN)
      IF(H.LE.U) S = H
C
C     BEGIN SECOND RECURSION LOOP
C
      DO 2 Z = LLZ,ULZ-1
         ZZ = ONE*Z
         H  = H*(RA-ZZ)*(RB-ZZ)*(RC-ZZ)*(WN-ZZ)/(ZZ+ONE)
     1         /(WAB+ZZ)/(WAC+ZZ)/(WBC+ZZ)
         T  = T+H
         IF(H.LE.U) S = S+H
    2 CONTINUE
      P = S/T
      WRITE(7,5) 'SECOND ORDER INTERACTION P-VALUE FOR 2X2X2 TABLE = ',P
    5 FORMAT(/,T5,A,G16.6)
      RETURN
      END
