      PROGRAM VCGF
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION  P(9000),C(9001),U(17)
      CHARACTER  FIN*40,FOUT*40
C
C   SIMULATED V-POWER COVERAGE GOODNESS-OF-FIT (VCGF) TEST WITH THE FORM
C                 CV = SUM{1,N+1}|C(I) - 1/(N+1)|**V
C   WHERE C(1),...,C(N+1) ARE THE N+1 COVERAGES.
C   SAMPLE SIZE N MUST BE LESS THAN 9001.
C
      WRITE(*,1) 'THE NULL HYPOTHESIS SPECIFIES THAT ',
     +           'CONTINUOUS RANDOM VARIABLE X HAS',
     +           'CUMULATIVE DISTRIBUTION FUNCTION ',
     +           'F(X).  IF Y1 < ... < YN ARE THE',
     +           'ORDER STATISTICS OF X1, ..., XN, ',
     +           'LET P(I) = F(XI) FOR I = 1, ..., N.',
     +           'THE P-VALUE HERE IS BASED ON THE ',
     +           'V-POWER COVERAGE GOODNESS-OF-FIT',
     +           'TEST STATISTIC (T).  THE INPUT DATA ',
     +           'FILE IS THE SEQUENTIAL LIST GIVEN',
     +           'BY N,P(1), ..., P(N).  ',
     +           'ENTER NAME OF INPUT FILE.'
    1 FORMAT(//,T2,A,A,/,T2,A,A,/,T2,A,A,/,T2,A,A,/,T2,A,A,/,T2,A,A)
      READ(*,2) FIN
    2 FORMAT(A)
      WRITE(*,3) 'ENTER NAME OF OUTPUT FILE.'
    3 FORMAT(/,T2,A)
      READ(*,2) FOUT
      WRITE(*,*) 'ENTER (1) V > 0, (2) NUMBER OF SIMULATIONS, AND (3)'
      WRITE(*,*) 'RANDOM NUMBER GENERATOR SEED (A POSITIVE INTEGER):'
      READ(*,*) V,LS,ISEED
      E = V/2
      ZERO = 0.0D0
      ONE = 1.0D0
      OPEN(4,FILE=FIN)
      READ(4,*) N
      READ(4,*) (P(I),I = 1,N)
      CLOSE(4)
      CALL ORDER(N,P)
      C(1) = P(1)
      DO 4 I = 2,N
         C(I) = P(I)-P(I-1)
    4 CONTINUE
      C(N+1) = ONE-P(N)
      EC = ONE/(N+1)
      CV = ZERO
      DO 5 I = 1,N+1
         CV = CV+((C(I)-EC)**2)**E
    5 CONTINUE
      CV = CV*(ONE-1.0D-12)
C
      K = 24
      I1 = MOD(IABS(ISEED),177)+1
      J1 = MOD(IABS(ISEED),167)+1
      K1 = MOD(IABS(ISEED),157)+1
      L1 = MOD(IABS(ISEED),147)+1
      DO 1230 II = 1,17
        S = ZERO
        T = 0.5D0
        DO 1220 JJ = 1,K
          M1 = MOD(MOD(I1*J1,179)*K1,179)
          I1 = J1
          J1 = K1
          K1 = M1
          L1 = MOD(53*L1+1,169)
          IF(MOD(L1*M1,64).GE.32) S = S+T
          T = T/2
 1220   CONTINUE
        U(II) = S
 1230 CONTINUE
C
      COUNT = ZERO
      DO 1500 IS = 1,LS
         DO 1410 I = 1,N
            P(I) = UNI(U)
 1410    CONTINUE
         CALL ORDER(N,P)
         CN = ((P(1)-EC)**2)**E+((ONE-P(N)-EC)**2)**E
         DO 1420 I = 2,N
            CN = CN+((P(I)-P(I-1)-EC)**2)**E
 1420    CONTINUE
         IF(CN.GT.CV) COUNT = COUNT+ONE
 1500 CONTINUE
      PVAL = COUNT/LS
      OPEN(7,FILE=FOUT,STATUS='NEW')
      WRITE(7,1900) '*** VCGF TEST ***'
 1900 FORMAT(T15,A)
      WRITE(7,2000) 'OBSERVED VALUE OF VCGF TEST STATISTIC = ',CV
      WRITE(7,2000) 'POWER OF |C(I) - 1/(N+1)| {I = 1,N+1} = ',V
 2000 FORMAT(T5,A,G15.8)
      WRITE(7,2000) 'ESTIMATED VCGF TEST P-VALUE = ',PVAL
      WRITE(7,2100) 'NUMBER OF INDEPENDENT VARIABLES = ',N
 2100 FORMAT(T5,A,I9)
      WRITE(7,2100) 'NUMBER OF BOOTSTRAP REPLICATIONS = ',LS
      WRITE(7,2100) 'RANDOM NUMBER GENERATOR SEED = ',ISEED
      CLOSE(7)
      END
C
C
      DOUBLE PRECISION FUNCTION UNI(U)
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION U(17)
      PARAMETER(CSAVE = 2.1602869033813D-2, CD = 0.45623308420181D0,
     +    CM = 0.99999982118607D0)
      SAVE I,J,C
      DATA I,J,C/17,5,CSAVE/
C
      UNI = U(I)-U(J)
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      U(I) = UNI
      I = I-1
      IF(I.EQ.0) I = 17
      J = J-1
      IF(J.EQ.0) J = 17
C
      C = C-CD
      IF(C.LT.0.0D0) C = C+CM
C
      UNI = UNI-C
      IF(UNI.LT.0.0D0) UNI = UNI+1.0D0
      RETURN
      END
C
C
      SUBROUTINE ORDER(N,P)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  N UNIVARIATE VALUES ARE ORDERED FROM SMALLEST TO LARGEST
C
      DIMENSION  P(9000)
C
      J = N
      K = J/2+1
   10 K = K-1
      W = P(K)
      GO TO 30
   20 W = P(J)
      P(J) = P(1)
      J = J-1
   30 II = K
   40 L = 2*II
      IF(L-J) 50,60,70
   50 IF(P(L+1).GE.P(L)) L = L+1
   60 IF(W.GE.P(L)) GO TO 70
      P(II) = P(L)
      II = L
      GO TO 40
   70 P(II) = W
      IF(K.GT.1) GO TO 10
      IF(J.GE.2) GO TO 20
      RETURN
      END
