      PROGRAM M20
      IMPLICIT REAL*8(A-H,O-Z)
C
C     PROGRAM M20 IS A DRIVER PROGRAM, CALLING A SPECIFIC
C     SUBROUTINE TO COMPUTE FISHER'S EXACT PROBABILITY FOR
C     20 CELL GOODNESS-OF-FIT TABLES.
C
      DIMENSION  L(20),P(20),PL(20)
C
C     COMPUTE FISHER'S EXACT TEST FOR 20 CELL G-O-F TABLES
C
      WRITE(*,2) 'FISHER''S EXACT TEST FOR SPARSE 20 CELL G-O-F TABLES'
    2 FORMAT(/,T2,A,/)
      WRITE(*,3) 'ENTER L1, L2, L3, L4, L5, L6, L7, L8, L9, L10, L11,',
     +           ' L12, L13, L14, L15, L16 L17, L18, L19 & L20:'
    3 FORMAT(T2,A,A)
      READ(*,*)  L(1),L(2),L(3),L(4),L(5),L(6),L(7),L(8),L(9),L(10),
     +           L(11),L(12),L(13),L(14),L(15),L(16),L(17),L(18),
     +           L(19),L(20)
      WRITE(*,3) 'ENTER P1, P2, P3, P4, P5, P6, P7, P8, P9, P10, P11,',
     +           ' P12, P13, P14, P15, P16, P17, P18 & P19:'
      READ(*,*)  P(1),P(2),P(3),P(4),P(5),P(6),P(7),P(8),P(9),P(10),
     +           P(11),P(12),P(13),P(14),P(15),P(16),P(17),P(18),P(19)
      P(20) = 1.0D0-P(1)-P(2)-P(3)-P(4)-P(5)-P(6)-P(7)-P(8)-P(9)-P(10)
     +        -P(11)-P(12)-P(13)-P(14)-P(15)-P(16)-P(17)-P(18)-P(19)
      N = 0
      DO 4 I = 1,20
           PL(I) = DLOG(P(I))
           N = N+L(I)
    4 CONTINUE
      WRITE(*,5)
    5 FORMAT(T15,'FISHER''S EXACT TEST FOR 20 CELL G-O-F TABLES',/)
      WRITE(*,6) '20 CELL VALUES: ',L(1),L(2),L(3),L(4),L(5),L(6),L(7),
     +   L(8),L(9),L(10),L(11),L(12),L(13),L(14),L(15),L(16),L(17),
     +   L(18),L(19),L(20)
    6 FORMAT(T2,A,20I3)
      WRITE(*,7) 'TOTAL OF ALL 20 CELL VALUES: ',N
    7 FORMAT(T2,A,I12)
      CALL FE20(L,P,PL,N,PT,CST,CZT,TLR,PFE,PCS,PZS,PLR,ITAB,
     +          IFE,ICS,IZS,ILR,CHECK)
      WRITE(*,7) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',ITAB
      WRITE(*,7) '# OF TABLES YIELDING FISHER''S EXACT P-VALUE = ',IFE
      WRITE(*,8) 'P(OBSERVED TABLE) = ',PT
    8 FORMAT(T2,A,D16.9)
      WRITE(*,8) 'FISHER''S EXACT 20 CELL G-O-F TEST P-VALUE = ',PFE
      WRITE(*,7) '# OF TABLES YIELDING EXACT CHI-SQUARED P-VALUE = ',ICS
      IF(CST.LT.0.0D0) CST = 0.0D0
      WRITE(*,8) 'OBSERVED CHI-SQUARED STATISTIC = ',CST
      WRITE(*,8) 'EXACT CHI-SQUARED 20 CELL G-O-F TEST P-VALUE = ',PCS
      WRITE(*,7) '# OF TABLES YIELDING EXACT ZELTERMAN P-VALUE = ',IZS
      IF(CZT.LT.0.0D0) CZT = 0.0D0
      WRITE(*,8) 'OBSERVED ZELTERMAN STATISTIC = ',CZT
      WRITE(*,8) 'EXACT ZELTERMAN 20 CELL G-O-F TEST P-VALUE = ',PZS
      WRITE(*,7) '# OF TABLES YIELDING EXACT LR TEST P-VALUE = ',ILR
      IF(TLR.LT.0.0D0) TLR = 0.0D0
      WRITE(*,8) 'OBSERVED LR TEST STATISTIC = ',TLR
      WRITE(*,8) 'EXACT LR TEST 20 CELL G-O-F TEST P-VALUE = ',PLR
      WRITE(*,8) 'SUM OF ALL CONFIGURATION PROBABILITIES  = ',CHECK
      END
C
C
      SUBROUTINE FE20(L,P,PL,N,PT,CST,CZT,TLR,PFE,PCS,PZS,PLR,ITAB,
     +                IFE,ICS,IZS,ILR,CHECK)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  A FISHER'S EXACT TEST FOR TESTING GOODNESS-OF-FIT FOR 20 CELLS.
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        L     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        PFE   -- OUTPUT, FISHER'S EXACT PROBABILITY VALUE (P-VALUE)
C        PT    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        IFE   -- OUTPUT, # OF TABLES COMPRISING FISHER'S EXACT P-VALUE
C        CHECK -- OUTPUT, SUM OF ALL CONFIGURATION PROBABILITIES
C     USAGE:
C        SUBROUTINE FE20 COMPUTES THE FISHER EXACT PROBABILITY TEST
C        FOR THE GOODNESS-OF-FIT OF 20 OBSERVED CELL FREQUENCIES
C
      DIMENSION L(20),P(20),PL(20)
C
      Z = FAC(N)
C  DETERMINE THE PROBABILITY OF THE OBSERVED 20 CELL FREQUENCIES.
      THISP = PR(Z,PL,L)*(1.0D0-1.0D-12)
      CST = CS(N,P,L)*(1.0D0-1.0D-12)
      IF(CST.LT.1.0D-11) CST = -1.0D-11
      CZT = CZ(N,P,L)*(1.0D0-1.0D-12)
      IF(CZT.LT.1.0D-11) CZT = -1.0D-11
      TLR = ZL(N,P,L)*(1.0D0-1.0D-12)
      IF(TLR.LT.1.0D-11) TLR = -1.0D-11
C  INITIALIZATION OF VARIOUS VALUES FOR SUBSEQUENT SUMMATIONS.
      ITAB = 0
      IFE = 0
      ICS = 0
      IZS = 0
      ILR = 0
      PFE = 0.0D0
      PCS = 0.0D0
      PZS = 0.0D0
      PLR = 0.0D0
      CHECK = 0.0D0
C  19 MAJOR LOOPS DETERMINE THE VALID TABLES FROM ALL VALUES OF 19 SPECIFIED
C  CELLS (THE REMAINING CELL IS OBTAINED FROM THE 19 SPECIFIED CELLS).
      DO 40 I1 = 0,N
      M2 = N-I1
      DO 39 I2 = 0,M2
      M3 = N-I1-I2
      DO 38 I3 = 0,M3
      M4 = N-I1-I2-I3
      DO 37 I4 = 0,M4
      M5 = N-I1-I2-I3-I4
      DO 36 I5 = 0,M5
      M6 = N-I1-I2-I3-I4-I5
      DO 35 I6 = 0,M6
      M7 = N-I1-I2-I3-I4-I5-I6
      DO 34 I7 = 0,M7
      M8 = N-I1-I2-I3-I4-I5-I6-I7
      DO 33 I8 = 0,M8
      M9 = N-I1-I2-I3-I4-I5-I6-I7-I8
      DO 32 I9 = 0,M9
      M10 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9
      DO 31 I10 = 0,M10
      M11 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10
      DO 30 I11 = 0,M11
      M12 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11
      DO 29 I12 = 0,M12
      M13 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12
      DO 28 I13 = 0,M13
      M14 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12-I13
      DO 27 I14 = 0,M14
      M15 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12-I13-I14
      DO 26 I15 = 0,M15
      M16 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12-I13-I14-I16
      DO 25 I16 = 0,M16
      M17 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12-I13-I14-I16-I17
      DO 24 I17 = 0,M17
      M18 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12-I13-I14-I16-I17-I18
      DO 23 I18 = 0,M18
      M19 = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12-I13-I14-I16-I17-I18
     +       -I19
      DO 22 I19 = 0,M19
      L(1) = I1
      L(2) = I2
      L(3) = I3
      L(4) = I4
      L(5) = I5
      L(6) = I6
      L(7) = I7
      L(8) = I8
      L(9) = I9
      L(10) = I10
      L(11) = I11
      L(12) = I12
      L(13) = I13
      L(14) = I14
      L(15) = I15
      L(16) = I16
      L(17) = I17
      L(18) = I18
      L(19) = I19
      L(20) = N-I1-I2-I3-I4-I5-I6-I7-I8-I9-I10-I11-I12-I13-I14-I15
     +         -I16-I17-I18-I19
C
       ITAB = ITAB+1
       CP = PR(Z,PL,L)
       EXCP = DEXP(CP)
       CHECK = CHECK+EXCP
       IF(CP.LT.THISP) THEN
        IFE = IFE+1
        PFE = PFE+EXCP
       ENDIF
       CC = CS(N,P,L)
       IF(CC.GT.CST) THEN
        ICS = ICS+1
        PCS = PCS+EXCP
       ENDIF
       U = CZ(N,P,L)
       IF(U.GT.CZT) THEN
        IZS = IZS+1
        PZS = PZS+EXCP
       ENDIF
       W = ZL(N,P,L)
       IF(W.GT.TLR) THEN
        ILR = ILR+1
        PLR = PLR+EXCP
       ENDIF
   22 CONTINUE
   23 CONTINUE
   24 CONTINUE
   25 CONTINUE
   26 CONTINUE
   27 CONTINUE
   28 CONTINUE
   29 CONTINUE
   30 CONTINUE
   31 CONTINUE
   32 CONTINUE
   33 CONTINUE
   34 CONTINUE
   35 CONTINUE
   36 CONTINUE
   37 CONTINUE
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
      PT = DEXP(THISP)
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION PR(Z,PL,L)
C  OBTAIN SUM OF LOG PROBABILITY FOR 20 CELLS.
      DOUBLE PRECISION  Z,FAC,PL(20)
      INTEGER  I,L(20)
      PR = Z
      DO 10 I = 1,20
         PR = PR-FAC(L(I))+L(I)*PL(I)
   10 CONTINUE
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION CS(N,P,L)
C  OBTAIN CHI-SQUARED STATISTIC
      DOUBLE PRECISION  ONE,P(20)
      INTEGER  I,L(20)
      ONE = 1.0D0
      CS = 0.0D0
      DO 10 I = 1,20
         CS = CS+(ONE*L(I))**2/P(I)
   10 CONTINUE
      CS = CS/N-N
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION CZ(N,P,L)
C  OBTAIN ZELTERMAN STATISTIC
      DOUBLE PRECISION  ONE,P(20)
      INTEGER  I,L(20)
      ONE = 1.0D0
      CZ = 0.0D0
      DO 10 I = 1,20
         CZ = CZ+ONE*L(I)*(L(I)-1)/P(I)
   10 CONTINUE
      CZ = CZ/N+20-N
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION ZL(N,P,L)
C  OBTAIN LR TEST STATISTIC
      DOUBLE PRECISION  X,RZ,P(20)
      INTEGER  I,L(20)
      ZL = 0.0D0
      DO 10 I = 1,20
         X = 1.0D0*L(I)
         IF(X.GT.1.5D0) THEN
            RZ = X*DLOG(X)
         ELSE
            RZ = 0.0D0
         ENDIF
         ZL = ZL+RZ-DLOG(P(I)*N)*L(I)
   10 CONTINUE
      ZL = ZL*2
      RETURN
      END
