      PROGRAM E3KAP2
      IMPLICIT REAL*8(A-H,O-Z)
C
C     PROGRAM E3KAP2 IS A DRIVER PROGRAM, CALLING A SPECIFIC
C     SUBROUTINE TO COMPUTE THE EXACT LOWER AND UPPER P-VALUES
C     OF THE WEIGHTED KAPPA TEST ASSOCIATED WITH 3 X 3
C     CROSS-CLASSIFICATION TABLES.
C
      CHARACTER  FOUT*40
      DIMENSION  N(3,3),W(3,3)
C
C     COMPUTE EXACT LOWER AND UPPER P-VALUES OF KAPPA TEST FOR 3 X 3 TABLES
C
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,1) FOUT
    1 FORMAT(A)
      WRITE(*,2) 'EXACT KAPPA TEST FOR SPARSE 3 X 3 TABLES'
    2 FORMAT(/,T2,A,/)
      DO 3 I = 1,3
         WRITE(*,*) 'ENTER FREQUENCIES FOR ROW',I
         READ(*,*) N(I,1),N(I,2),N(I,3)
    3 CONTINUE
      DO 4 I = 1,3
         WRITE(*,*) 'ENTER WEIGHTS FOR ROW',I
         READ(*,*) W(I,1),W(I,2),W(I,3)
    4 CONTINUE
      CALL EK3(N,W,PT,WO,CKL,CKU,CK,PKL,PKU,PK,CTAB,CHECK)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      WRITE(6,7) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',CTAB
    7 FORMAT(T5,A,D20.13)
      WRITE(6,7) 'P(OBSERVED TABLE) = ',PT
      WRITE(6,7) 'OBSERVED KAPPA TEST STATISTIC = ',WO
      WRITE(6,7) 'EXPECTED KAPPA TEST STATISTIC = ',0.0D0
      WRITE(6,7) '# OF TABLES YIELDING LOWER TAIL KAPPA P-VALUE = ',CKL
      WRITE(6,7) '# OF TABLES YIELDING UPPER TAIL KAPPA P-VALUE = ',CKU
      WRITE(6,7) '# OF TABLES YIELDING TWO-SIDED P-VALUE = ',CK
      WRITE(6,7) 'EXACT LOWER TAIL KAPPA P-VALUE = ',PKL
      WRITE(6,7) 'EXACT UPPER TAIL KAPPA P-VALUE = ',PKU
      WRITE(6,7) 'EXACT TWO-SIDED KAPPA P-VALUE = ',PK
      WRITE(6,7) 'SUM OF ALL CONFIGURATION PROBABILITIES = ',CHECK
      END
C
C
      SUBROUTINE EK3(L,W,PT,WO,CKL,CKU,CK,PKL,PKU,PK,CTAB,CHECK)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  AN EXACT KAPPA TEST FOR LOWER AND UPPER P-VALUES ASSOCIATED
C  WITH A 3 X 3 CONTINGENCY TABLE.
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        L     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        PT    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        ITAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        CHECK -- OUTPUT, TOTAL SUM OF PROBABILITIES OF THE TABLE
C     USAGE:
C        SUBROUTINE EK3 COMPUTES THE EXACT LOWER AND UPPER P-VALUES
C        FOR KAPPA IN A 3 X 3 CROSS-CLASSIFICATION TABLE
C
      DIMENSION K(3,3),L(3,3),W(3,3),IR(3),IC(3)
C
C  OBTAIN THE ROW AND COLUMN MARGINAL TOTALS AND THE GRAND TOTAL.
C  FIND THE COMMON PART OF EACH TABLES EXACT PROBABILITY VALUE.
C
      ZERO = 0.0D0
      ONE = 1.0D0
      IT = 0
      PSTART = ZERO
      DO 2 I = 1,3
         IR(I) = 0
         DO 1 J = 1,3
            IR(I) = IR(I)+L(I,J)
    1    CONTINUE
         IT = IT+IR(I)
         PSTART = PSTART+FAC(IR(I))
    2 CONTINUE
      DO 4 J = 1,3
         IC(J) = 0
         DO 3 I = 1,3
            IC(J) = IC(J)+L(I,J)
    3    CONTINUE
         PSTART = PSTART+FAC(IC(J))
    4 CONTINUE
      TOT = ONE*IT
      WO = ZERO
      EW = ZERO
      DO 8 I = 1,3
         DO 7 J = 1,3
            WO = WO+W(I,J)*L(I,J)/IT
            EW = EW+W(I,J)*IR(I)*IC(J)/IT/IT
    7    CONTINUE
    8 CONTINUE
      WO = ONE-WO/EW
      IF(WO.GT.ZERO) THEN
         WOL = WO*(ONE+1.0D-10)
         WOU = WO*(ONE-1.0D-10)
         GO TO 9
      END IF
      IF(WO.LT.ZERO) THEN
         WOL = WO*(ONE-1.0D-10)
         WOU = WO*(ONE+1.0D-10)
         GO TO 9
      END IF
      IF(WO.EQ.ZERO) THEN
         WOL = 1.0D-11
         WOU = -1.0D-11
      END IF
    9 WO2 = DABS(WO)-1.0D-11
      PSTART = PSTART-FAC(IT)
C  DETERMINE THE PROBABILITY OF THE OBSERVED TABLE.
      PT = DEXP(PSTART-CELLFAC(L))
C  INITIALIZATION OF VARIOUS VALUES FOR SUBSEQUENT SUMMATIONS.
      CTAB = ZERO
      CKL = ZERO
      CKU = ZERO
      CK = ZERO
      PKL = ZERO
      PKU = ZERO
      PK = ZERO
      CHECK = ZERO
C  4 MAJOR LOOPS DETERMINE THE VALID TABLES FROM ALL VALUES OF 4 SPECIFIED
C  CELLS (THE 5 REMAINING CELLS ARE OBTAINED FROM THE 4 SPECIFIED CELLS).
      M11 = MIN0(IR(1),IC(1))
      L11 = MAX0(IR(1)+IC(1)-IT,0)
      DO 40 I11 = L11,M11
       K(1,1) = I11
      M21 = MIN0(IR(2),IC(1)-I11)
      L21 = MAX0(IR(2)+IC(1)-I11-IT+IR(1),0)
      DO 39 I21 = L21,M21
       K(2,1) = I21
       K(3,1) = IC(1)-I11-I21
      M12 = MIN0(IR(1)-I11,IC(2))
      L12 = MAX0(IR(1)-I11+IC(2)-IT+IC(1),0)
      DO 38 I12 = L12,M12
       K(1,2) = I12
      M22 = MIN0(IR(2)-I21,IC(2)-I12)
      L22 = MAX0(IR(2)-I21+IC(2)-I12-IT+IR(1)+IC(1)-I11,0)
      DO 37 I22 = L22,M22
       K(2,2) = I22
       K(3,2) = IC(2)-I12-I22
       K(1,3) = IR(1)-I11-I12
       K(2,3) = IR(2)-I21-I22
       K(3,3) = IC(3)-K(1,3)-K(2,3)
       CTAB = CTAB+ONE
       CP = PSTART-CELLFAC(K)
       EXCP = DEXP(CP)
       CHECK = CHECK+EXCP
       WX = ZERO
       DO 28 I = 1,3
        DO 27 J = 1,3
         WX = WX+W(I,J)*K(I,J)/IT
   27   CONTINUE
   28  CONTINUE
       WX = ONE-WX/EW
       IF(WX.LT.WOL) THEN
        CKL = CKL+ONE
        PKL = PKL+EXCP
       END IF
       IF(WX.GT.WOU) THEN
        CKU = CKU+ONE
        PKU = PKU+EXCP
       END IF
       IF(DABS(WX).GT.WO2) THEN
        CK = CK+ONE
        PK = PK+EXCP
       END IF
   37 CONTINUE
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C  ...........................................................................
      DOUBLE PRECISION FUNCTION CELLFAC(N)
C  OBTAIN SUM OF LOG FACTORIALS FOR 9 CELLS.
      DOUBLE PRECISION  FAC
      INTEGER  I,J,N(3,3)
      CELLFAC = 0.0D0
      DO 20 I = 1,3
         DO 10 J = 1,3
               CELLFAC = CELLFAC+FAC(N(I,J))
   10    CONTINUE
   20 CONTINUE
      RETURN
      END
