      PROGRAM F2X14
      IMPLICIT REAL*8(A-H,O-Z)
C
C     PROGRAM F2X14 IS A DRIVER PROGRAM, CALLING A SPECIFIC SUBROUTINE
C     TO COMPUTE FISHER'S EXACT, EXACT CHI-SQUARE, EXACT ZELTERMAN,
C     AND EXACT LOG-LIKELIHOOD RATIO TEST P-VALUES FOR 2 X 14
C     CROSS-CLASSIFICATION TABLES.
C
      CHARACTER  FOUT*40
      DIMENSION  N(2,14)
C
C     COMPUTE FISHER'S EXACT TEST FOR 2 X 14 TABLES
C
      WRITE(*,*) 'ENTER FILE NAME FOR OUPUT RESULTS'
      READ(*,1) FOUT
    1 FORMAT(A)
      WRITE(*,2) 'FISHER''S EXACT TEST FOR SPARSE 2 X 14 TABLES'
    2 FORMAT(/,T2,A,/)
      WRITE(*,3) 'ENTER N11, N12, N13, N14, N15, N16, N17, N18, N19',
     +           ' N110, N111, N112, N113 & N114'
    3 FORMAT(T2,A,/,A)
      READ(*,*)  N(1,1),N(1,2),N(1,3),N(1,4),N(1,5),N(1,6),N(1,7),
     +           N(1,8),N(1,9),N(1,10),N(1,11),N(1,12),N(1,13),N(1,14)
      WRITE(*,3) 'ENTER N21, N22, N23, N24, N25, N26, N27, N28, N29',
     +           ' N210, N211, N212, N213 & N214'
      READ(*,*)  N(2,1),N(2,2),N(2,3),N(2,4),N(2,5),N(2,6),N(2,7),
     +           N(2,8),N(2,9),N(2,10),N(2,11),N(2,12),N(2,13),N(2,14)
      CALL FE2X14(N,PT,CST,CZT,ZLT,PFE,PCS,PZS,PLS,CTAB,
     +  CFE,CCS,CZS,CLS,CHECK)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      WRITE(6,5)
    5 FORMAT(T15,'FISHER''S EXACT TEST FOR 2 X 14 TABLES',/)
      WRITE(6,6) '1ST ROW OF 2 X 14 TABLE: ',N(1,1),N(1,2),N(1,3),
     +  N(1,4),N(1,5),N(1,6),N(1,7),N(1,8),N(1,9),N(1,10),N(1,11),
     +  N(1,12),N(1,13),N(1,14)
    6 FORMAT(T5,A,/,T2,14I4)
      WRITE(6,6) '2ND ROW OF 2 X 14 TABLE: ',N(2,1),N(2,2),N(2,3),
     +  N(2,4),N(2,5),N(2,6),N(2,7),N(2,8),N(2,9),N(2,10),N(2,11),
     +  N(2,12),N(2,13),N(2,14)
      WRITE(6,7) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',CTAB
    7 FORMAT(T5,A,D20.13)
      WRITE(6,7) '# OF TABLES YIELDING FISHER''S EXACT P-VALUE = ',CFE
      WRITE(6,7) 'P(OBSERVED TABLE) = ',PT
      WRITE(6,7) 'FISHER''S EXACT 2 X 14 TABLE P-VALUE = ',PFE
      WRITE(6,7) '# OF TABLES YIELDING EXACT CHI-SQUARED P-VALUE = ',CCS
      IF(CST.LT.0.0D0) CST = 0.0D0
      WRITE(6,7) 'OBSERVED CHI-SQUARED STATISTIC = ',CST
      WRITE(6,7) 'EXACT CHI-SQUARED 2 X 14 TABLE P-VALUE = ',PCS
      WRITE(6,7) '# OF TABLES YIELDING EXACT ZELTERMAN P-VALUE = ',CZS
      IF(CZT.LT.0.0D0) CZT = 0.0D0
      WRITE(6,7) 'OBSERVED ZELTERMAN STATISTIC = ',CZT
      WRITE(6,7) 'EXACT ZELTERMAN 2 X 14 TABLE P-VALUE = ',PZS
      WRITE(6,7) '# OF TABLES YIELDING EXACT LOG LR P-VALUE = ',CLS
      IF(ZLT.LT.0.0D0) ZLT = 0.0D0
      WRITE(6,7) 'OBSERVED LOG-LR STATISTIC = ',ZLT
      WRITE(6,7) 'EXACT LOG LR 2 X 14 TABLE P-VALUE = ',PLS
      WRITE(6,7) 'SUM OF ALL CONFIGURATION PROBABILITIES  = ',CHECK
      END
C
C
      SUBROUTINE FE2X14(L,PT,CST,CZT,ZLT,PFE,PCS,PZS,PLS,CTAB,
     +   CFE,CCS,CZS,CLS,CHECK)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  A FISHER'S EXACT TEST FOR TESTING THE MUTUAL INDEPENDENCE OF
C  CLASSIFICATIONS IN A 2 X 14 CONTINGENCY TABLE.
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        L     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        PFE   -- OUTPUT, FISHER'S EXACT PROBABILITY VALUE (P-VALUE)
C        PT    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        CTAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        CFE   -- OUTPUT, # OF TABLES COMPRISING FISHER'S EXACT P-VALUE
C     USAGE:
C        SUBROUTINE FE2X14 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 2 X 14 CROSS-CLASSIFICATION TABLES
C
      DIMENSION K(2,14),L(2,14),IR(2),IC(14)
C
C  OBTAIN THE ROW AND COLUMN MARGINAL TOTALS AND THE GRAND TOTAL.
C  FIND THE COMMON PART OF EACH TABLES EXACT PROBABILITY VALUE.
C
      ZERO = 0.0D0
      ONE = 1.0D0
      IT = 0
      PSTART = ZERO
      DO 2 I = 1,2
         IR(I) = 0
         DO 1 J = 1,14
            IR(I) = IR(I)+L(I,J)
    1    CONTINUE
         IT = IT+IR(I)
         PSTART = PSTART+FAC(IR(I))
    2 CONTINUE
      DO 4 J = 1,14
         IC(J) = 0
         DO 3 I = 1,2
            IC(J) = IC(J)+L(I,J)
    3    CONTINUE
         PSTART = PSTART+FAC(IC(J))
    4 CONTINUE
      PSTART = PSTART-FAC(IT)
      CZ = RZ(IT)
      DO 5 I = 1,2
         CZ = CZ-RZ(IR(I))
    5 CONTINUE
      DO 6 I = 1,14
         CZ = CZ-RZ(IC(I))
    6 CONTINUE
C  DETERMINE THE PROBABILITY OF THE OBSERVED TABLE.
      THISP = (PSTART-CELLFAC(L))*(1.0D0-1.0D-12)
      CST = CS(L,IR,IC,IT)*(1.0D0-1.0D-12)
      IF(CST.LT.1.0D-11) CST = -1.0D-11
      CZT = DS(L,IR,IC)*(1.0D0-1.0D-12)
      IF(CZT.LT.1.0D-11) CZT = -1.0D-11
      ZLT = ZL(L,CZ)*(1.0D0-1.0D-12)
      IF(ZLT.LT.1.0D-11) ZLT = -1.0D-11
C  INITIALIZATION OF VARIOUS VALUES FOR SUBSEQUENT SUMMATIONS.
      CTAB = ZERO
      CFE = ZERO
      CCS = ZERO
      CZS = ZERO
      CLS = ZERO
      PFE = ZERO
      PCS = ZERO
      PZS = ZERO
      PLS = ZERO
      CHECK = ZERO
C  13 MAJOR LOOPS DETERMINE THE VALID TABLES FROM ALL VALUES OF 13 SPECIFIED
C  CELLS (THE 15 REMAINING CELLS ARE OBTAINED FROM THE 13 SPECIFIED CELLS).
      M11 = MIN0(IR(1),IC(1))
      L11 = MAX0(IR(1)+IC(1)-IT,0)
      DO 40 I11 = L11,M11
       K(1,1) = I11
       K(2,1) = IC(1)-I11
C
      M12 = MIN0(IR(1)-I11,IC(2))
      L12 = MAX0(IR(1)-I11+IC(2)-IT+IC(1),0)
      DO 39 I12 = L12,M12
       K(1,2) = I12
       K(2,2) = IC(2)-I12
C
      M13 = MIN0(IR(1)-I11-I12,IC(3))
      L13 = MAX0(IR(1)-I11-I12+IC(3)-IT+IC(1)+IC(2),0)
      DO 38 I13 = L13,M13
       K(1,3) = I13
       K(2,3) = IC(3)-I13
C
      M14 = MIN0(IR(1)-I11-I12-I13,IC(4))
      L14 = MAX0(IR(1)-I11-I12-I13+IC(4)-IT+IC(1)+IC(2)+IC(3),0)
      DO 37 I14 = L14,M14
       K(1,4) = I14
       K(2,4) = IC(4)-I14
C
      M15 = MIN0(IR(1)-I11-I12-I13-I14,IC(5))
      L15 = MAX0(IR(1)-I11-I12-I13-I14+IC(5)-IT
     +      +IC(1)+IC(2)+IC(3)+IC(4),0)
      DO 36 I15 = L15,M15
       K(1,5) = I15
       K(2,5) = IC(5)-I15
C
      M16 = MIN0(IR(1)-I11-I12-I13-I14-I15,IC(6))
      L16 = MAX0(IR(1)-I11-I12-I13-I14-I15+IC(6)-IT
     +      +IC(1)+IC(2)+IC(3)+IC(4)+IC(5),0)
      DO 35 I16 = L16,M16
       K(1,6) = I16
       K(2,6) = IC(6)-I16
C
      M17 = MIN0(IR(1)-I11-I12-I13-I14-I15-I16,IC(7))
      L17 = MAX0(IR(1)-I11-I12-I13-I14-I15-I16+IC(7)-IT
     +      +IC(1)+IC(2)+IC(3)+IC(4)+IC(5)+IC(6),0)
      DO 34 I17 = L17,M17
       K(1,7) = I17
       K(2,7) = IC(7)-I17
C
      M18 = MIN0(IR(1)-I11-I12-I13-I14-I15-I16-I17,IC(8))
      L18 = MAX0(IR(1)-I11-I12-I13-I14-I15-I16-I17+IC(8)-IT
     +      +IC(1)+IC(2)+IC(3)+IC(4)+IC(5)+IC(6)+IC(7),0)
      DO 33 I18 = L18,M18
       K(1,8) = I18
       K(2,8) = IC(8)-I18
C
      M19 = MIN0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18,IC(9))
      L19 = MAX0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18+IC(9)-IT
     +      +IC(1)+IC(2)+IC(3)+IC(4)+IC(5)+IC(6)+IC(7)+IC(8),0)
      DO 32 I19 = L19,M19
       K(1,9) = I19
       K(2,9) = IC(9)-I19
C
      M110 = MIN0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19,IC(10))
      L110 = MAX0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19+IC(10)-IT
     +       +IC(1)+IC(2)+IC(3)+IC(4)+IC(5)+IC(6)+IC(7)+IC(8)+IC(9),0)
      DO 31 I110 = L110,M110
       K(1,10) = I110
       K(2,10) = IC(10)-I110
C
      M111 = MIN0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19-I110,IC(11))
      L111 = MAX0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19-I110+IC(11)
     +       -IT+IC(1)+IC(2)+IC(3)+IC(4)+IC(5)+IC(6)+IC(7)+IC(8)+IC(9)
     +       +IC(10),0)
      DO 30 I111 = L111,M111
       K(1,11) = I111
       K(2,11) = IC(11)-I111
C
      M112 = MIN0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19-I110-I111,
     +       IC(12))
      L112 = MAX0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19-I110-I111
     +       +IC(12)-IT+IC(1)+IC(2)+IC(3)+IC(4)+IC(5)+IC(6)+IC(7)+IC(8)
     +       +IC(9)+IC(10)+IC(11),0)
      DO 29 I112 = L112,M112
       K(1,12) = I112
       K(2,12) = IC(12)-I112
C
      M113 = MIN0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19-I110-I111
     +       -I112,IC(13))
      L113 = MAX0(IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19-I110-I111
     +       -I112+IC(13)-IT+IC(1)+IC(2)+IC(3)+IC(4)+IC(5)+IC(6)+IC(7)
     +       +IC(8)+IC(9)+IC(10)+IC(11)+IC(12),0)
      DO 28 I113 = L113,M113
       K(1,13) = I113
       K(2,13) = IC(13)-I113
C
       K(1,14) = IR(1)-I11-I12-I13-I14-I15-I16-I17-I18-I19-I110-I111
     +          -I112-I113
       K(2,14) = IR(2)-K(2,1)-K(2,2)-K(2,3)-K(2,4)-K(2,5)-K(2,6)-K(2,7)
     +          -K(2,8)-K(2,9)-K(2,10)-K(2,11)-K(2,12)-K(2,13)
C
       CTAB = CTAB+ONE
       CP = PSTART-CELLFAC(K)
       EXCP = DEXP(CP)
       CHECK = CHECK+EXCP
       IF(CP.LT.THISP) THEN
        CFE = CFE+ONE
        PFE = PFE+EXCP
       ENDIF
       Z = CS(K,IR,IC,IT)
       IF(Z.GT.CST) THEN
        CCS = CCS+ONE
        PCS = PCS+EXCP
       ENDIF
       Y = DS(K,IR,IC)
       IF(Y.GT.CZT) THEN
        CZS = CZS+ONE
        PZS = PZS+EXCP
       ENDIF
       W = ZL(K,CZ)
       IF(W.GT.ZLT) THEN
        CLS = CLS+ONE
        PLS = PLS+EXCP
       ENDIF
   28 CONTINUE
   29 CONTINUE
   30 CONTINUE
   31 CONTINUE
   32 CONTINUE
   33 CONTINUE
   34 CONTINUE
   35 CONTINUE
   36 CONTINUE
   37 CONTINUE
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
      PT = DEXP(THISP)
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION RZ(I)
C  FIND I(LOG(I))
      DOUBLE PRECISION  X
      X = 1.0D0*I
      IF(X.GT.1.5D0) THEN
         RZ = X*DLOG(X)
      ELSE
         RZ = 0.0D0
      ENDIF
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION CELLFAC(N)
C  OBTAIN SUM OF LOG FACTORIALS FOR 28 CELLS.
      DOUBLE PRECISION  FAC
      INTEGER  I,J,N(2,14)
      CELLFAC = 0.0D0
      DO 20 I = 1,2
         DO 10 J = 1,14
               CELLFAC = CELLFAC+FAC(N(I,J))
   10    CONTINUE
   20 CONTINUE
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION CS(N,IR,IC,IT)
C  OBTAIN CHI-SQUARED STATISTIC
      DOUBLE PRECISION  ZERO,ONE
      INTEGER  N(2,14),IR(2),IC(14),IT,I,J
      ZERO = 0.0D0
      ONE = 1.0D0
      CS = ZERO
      DO 20 I = 1,2
         DO 10 J = 1,14
            CS = CS+(ONE*N(I,J))**2/IR(I)/IC(J)
   10    CONTINUE
   20 CONTINUE
      CS = (CS-ONE)*IT
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION DS(N,IR,IC)
C  OBTAIN ZELTERMAN STATISTIC
      DOUBLE PRECISION  ZERO,ONE
      INTEGER  N(2,14),IR(2),IC(14),I,J
      ZERO = 0.0D0
      ONE = 1.0D0
      DS = ZERO
      DO 20 I = 1,2
         DO 10 J = 1,14
            DS = DS+ONE*N(I,J)*(N(I,J)-1)/IR(I)/IC(J)
   10    CONTINUE
   20 CONTINUE
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION ZL(N,CZ)
C  OBTAIN LOG LIKELIHOOD RATIO STATISTIC
      DOUBLE PRECISION  ZERO,CZ,RZ
      INTEGER  N(2,14),I,J
      ZERO = 0.0D0
      ZL = ZERO
      DO 20 I = 1,2
         DO 10 J = 1,14
            ZL = ZL+RZ(N(I,J))
   10    CONTINUE
   20 CONTINUE
      ZL = (ZL+CZ)*2
      RETURN
      END
