      PROGRAM Y3X5
      IMPLICIT REAL*8(A-H,O-Z)
C
C     PROGRAM F3X5 IS A DRIVER PROGRAM, CALLING A SPECIFIC SUBROUTINE
C     TO COMPUTE FISHER'S EXACT, EXACT CHI-SQUARE, EXACT TAU-A, EXACT
C     TAU-B, EXACT ZELTERMAN,AND EXACT LOG-LIKELIHOOD RATIO TEST P-VALUES
C     FOR 3 X 5 CROSS-CLASSIFICATION TABLES.
C
      CHARACTER  FOUT*40
      DIMENSION  N(3,5)
C
C     COMPUTE FISHER'S EXACT TEST FOR 3 X 5 TABLES
C
      WRITE(*,*) 'ENTER FILE NAME FOR OUTPUT RESULTS'
      READ(*,1) FOUT
    1 FORMAT(A)
      WRITE(*,2) 'FISHER''S EXACT TEST FOR SPARSE 3 X 5 TABLES'
    2 FORMAT(/,T2,A,/)
      WRITE(*,3) 'ENTER N11, N12, N13, N14 & N15'
    3 FORMAT(T2,A)
      READ(*,*)  N(1,1),N(1,2),N(1,3),N(1,4),N(1,5)
      WRITE(*,3) 'ENTER N21, N22, N23, N24 & N25'
      READ(*,*)  N(2,1),N(2,2),N(2,3),N(2,4),N(2,5)
      WRITE(*,3) 'ENTER N31, N32, N33, N34 & N35'
      READ(*,*)  N(3,1),N(3,2),N(3,3),N(3,4),N(3,5)
      CALL FE3X5(N,PT,CST,CTA,CTB,CZT,ZLT,PFE,PCS,PTA,PTB,PZS,PLS,CTAB,
     +  CFE,CCS,CCA,CCB,CZS,CLS,CHECK)
      OPEN(6,FILE=FOUT,STATUS='NEW')
      WRITE(6,5)
    5 FORMAT(T15,'FISHER''S EXACT TEST FOR 3 X 5 TABLES',/)
      WRITE(6,6) '1ST ROW OF 3 X 5 TABLE: ',N(1,1),N(1,2),N(1,3),
     +  N(1,4),N(1,5)
    6 FORMAT(T5,A,5I4)
      WRITE(6,6) '2ND ROW OF 3 X 5 TABLE: ',N(2,1),N(2,2),N(2,3),
     +  N(2,4),N(2,5)
      WRITE(6,6) '3RD ROW OF 3 X 5 TABLE: ',N(3,1),N(3,2),N(3,3),
     +  N(3,4),N(3,5)
      WRITE(6,7) '# OF TABLES CONSISTENT WITH MARGINAL TOTALS = ',CTAB
    7 FORMAT(T5,A,D20.13)
      WRITE(6,7) '# OF TABLES YIELDING FISHER''S EXACT P-VALUE = ',CFE
      WRITE(6,7) 'P(OBSERVED TABLE) = ',PT
      WRITE(6,7) 'FISHER''S EXACT 3 X 5 TABLE P-VALUE = ',PFE
      WRITE(6,7) '# OF TABLES YIELDING EXACT CHI-SQUARED P-VALUE = ',CCS
      IF(CST.LT.0.0D0) CST = 0.0D0
      WRITE(6,7) 'OBSERVED CHI-SQUARED STATISTIC = ',CST
      WRITE(6,7) 'EXACT CHI-SQUARED 3 X 5 TABLE P-VALUE = ',PCS
      WRITE(6,7) '# OF TABLES YIELDING EXACT TAU-A P-VALUE = ',CCA
      IF(CTA.LT.0.0D0) CTA = 0.0D0
      WRITE(6,7) 'OBSERVED TAU-A STATISTIC = ',CTA
      WRITE(6,7) 'EXACT TAU-A 3 X 5 TABLE P-VALUE = ',PTA
      WRITE(6,7) '# OF TABLES YIELDING EXACT TAU-B P-VALUE = ',CCB
      IF(CTB.LT.0.0D0) CTB = 0.0D0
      WRITE(6,7) 'OBSERVED TAU-B STATISTIC = ',CTB
      WRITE(6,7) 'EXACT TAU-B 3 X 5 TABLE P-VALUE = ',PTB
      WRITE(6,7) '# OF TABLES YIELDING EXACT ZELTERMAN P-VALUE = ',CZS
      IF(CZT.LT.0.0D0) CZT = 0.0D0
      WRITE(6,7) 'OBSERVED ZELTERMAN STATISTIC = ',CZT
      WRITE(6,7) 'EXACT ZELTERMAN 3 X 5 TABLE P-VALUE = ',PZS
      WRITE(6,7) '# OF TABLES YIELDING EXACT LOG LR P-VALUE = ',CLS
      IF(ZLT.LT.0.0D0) ZLT = 0.0D0
      WRITE(6,7) 'OBSERVED LOG-LR STATISTIC = ',ZLT
      WRITE(6,7) 'EXACT LOG LR 3 X 5 TABLE P-VALUE = ',PLS
      WRITE(6,7) 'SUM OF ALL CONFIGURATION PROBABILITIES = ',CHECK
      END
C
C
      SUBROUTINE FE3X5(L,PT,CST,CTA,CTB,CZT,ZLT,PFE,PCS,PTA,PTB,PZS,PLS,
     +   CTAB,CFE,CCS,CCA,CCB,CZS,CLS,CHECK)
      IMPLICIT REAL*8(A-H,O-Z)
C
C  A FISHER'S EXACT TEST FOR TESTING THE MUTUAL INDEPENDENCE OF
C  CLASSIFICATIONS IN A 3 X 5 CONTINGENCY TABLE.
C
C     DESCRIPTION OF VARIABLES IN CALL LIST:
C        L     -- INPUT,  ARRAY CONTAINING THE CELL FREQUENCIES
C        PFE   -- OUTPUT, FISHER'S EXACT PROBABILITY VALUE (P-VALUE)
C        PT    -- OUTPUT, POINT PROBABILITY OF OBSERVED TABLE
C        CTAB  -- OUTPUT, # OF TABLES CONSISTENT WITH MARGINAL TOTALS
C        CFE   -- OUTPUT, # OF TABLES COMPRISING FISHER'S EXACT P-VALUE
C     USAGE:
C        SUBROUTINE FE3X5 COMPUTES THE FISHER EXACT PROBABILITY
C        TEST FOR 3 X 5 CROSS-CLASSIFICATION TABLES
C
      DIMENSION K(3,5),L(3,5),IR(3),IC(5)
C
C  OBTAIN THE ROW AND COLUMN MARGINAL TOTALS AND THE GRAND TOTAL.
C  FIND THE COMMON PART OF EACH TABLES EXACT PROBABILITY VALUE.
C
      ZERO = 0.0D0
      ONE = 1.0D0
      IT = 0
      SR = ZERO
      SC = ZERO
      PSTART = ZERO
      DO 2 I = 1,3
         IR(I) = 0
         DO 1 J = 1,5
            IR(I) = IR(I)+L(I,J)
    1    CONTINUE
         IT = IT+IR(I)
         SR = SR+(ONE*IR(I))**2
         PSTART = PSTART+FAC(IR(I))
    2 CONTINUE
      DO 4 J = 1,5
         IC(J) = 0
         DO 3 I = 1,3
            IC(J) = IC(J)+L(I,J)
    3    CONTINUE
         SC = SC+(ONE*IC(J))**2
         PSTART = PSTART+FAC(IC(J))
    4 CONTINUE
      PSTART = PSTART-FAC(IT)
      CZ = RZ(IT)
      DO 5 I = 1,3
         CZ = CZ-RZ(IR(I))
    5 CONTINUE
      DO 6 I = 1,5
         CZ = CZ-RZ(IC(I))
    6 CONTINUE
C  DETERMINE THE PROBABILITY OF THE OBSERVED TABLE.
      THISP = (PSTART-CELLFAC(L))*(1.0D0-1.0D-12)
      CST = CS(L,IR,IC,IT)*(1.0D0-1.0D-12)
      IF(CST.LT.1.0D-11) CST = -1.0D-11
      CTA = CA(L,IC,SR,IT)*(1.0D0-1.0D-12)
      IF(CTA.LT.1.0D-11) CTA = -1.0D-11
      CTB = CB(L,IR,SC,IT)*(1.0D0-1.0D-12)
      IF(CTB.LT.1.0D-11) CTB = -1.0D-11
      CZT = DS(L,IR,IC)*(1.0D0-1.0D-12)
      IF(CZT.LT.1.0D-11) CZT = -1.0D-11
      ZLT = ZL(L,CZ)*(1.0D0-1.0D-12)
      IF(ZLT.LT.1.0D-11) ZLT = -1.0D-11
C  INITIALIZATION OF VARIOUS VALUES FOR SUBSEQUENT SUMMATIONS.
      CTAB = ZERO
      CFE = ZERO
      CCS = ZERO
      CCA = ZERO
      CCB = ZERO
      CZS = ZERO
      CLS = ZERO
      PFE = ZERO
      PCS = ZERO
      PTA = ZERO
      PTB = ZERO
      PZS = ZERO
      PLS = ZERO
      CHECK = ZERO
C  8 MAJOR LOOPS DETERMINE THE VALID TABLES FROM ALL VALUES OF 8 SPECIFIED
C  CELLS (THE 7 REMAINING CELLS ARE OBTAINED FROM THE 8 SPECIFIED CELLS).
      M11 = MIN0(IR(1),IC(1))
      L11 = MAX0(IR(1)+IC(1)-IT,0)
      DO 40 I11 = L11,M11
       K(1,1) = I11
      M21 = MIN0(IR(2),IC(1)-I11)
      L21 = MAX0(IR(2)+IC(1)-I11-IT+IR(1),0)
      DO 39 I21 = L21,M21
       K(2,1) = I21
       K(3,1) = IC(1)-I11-I21
C
      M12 = MIN0(IR(1)-I11,IC(2))
      L12 = MAX0(IR(1)-I11+IC(2)-IT+IC(1),0)
      DO 38 I12 = L12,M12
       K(1,2) = I12
      M22 = MIN0(IR(2)-I21,IC(2)-I12)
      L22 = MAX0(IR(2)-I21+IC(2)-I12-IT+IR(1)+IC(1)-I11,0)
      DO 37 I22 = L22,M22
       K(2,2) = I22
       K(3,2) = IC(2)-I12-I22
C
      M13 = MIN0(IR(1)-I11-I12,IC(3))
      L13 = MAX0(IR(1)-I11-I12+IC(3)-IT+IC(1)+IC(2),0)
      DO 36 I13 = L13,M13
       K(1,3) = I13
      M23 = MIN0(IR(2)-I21-I22,IC(3)-I13)
      L23 = MAX0(IR(2)-I21-I22+IC(3)-I13-IT+IR(1)+IC(1)+IC(2)-I11-I12,0)
      DO 35 I23 = L23,M23
       K(2,3) = I23
       K(3,3) = IC(3)-I13-I23
C
      M14 = MIN0(IR(1)-I11-I12-I13,IC(4))
      L14 = MAX0(IR(1)-I11-I12-I13+IC(4)-IT+IC(1)+IC(2)+IC(3),0)
      DO 34 I14 = L14,M14
       K(1,4) = I14
      M24 = MIN0(IR(2)-I21-I22-I23,IC(4)-I14)
      L24 = MAX0(IR(2)-I21-I22-I23+IC(4)-I14-IT+IR(1)
     +      +IC(1)+IC(2)+IC(3)-I11-I12-I13,0)
      DO 33 I24 = L24,M24
       K(2,4) = I24
       K(3,4) = IC(4)-I14-I24
C
       K(1,5) = IR(1)-I11-I12-I13-I14
       K(2,5) = IR(2)-I21-I22-I23-I24
       K(3,5) = IC(5)-K(1,5)-K(2,5)
C
       CTAB = CTAB+ONE
       CP = PSTART-CELLFAC(K)
       EXCP = DEXP(CP)
       CHECK = CHECK+EXCP
       IF(CP.LT.THISP) THEN
        CFE = CFE+ONE
        PFE = PFE+EXCP
       ENDIF
       Z = CS(K,IR,IC,IT)
       IF(Z.GT.CST) THEN
        CCS = CCS+ONE
        PCS = PCS+EXCP
       ENDIF
       ZA = CA(K,IC,SR,IT)
       IF(ZA.GT.CTA) THEN
        CCA = CCA+ONE
        PTA = PTA+EXCP
       ENDIF
       ZB = CB(K,IR,SC,IT)
       IF(ZB.GT.CTB) THEN
        CCB = CCB+ONE
        PTB = PTB+EXCP
       ENDIF
       Y = DS(K,IR,IC)
       IF(Y.GT.CZT) THEN
        CZS = CZS+ONE
        PZS = PZS+EXCP
       ENDIF
       W = ZL(K,CZ)
       IF(W.GT.ZLT) THEN
        CLS = CLS+ONE
        PLS = PLS+EXCP
       ENDIF
   33 CONTINUE
   34 CONTINUE
   35 CONTINUE
   36 CONTINUE
   37 CONTINUE
   38 CONTINUE
   39 CONTINUE
   40 CONTINUE
      PT = DEXP(THISP)
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION RZ(I)
C  FIND I(LOG(I))
      DOUBLE PRECISION  X
      X = 1.0D0*I
      IF(X.GT.1.5D0) THEN
         RZ = X*DLOG(X)
      ELSE
         RZ = 0.0D0
      ENDIF
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION FAC(I)
C  FIND LOG(I!)
      DOUBLE PRECISION  W,PREV(2500)
      INTEGER  I,J,MAX
      DATA  MAX/1/
C  INITIALIZE LOG(0!) = LOG(1) = 0
      FAC = 0.0D0
      PREV(1) = 0.0D0
      IF(I.LE.1) RETURN
C  CHECK TO SEE IF CALCULATION HAS ALREADY BEEN DONE.
      IF(I.LE.MAX) THEN
         FAC = PREV(I)
         RETURN
      ENDIF
      FAC = PREV(MAX)
      DO 10 J = MAX+1,I
         W = 1.0D0*J
         FAC = FAC+DLOG(W)
         PREV(J) = FAC
   10 CONTINUE
      MAX = I
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION CELLFAC(N)
C  OBTAIN SUM OF LOG FACTORIALS FOR 15 CELLS.
      DOUBLE PRECISION  FAC
      INTEGER  I,J,N(3,5)
      CELLFAC = 0.0D0
      DO 20 I = 1,3
         DO 10 J = 1,5
            CELLFAC = CELLFAC+FAC(N(I,J))
   10    CONTINUE
   20 CONTINUE
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION CS(N,IR,IC,IT)
C  OBTAIN CHI-SQUARED STATISTIC
      DOUBLE PRECISION  ZERO,ONE
      INTEGER  N(3,5),IR(3),IC(5),IT,I,J
      ZERO = 0.0D0
      ONE = 1.0D0
      CS = ZERO
      DO 20 I = 1,3
         DO 10 J = 1,5
            CS = CS+(ONE*N(I,J))**2/IR(I)/IC(J)
   10    CONTINUE
   20 CONTINUE
      CS = (CS-ONE)*IT
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION CA(N,IC,SR,IT)
C  OBTAIN TAU-A STATISTIC
      DOUBLE PRECISION  ZERO,ONE,SR
      INTEGER  N(3,5),IC(5),IT,I,J
      ZERO = 0.0D0
      ONE = 1.0D0
      CA = ZERO
      DO 20 I = 1,3
         DO 10 J = 1,5
            CA = CA+(ONE*N(I,J))**2/IC(J)
   10    CONTINUE
   20 CONTINUE
      CA = (CA*IT-SR)/(ONE*IT*IT-SR)
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION CB(N,IR,SC,IT)
C  OBTAIN TAU-B STATISTIC
      DOUBLE PRECISION  ZERO,ONE,SC
      INTEGER  N(3,5),IR(3),IT,I,J
      ZERO = 0.0D0
      ONE = 1.0D0
      CB = ZERO
      DO 20 I = 1,3
         DO 10 J = 1,5
            CB = CB+(ONE*N(I,J))**2/IR(I)
   10    CONTINUE
   20 CONTINUE
      CB = (CB*IT-SC)/(ONE*IT*IT-SC)
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION DS(N,IR,IC)
C  OBTAIN ZELTERMAN STATISTIC
      DOUBLE PRECISION  ZERO,ONE
      INTEGER  N(3,5),IR(3),IC(5),I,J
      ZERO = 0.0D0
      ONE = 1.0D0
      DS = ZERO
      DO 20 I = 1,3
         DO 10 J = 1,5
            DS = DS+ONE*N(I,J)*(N(I,J)-1)/IR(I)/IC(J)
   10    CONTINUE
   20 CONTINUE
      RETURN
      END
C...........................................................................
      DOUBLE PRECISION FUNCTION ZL(N,CZ)
C  OBTAIN LOG LIKELIHOOD RATIO STATISTIC
      DOUBLE PRECISION  ZERO,CZ,RZ
      INTEGER  N(3,5),I,J
      ZERO = 0.0D0
      ZL = ZERO
      DO 20 I = 1,3
         DO 10 J = 1,5
            ZL = ZL+RZ(N(I,J))
   10    CONTINUE
   20 CONTINUE
      ZL = (ZL+CZ)*2
      RETURN
      END
